// coach main js

// Setting up namespaces start
dojo.provide("com.ibm.bpm.coach");
// Setting up nameSpaces end

dojo.registerModulePath("com.ibm.bpm.social.portal.util", "/portal/js");
dojo.require("com.ibm.bpm.social.portal.util.collaboration");

// View superclass starts
com.ibm.bpm.coach.CoachView = function (element, binding, option, readonly) {
	
	// Establish the context object
	this.context = {"element": element, "binding": binding, 
			"options": option, "readonly": readonly, "subview": {},
			"bpm": {"system": com.ibm.bpm.coach.GlobalAssets.system},
			"_parent": this
		       };
	
	// if the view binds with a list and is framework managed, use following templates to handle the dynamic adding (e.g. list has been inserted a new item)
	this._contentBoxTemplate = null;
	this._contentBoxTemplates = {};
	// use this counter to work out the unique id (div id and viewId) for the content in the repeating ContentBox
	this._counter = 0;
	
    //According to the design wiki commented from Aalim, this is no longer offered
	this.context.bind = function () {
		// TODO
	};
	
	this.context.setVisibility = function(newValue){
	
		if (!this.options){
		  this.options = {};
		  this.options._metadata = {};
		  
		}
		this.options._metadata.visibility = newValue;
		var event = {type : "config", property: "_metadata.visibility", oldVal: null, newVal: newValue};
		this._parent.change(event);
		this._parent.view(); //incase the view did not implement the this.change function, which is what Leonard does for his button control.
		//the view author can choose to override this method, to update the view based on the new value of visibility
	};	
	
	//Private function which will be used by the framework to remove the old binding and bind to the new object
	this.context._updateBinding = function (newObject, bindingStr) {
		//cancel the registration
		if (!this.binding){
			return;
		}
		if(!bindingStr) {
			bindingStr = this.element.getAttribute("data-bindingRT");
		}
		if(!bindingStr) {
			bindingStr = this.element.getAttribute("data-binding");
		}
		if (!!this.binding.handle && !!this.binding.handle.cancel){
			this.binding.handle.cancel();
		}
		else if(!!this.binding.handle && !!this.binding.handle.unwatch){
		    this.binding.handle.unwatch();
		}
		//var property = this.binding.property;
		
		//this seems not right. we should always using the bindingStr, not an object
		this.binding = com.ibm.bpm.coach.resolveBinding.call(com.ibm.bpm.coach.GlobalAssets, bindingStr);
		//this.binding = com.ibm.bpm.coach.createBinding(newObject, property);
	};
	
	this.context.trigger = function(callbackfn) {
		var bdEventId = this.element.getAttribute('data-eventid');
		if (!!bdEventId) {
            var bdEventEndStateId = com.ibm.bpm.coach.findEndStateId(bdEventId);
			dojo.publish("com.ibm.bpm.coach.BoundaryEventTriggered", [{
				boundaryEventId:bdEventId, boundaryEventEndStateId:bdEventEndStateId, callback:callbackfn
			 }]);
		}
	};
	
	this.context.createView = function(divNode, index) {
		/* parentViewScope may not be this._parent, for example,
		viewB
			ViewA
				CB_1 (fwk managed)
					ViewC
					CB_2 (view managed, constructed by ViewB)
						ViewD
					CB_3 (fwk managed)
						ViewE
		
		CB_2 is constructed by ViewB, but viewB is not the parentViewScope of the subview (ViewD) in CB_2 			
		*/		 
		var parentViewScope = com.ibm.bpm.coach._findParentViewScope(this._parent, divNode);
		
		if(!!index || index == 0) {
			com.ibm.bpm.coach.adjustIdAndBinding(divNode, parentViewScope, index);
		}
		
		if(com.ibm.bpm.coach.isContentBox(divNode)) {
			var directSubViewScopes = new Array();
			var directViewNodes = com.ibm.bpm.coach.shallowQueryByClass(divNode, 'view');
			dojo.forEach(directViewNodes, function(item, index) {
				directSubViewScopes[index] = com.ibm.bpm.coach.initView.apply(com.ibm.bpm.coach.GlobalAssets, [item, parentViewScope]);
				com.ibm.bpm.coach.callView(directSubViewScopes[index]);
			}, this);
			return directSubViewScopes;
		} else {
			// this divNode is expected to be a view
	    	var viewScope = com.ibm.bpm.coach.initView.apply(com.ibm.bpm.coach.GlobalAssets, [divNode, parentViewScope]);
			com.ibm.bpm.coach.callView(viewScope);
			return viewScope;
		}				
	};
	
	this.context.deleteView = function(divNode) {
	
		if(!divNode) 	return;
		
		// todo: check if this view is view managed. If not, throw exception to let view developer to know it's not allowed to call this method 
		// todo: rebust check to see if the provided viewId is the direct subview
		
		var handleOneView = function(subviewDivNode, ownerView) {
			var viewId = subviewDivNode.getAttribute("data-viewId");
			var divId = subviewDivNode.getAttribute("id");
			var subviews = ownerView.subview[viewId];
			if(!!subviews && !!subviews[divId]) {
				com.ibm.bpm.coach.callUnload(subviews[divId]);
				delete subviews[divId];
			}
			if(subviews.length == 0) {
				// delete ownerView.subview[viewId];
				delete subviews;
			}					
		};
		
		// unload
		if(com.ibm.bpm.coach.isContentBox(divNode)) {
			var directViewNodes = com.ibm.bpm.coach.shallowQueryByClass(divNode, 'view');
			dojo.forEach(directViewNodes, function(item, index) {
				handleOneView(item, this);				
			}, this);
		} else {
			// this divNode is expected to be a view
			handleOneView(divNode, this);				
		}				
	};
	
    this.view = function() {
    	// Super-class lifeCycle view method
    };
    
    // Internal method
    this.preLoad = function () {
    	this.context.viewid = this.context.element.getAttribute('data-viewid');
    }
    
    this.load = function() {
    	// Super-class lifeCycle load method
    };

    this.unload = function() {
    	// Super-class lifeCycle unload method
    	console.debug("unload() is called on viewid " + this.context.viewid + ", Class is " + this.context.element.getAttribute("Class"));
    };

    this.change = function(event) {
		// Default behavior.  Call view method if subview do not have any change logic
    	this.view();
    };	
	
	this.context.refreshView = function(){
		com.ibm.bpm.coach._refreshView(this._parent);
	};
	
    this.collaboration = function(event) {
        dojo.require("dijit.TooltipDialog");
        
		var node = document.getElementById(event.domId);
    	if(event.type == "focus") {
			if(!this._originalBorder && !!node.style.border && node.style.border != "") {
				this._originalBorder = node.style.border;
			}
			node.style.border = "solid " + event.color;
			;				   		
    	} else if(event.type == "blur") {
 			if(!!this._originalBorder) {
				node.style.border = this._originalBorder;
				delete this._originalBorder;
			} else {
   				node.style.border = null;
			}
    		
    	} else if(event.type == "mouseOn") {
			var newContent = "<div class='hoverNode help_hover' style=" + "'color: " + event.color + ";background-color: " + event.bgColor + ";'><label>"  + event.message + "</label></div>";
			if(!hoverBox) {
				hoverBox = com.ibm.bpm.coach._createHoverBox(node, newContent);
			} else {
				hoverBox.attr("content", newContent);
			}
			
			// tooltipDialog.startup(); 
try{
			dijit.popup.open({ 
		        popup: hoverBox, 
		        around: node, 
		        orient: {TR: 'BR', BL: 'BR'}                           
		    }); 
} catch(e) {
console.debug("this.collaboration, mouseOn: " + e);
}
		    /*
	        dojo.connect( 
	            hoverBox, "onBlur", 
	            function(e) { dijit.popup.close(hoverBox);}, 
	            true 
       		 );  */
    		
    	} else if(event.type == "mouseOff") {
			if(!!hoverBox) {
				dijit.popup.close(hoverBox);
			}
    		
    	} else if(event.type == "dataChange") {
    	
//			var newContent = "<div class='bubbleNode' style=" + "'color: " + event.color + ";background-color: " + event.bgColor + ";'><label>"  + event.message + "</label> <a href='#' onClick='alert()'>detail</a> </div>";
			var newContent = "<div class='bubbleNode'><div class='help_hover help_text_bubble'><label>"  + event.message + "</label> <a href='#' onClick=''>Detail</a> </div><div class='help_hover help_point'/></div>";
			if(!bubbleBox) {
				bubbleBox = com.ibm.bpm.coach._createBubbleBox(node, newContent);
			} else {
				bubbleBox.attr("content", newContent);
			}
			
try{
			dijit.popup.open({ 
		        popup: bubbleBox, 
		        around: node, 
		        orient: {TR: 'BR', BL: 'BR'}                           
		    }); 
} catch(e) {
console.debug("this.collaboration, dataChange: " + e);
}
    	}    	
    };
    
	this._buildChildren = function() {
		var currentViewScope = this;
		var currentDiv = currentViewScope.context.element;
	
		// view could hold subviews directly, those direct subview won't repeat even if parent view bend with a list
		var directViewList = com.ibm.bpm.coach.shallowQueryByClass(currentDiv, 'view');
		dojo.forEach(directViewList, function(item, index) {
			com.ibm.bpm.coach.initView.apply(com.ibm.bpm.coach.GlobalAssets, [item, currentViewScope]);
		});			
	
		// find the direct ContentBox under this view
		var immediateContentBoxList = com.ibm.bpm.coach.shallowQueryByClass(currentDiv, 'ContentBox');
		// only care about the framework managed ContentBox. view managed ContentBox will be built by its parent view.
		var fwkManagedContentBoxs = new Array();
		for(var k=0; k<immediateContentBoxList.length; k++) {
			if(immediateContentBoxList[k].getAttribute('data-view-managed') != "true") {
				// Framework managed
				fwkManagedContentBoxs[fwkManagedContentBoxs.length] = immediateContentBoxList[k];
			}
		}
		
		if(fwkManagedContentBoxs.length == 0) {
			// no Framework managed contentBox, so job is done
			return;
		}
		
		if(!!currentViewScope.context.binding && currentViewScope.context.binding.get("value") instanceof com.ibm.bpm.coach.List) {
			// Register listener for List changes
			currentViewScope.context.binding.get("value").bindAll(dojo.hitch(currentViewScope, currentViewScope._listUpdated));
			
			// need to repeat the framework managed contentBox based on the data-binding						
			for(var k=0; k<currentViewScope.context.binding.get("value").length(); k++) {
				for(var i=0; i<fwkManagedContentBoxs.length; i++) {
					var oneDiv = fwkManagedContentBoxs[i].cloneNode(true);
					currentDiv.insertBefore(oneDiv, fwkManagedContentBoxs[0]);
					com.ibm.bpm.coach.adjustIdAndBinding(oneDiv, currentViewScope, k);
					com.ibm.bpm.coach._handleFwkManagedContentBox(currentViewScope, oneDiv);
				}
			}		

/* keep the templates in the dom since we need them to indicate where to insert when dynamically adding from nothing (the list was empty before adding)								
			for(var i=0; i<fwkManagedContentBoxs.length; i++) {
				currentDiv.removeChild(fwkManagedContentBoxs[i]);
			}
*/
			// save the template in the viewScope
			this._contentBoxTemplates = fwkManagedContentBoxs;
			// delete this line once Alice updates hers
			this._contentBoxTemplate = fwkManagedContentBoxs[0];
			
		} else {
			for(var i=0; i<fwkManagedContentBoxs.length; i++) {
				com.ibm.bpm.coach._handleFwkManagedContentBox(currentViewScope, fwkManagedContentBoxs[i]);
			}
		}	
	}

    this._listUpdated = function(changeEvent) {
    	// Super-class lifeCycle change method
		console.debug("list change detected, object: ", changeEvent.object, 
		"removedFrom: ",changeEvent.removedFrom, 
		"insertedInto: ",changeEvent.insertedInto);
		
		// one item has been removed from the list.	
		if(changeEvent.removedFrom >= 0) {
 			var immediateContentBoxList = com.ibm.bpm.coach.shallowQueryByClass(this.context.element, 'ContentBox');
			var numOfContentBoxToDelete = this._contentBoxTemplates.length;
			var startPosition = changeEvent.removedFrom * numOfContentBoxToDelete;
			for(var i=0; i<numOfContentBoxToDelete; i++) {
	    		var contentBoxNode = immediateContentBoxList[startPosition + i];
	    		if(!!contentBoxNode) {
		    		this.context.deleteView(contentBoxNode);
		    		this.context.element.removeChild(contentBoxNode);
	    		}
			}
		}
		
		if(changeEvent.insertedInto >= 0 && !!this._contentBoxTemplates.length != 0) {
			// now we only support to add to the end of the list, figure out the proper position to insert when list supports to add item at any position.
			var insertBeforeNode = this._contentBoxTemplates[0];
			for(var i=0; i<this._contentBoxTemplates.length; i++) {
				var oneDiv = this._contentBoxTemplates[i].cloneNode(true);
				this.context.element.insertBefore(oneDiv, insertBeforeNode);
				this.context.createView(oneDiv, changeEvent.insertedInto);
			}
		}		
    };	
    
    
//##Data Binding start
    this.system = new Object();
    
    this.system.changeObject = dojo.hitch(this, function(property, oldVal, newVal) {
    	//Change handler for Stateful objects, calls the subclass change handler
    	var event = {property: property, oldVal: oldVal, newVal: newVal, removedFrom: null, insertedInto: null};
    	this.change(event);
    });
    

    this.system.changeList = dojo.hitch(this, function(object, removedFrom, insertedInto) {
    	//Change handler for Stateful lists, calls the subclass change handler
    	var newVal;
    	//set newVal to be the added/changed list item in an add or a put
    	if (insertedInto > -1) {
    		newVal = object.listItem;
    	}
    	var event = {property: null, oldVal: null, newVal: newVal, removedFrom: removedFrom, insertedInto: insertedInto};
    	this.change(event);
    });	   
	
    this.system.changeObjectForOptions = dojo.hitch(this, function(object, oldVal, newVal) {
    	//Change handler for Stateful objects, calls the subclass change handler
    	console.debug("system change object");
		
		var optionKey;
		
		
    	var event = {property: optionKey, oldVal: oldVal, newVal: newVal, removedFrom: null, insertedInto: null};
    	this.change(event);
    });
    	
//##Data Binding -end

}

//Internal helper function for refresh all the views in the coach
com.ibm.bpm.coach._refreshViews = function(oldValue, newValue){
	for (var viewId in com.ibm.bpm.coach.GlobalAssets.topLevelViewScope) {
		var viewScope = com.ibm.bpm.coach.GlobalAssets.topLevelViewScope[viewId];
		com.ibm.bpm.coach._refreshView(viewScope, oldValue._objectPath);
	}
};

//Internal helper function for refresh one view and its descendants
//objectPath optional parameter: only used when this function is invoked by a set method of Stateful object
com.ibm.bpm.coach._refreshView = function(viewScope, objectPath){
    
    //if it is array, then binding Str is RT
    var bindingStr = viewScope.context.element.getAttribute("data-bindingRT");
	
	if(!bindingStr) {
		bindingStr = viewScope.context.element.getAttribute("data-binding");
	}
	
	//1. update binding only if it matters
	//bindingStr   local.phoneBook.publishData.prop.prop1
	//objectPath   local.phoneBook.publishData
	var affected = !objectPath || bindingStr.indexOf(objectPath) >= 0 && bindingStr != objectPath;
	var dataBinding;
	var oldDataBinding;
	if (affected){
			oldDataBinding = viewScope.context.binding;
			viewScope.context._updateBinding();	
            //register call back on the new binding object
			var handleToCallBack;
			dataBinding = viewScope.context.binding;
			if (dataBinding != null  && dataBinding.boundObject != null) {
				if (dataBinding.boundObject instanceof com.ibm.bpm.coach.Stateful) {
					//register the callback on the parent(boundObject)
					handleToCallBack = dataBinding.boundObject.registerCallback(dataBinding.property, viewScope.system.changeObject);					
                    //newChangeEvent = {property: binding.property, oldVal: null, newVal: dataBinding.get("value"), removedFrom: null, insertedInto: null};					
				}
				//will never happen due to our design...
				// else {
					// handleToCallBack = dataBinding.boundObject.registerCallback(dataBinding.property, subViewScope.system.changeList);
					// newChangeEvent = {property: binding.property, oldVal: null, newVal: dataBinding.get("value"), removedFrom: null, insertedInto: null};					
				// }
				
			}
			dataBinding.handle = handleToCallBack;					
	}
		
	//2. recursively goes to children
	//if it is array
	if (!!dataBinding && Object.prototype.toString.call( dataBinding.boundObject[oldDataBinding.property] ) === "[object Array]" ){
	    //check if frm = true;
		var immediateContentBoxList = com.ibm.bpm.coach.shallowQueryByClass(viewScope.context.element, 'ContentBox'); 
		if(immediateContentBoxList.length > 0 && immediateContentBoxList[0].getAttribute('data-view-managed') != 'true') {
			com.ibm.bpm.coach._refreshRepeatableView(viewScope, objectPath, oldDataBinding.boundObject[oldDataBinding.property], dataBinding.boundObject[oldDataBinding.property]);			
		}
	}
	else{	
		var subViewScope;
		for (var viewId in viewScope.context.subview) {		    
			var subviews = viewScope.context.subview[viewId];
			for(var divId in subviews) {
				subViewScope = subviews[divId];
				com.ibm.bpm.coach._refreshView(subViewScope, objectPath);
			}
		}
	}
	
	//3. Call this.change
	if( !!dataBinding  && affected){
		event = {property: dataBinding.property, oldVal: oldDataBinding.boundObject[oldDataBinding.property], newVal: dataBinding.boundObject[oldDataBinding.property], removedFrom: null, insertedInto: null};					
		viewScope.change(event);
	}
};

com.ibm.bpm.coach._refreshRepeatableView = function(viewScope, objectPath, oldValue, newValue){
		
    	var subviewScope;		
		var subViewbinding;
		var newChangeEvent;
		var tempViewDataBinding 
		var handleToCallBack;
		
		var immediateContentBoxList = com.ibm.bpm.coach.shallowQueryByClass(viewScope.context.element, 'ContentBox');
		if (oldValue.length == newValue.length){	
			for (var viewId in viewScope.context.subview) {		    
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					subViewScope = subviews[divId];
					com.ibm.bpm.coach._refreshView(subViewScope, objectPath);
				}
			}			
		}
		else{
			for (var i=0; i < immediateContentBoxList.length; i++){
				var contentBoxNode = immediateContentBoxList[i];
				if(!!contentBoxNode) {
					viewScope.context.deleteView(contentBoxNode);
					viewScope.context.element.removeChild(contentBoxNode);
				}
			}
		
			for (var i=0; i < newValue.length; i++){
				if(!!viewScope._contentBoxTemplate) {
					var oneDiv = viewScope._contentBoxTemplate.cloneNode(true);
					viewScope.context.element.appendChild(oneDiv);
					viewScope.context.createView(oneDiv, i);
				}
			}
		}
    };

com.ibm.bpm.coach.initCustomHTML = function() {
	//find all customHTML
	var customHTML = dojo.query('#mainBody .customHTML');
	dojo.forEach(customHTML, function(element, index) {
	
		//resolve binding object
		var bindingStr = element.getAttribute("data-binding");
		if (!!bindingStr){
			var result = com.ibm.bpm.coach.resolveBinding.call(this, bindingStr);
		
			//register callback functionthe callback function is used to replace the div content with the new value
			var callBackFunction = function(property, oldValue, newValue){
				element.innerHTML = newValue;		
			};
		
			//will use this information to update the binding information when parent object is changed
			result.handle = result.boundObject.registerCallback(result.property, callBackFunction);
		
			//later will do result.handle.cancel
			//and register the functoin on the new object
		}
	}, this);		
		
};


com.ibm.bpm.coach.initViews = function() {
	// find out all top level views
	var topLevelViewList = dojo.query('#mainBody .topLevel');
	dojo.forEach(topLevelViewList, function(item, index) {		
		this.topLevelViewScope[item.getAttribute("data-viewid")] = com.ibm.bpm.coach.initView.apply(this, [item, null]);
	}, this);
};

/**
 * create object for option value that refers to business object
 * also create a copy in GlobalAssets.bindingObjectForOptions, 
 * but do not create this for those pointing to object in array with un adjusted array index
 * 
 */
com.ibm.bpm.coach.resolveOptionValue = function(configObjId, property, optionValueStr, updateFunction, indexAdjusted){
	
	var optionValue = {};
	optionValue.bindingStr = optionValueStr;
	optionValue.updateFunction = updateFunction;
	optionValue.property = property;
	//if object is not in array, or the index is present
	if (optionValueStr.indexOf("[]") <0 || !!indexAdjusted) {
	    //resolve the value here. note that later if want to do update by using bo to binding mapping tree, 
		//then do not resolve here, instead resolve it when initView so that we can access the viewScope,
		//if we do it in initView
		   //get the config object id from dom node, get the bindingObjectForOptions[configId]
		   //call com.ibm.bpm.coach.resolveBinding.call to resolve the object
		   //set the returned value to the real option object by the property...and 
		   //set this.option to the option object in global
		   //register function
		//if we really use bo to binding mapping tree, then GlobalAssets.bindingObjectForOptions will become temp var which can be deleted after initView
		//otherwise, GlobalAssets.bindingObjectForOptions is used to for update the bo to config binding when a bo is set
		/*var binding = com.ibm.bpm.coach.resolveBinding.call(com.ibm.bpm.coach.GlobalAssets, optionValueStr);	
		binding.bindingStr = optionValueStr;
		binding.updateFunction = updateFunction;*/
		
		//store the information in globalAssets.bindingObjectForOptions so that later can use this information
		if (! com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions){
			com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions = {};   
		}	
		if (!com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions[configObjId] ){	
			com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions[configObjId]={};
		}
	
		com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions[configObjId][property] = optionValue;
		//return binding.get("value");
		return optionValue;
	}
	else{
		//var optionValue =  {};
		//optionValue.bindingStr = optionValueStr;
		//optionValue.updateFunction = updateFunction;
		return optionValue;
	}
	
}

com.ibm.bpm.coach.resolveOptionValueOrig = function(viewId, property, optionValueStr){
    
	var result = com.ibm.bpm.coach.resolveBinding.call(com.ibm.bpm.coach.GlobalAssets, optionValueStr);	
	//TODO: Not good to create variables just for storing something temp..
	if (! com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions){
		com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions = {};   
	}	
    if (!com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions[viewId] ){	
		com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions[viewId]={};
	}
	
     com.ibm.bpm.coach.GlobalAssets.bindingObjectForOptions[viewId][property] = result;	
	return result.get("value");
}

 //get the config object id from dom node, get the bindingObjectForOptions[configId]
		   //call com.ibm.bpm.coach.resolveBinding.call to resolve the object
		   //set the returned value to the real option object by the property...and 
		   //set this.option to the option object in global
		   //register function
com.ibm.bpm.coach.registerOptionsBindingCallBack = function(options, completeOptions) {
    var optionValue;
	var scopeObject = this;
	var bindingStr;
	var binding;
	for(var optionKey in options) {
		 optionValue = options[optionKey];	
         //resolve the object
		binding = com.ibm.bpm.coach.resolveBinding.call(com.ibm.bpm.coach.GlobalAssets, optionValue.bindingStr);
		
		//set the value
		var metaDataIndex = optionValue.property.indexOf("_metadata");
		if (metaDataIndex < 0){
		    completeOptions[optionKey] = binding.get("value");
		}
		else{
			completeOptions[optionKey.substring(0, 9)][	optionKey.substring(10)] = binding.get("value");
		}
		
		//register callback function
		if (!!binding.boundObject && binding.boundObject instanceof com.ibm.bpm.coach.Stateful) {
		 
			var callBackFunction = com.ibm.bpm.coach._createDataConfigCallBack(scopeObject, optionKey);
			binding.boundObject.registerCallback(binding.property, callBackFunction);
			
			//TODO: do we want to store a handle to the binding object so that later we can cancel the callback?
			//not.. because if the object is deleted, then the callback is also gone..
		} else if(!!optionValue) { //list case
				//binding.boundObject.registerCallback(optionValue.property, viewScope.system.changeList);
		}	
	}	
}

com.ibm.bpm.coach._createDataConfigCallBack = function(scopeObject, optionKey){
       
	   //this function will be used for the callback for stateful.watch
       function optionsBindingcallBack (property, oldVal, newVal){	   
			var metaDataIndex = optionKey.indexOf("_metadata");
			if (metaDataIndex < 0){
				scopeObject.context.options[optionKey] = newVal;
			}
			else{
				scopeObject.context.options[optionKey.substring(0, 9)][	optionKey.substring(10)] = newVal;
			}
			var event = { type : "config", property:optionKey, oldVal: oldVal, newVal: newVal, removedFrom: null, insertedInto: null};
				
			scopeObject.change(event);
	   }
       return optionsBindingcallBack;
}

com.ibm.bpm.coach.registerOptionsBindingCallBackOrig = function(options) {
    var optionValue;
	var scopeObject = this;
	for(var optionKey in options) {
		 optionValue = options[optionKey];				
		if (!!optionValue.binding.boundObject && optionValue.binding.boundObject instanceof com.ibm.bpm.coach.Stateful) {
		    
			//var callBackFunction = com.ibm.bpm.coach.resolveOptionsBindingCallBack.call(this, optionKey);
			var callBackFunction = function(property, oldVal, newVal){	   
				var event = { type : "config", property:optionKey, oldVal: oldVal, newVal: newVal, removedFrom: null, insertedInto: null};
				//Note that the value of the optionKey is not changed here,
				//so in the change method, what user get from this.context.options[optionKey] is the old value
				scopeObject.change(event);
			}
			optionValue.binding.boundObject.registerCallback(optionValue.property, callBackFunction);
		} else if(!!optionValue) { //list case
				//optionValue.binding.boundObject.registerCallback(optionValue.property, viewScope.system.changeList);
		}	
	}	
}

//##Data Binding start
com.ibm.bpm.coach.resolveBinding = function(bindingString) {
	//Resolves a binding string in the form of BO.property

	if (bindingString == null || bindingString == "") return;

	var result;
	var refArray = bindingString.split(".");
	if (refArray[0] == "local") {
		if (!(this["local"] instanceof com.ibm.bpm.coach.Stateful)) {
			this["local"] = new com.ibm.bpm.coach.Stateful(this["local"], "local");
		}
		var parentObject = this["local"];
		var boundObject = this["local"];
		var prop = null;
		for (var i=1; i<refArray.length; i++) {
			parentObject = boundObject;
			if (refArray[i].indexOf("[]") != -1) {
				refArray[i] = refArray[i].substring(0, refArray[i].indexOf("[]"));
			}
			var subMatch = refArray[i].match(/\[.*\]/);
			var index = null;
			if (subMatch != null) {
				index = subMatch[0].substring(1, subMatch[0].length-1);
				refArray[i] = refArray[i].substring(0, subMatch.index);
			}
			prop = refArray[i];
			if(index == null) {
				boundObject = boundObject.get(refArray[i]);
			} else {
				parentObject = boundObject.get(refArray[i]);
				boundObject = boundObject.get(refArray[i]).get(index);
				prop = index;
			}
		}
		
//		//if the boundObject is stateful, then we need to register the callback on itself, rather than the parent
//		if ( (boundObject instanceof com.ibm.bpm.coach.Stateful) || (boundObject instanceof com.ibm.bpm.coach.List)) {
//			parentObject = boundObject;
//			prop = null;
//		}
		
		var get = function (propName) {return (propName=="value")?this.boundObject.get(this.property):null};
		var set = function(propName, val) {(propName=="value")?this.boundObject.set(this.property, val):0};
		result = {boundObject: parentObject, property: prop, get: get, set: set};
	} else {
		//handle system, resource, epv
		var get = function (propName) {return (propName=="value")?this.value:null};
		bindingString = bindingString.substring(refArray[0].length);
		var value = eval("this['"+refArray[0]+"']" + bindingString);
		result = {value: value, get: get};
	}
	return result;
}

//create binding to the new object and register callback to the new object
com.ibm.bpm.coach.createBinding = function(parentObject, property) {
	var get = function (propName) {return (propName=="value")?this.boundObject.get(this.property):null};
	var set = function(propName, val) {(propName=="value")?this.boundObject.set(this.property, val):0};
	var result = {boundObject: parentObject, property: property, get: get, set: set};
	return result;
}

//##Data Binding end

com.ibm.bpm.coach.initView = function(viewElement, parentViewScope) {
	var tempViewDivId = viewElement.getAttribute("id");
	var tempViewId = viewElement.getAttribute("data-viewid");
	var tempViewType = viewElement.getAttribute("data-type");
	var tempViewDataBinding = com.ibm.bpm.coach.rtDataBinding._get(viewElement);
	var tempViewCfgBinding = viewElement.getAttribute("data-config");
	
	// delete the runtime dataBinding info since we have already used it, no need to keep any more.
	com.ibm.bpm.coach.rtDataBinding._delete(viewElement);	
	
//##Data Binding start
	//data
	var dataBinding = com.ibm.bpm.coach.resolveBinding.call(this, tempViewDataBinding);

	//config
	//var cfgBinding = com.ibm.bpm.coach.resolveBinding.call(this, tempViewCfgBinding);
	var cfgBinding = this.options[tempViewCfgBinding]; //com.ibm.bpm.coach.resolveConfig.call(this, tempViewCfgBinding);
	
	var viewScopeStr = 'return new ' + tempViewType + '(viewElement, dataBinding, cfgBinding)';
	var createFunction = new Function("viewElement", "dataBinding", "cfgBinding", viewScopeStr);
	var viewScope = createFunction(viewElement, dataBinding, cfgBinding);

	//data
	if (dataBinding != null  && dataBinding.boundObject != null) {
		if (dataBinding.boundObject instanceof com.ibm.bpm.coach.Stateful) {
		    //register the callback on the parent(boundObject)
			dataBinding.boundObject.registerCallback(dataBinding.property, viewScope.system.changeObject);
		    //ChangeNote: if bindingObject(selected at designer) is list
			//Note2: we should not do this. View should register theirself
			//if (dataBinding.get("value") instanceof com.ibm.bpm.coach.List){
			//   dataBinding.get("value").registerCallback(dataBinding.property, viewScope.system.changeList);
			//}
			
		} else {
			dataBinding.boundObject.registerCallback(dataBinding.property, viewScope.system.changeList);
		}
	}

	//config
	if ((!!this.bindingObjectForOptions) && (!!this.bindingObjectForOptions[tempViewCfgBinding])) {
		com.ibm.bpm.coach.registerOptionsBindingCallBack.call(viewScope, this.bindingObjectForOptions[tempViewCfgBinding], cfgBinding);
	}
	 
//##Data Binding end
	
	// TODO: shall we populate context.subview[] here? When we do it here, the subviews of view-managed view show in the context.subview[] too.
	if (!!parentViewScope) {
		// Set the scope to the parent
		var viewId = viewElement.getAttribute("data-viewid");
		var divId = viewElement.getAttribute("id");
		if(!parentViewScope.context.subview[viewId]) {
			parentViewScope.context.subview[viewId] = {};
		}
		parentViewScope.context.subview[viewId][divId] = viewScope;
	}
	
	// continue to build the children views
	viewScope._buildChildren(this);

	viewScope.preLoad();
	
	// Call the view onLoad lifecycle method
	viewScope.load();

	return viewScope;
};

com.ibm.bpm.coach.prepareView = function() {
	for (var viewId in this.topLevelViewScope) {
		   var viewScope = this.topLevelViewScope[viewId];
		   com.ibm.bpm.coach.callView(viewScope);
	}
};

com.ibm.bpm.coach.callView = function(viewScope) {
	// Call nested view view first
	if (!!viewScope.context.subview) {
		for (var viewId in viewScope.context.subview) {
			var subviews = viewScope.context.subview[viewId];
			for(var divId in subviews) {
				var subViewScope = subviews[divId];
				com.ibm.bpm.coach.callView(subViewScope);
			}
		}		
	}
	
    // Call the view view lifecycle method
	viewScope.view();
    
    // Display the content
	// viewScope.context.element.style.display = 'block';
	var viewClass = viewScope.context.element.getAttribute('class');
	if(viewClass.indexOf("view_show") == -1) {
		viewScope.context.element.setAttribute('class', viewClass + " view_show");
	}
};

//The function, adjustIdAndBinding, is called from 2 different places
//one is from createView. The node( divNode )is cloend and passed into createView
//another is from _buildChildren, the node, divNode, is cloned in _buildChildren and passed into this function
//I need to clone the data config object and ajust the binding str based on the index.
//If clone the data before this function is called, then I need to do recursive traveral and clone 
//Rather, do it here, so that it use the same traveral code
com.ibm.bpm.coach.adjustIdAndBinding = function(divNode, parentViewScope, bindingIndex) {
	if(divNode.tagName != "DIV")
		return null;
	
	// var idSuffix = "_" + bindingIndex;  // bindingIndex is not enough as idSuffix due to the dynamic delete/add
	var idSuffix = "_" + parentViewScope._counter;
	parentViewScope._counter++;
	
	var parentDataBinding = com.ibm.bpm.coach.rtDataBinding._get(parentViewScope.context.element);

	var resolvedVersion = parentDataBinding.replace("[]", "[" + bindingIndex + "]");
	
	var contentBoxList = dojo.query(' .ContentBox', divNode); // get all the contentBox inside this divNode
	var viewList =  dojo.query(' .view', divNode); // get all the view inside this divNode
	
	if(com.ibm.bpm.coach.isContentBox(divNode)) {
		// this divNode is a contentBox
		contentBoxList[contentBoxList.length] = divNode;
	} else {
		// this divNode is expected to be a view
		viewList[viewList.length] = divNode;
	}
		
	// handle ContentBox
	dojo.forEach(contentBoxList, function(item, index) {		
		item.setAttribute('id', item.id + idSuffix);
	}, parentViewScope);
	
	// handle view
	dojo.forEach(viewList, function(item, index) {
		item.setAttribute('id', item.id + idSuffix);
		
		var viewDataBinding = com.ibm.bpm.coach.rtDataBinding._get(item);			
		com.ibm.bpm.coach.rtDataBinding._set(item, viewDataBinding.replace(parentDataBinding, resolvedVersion));
		
		com.ibm.bpm.coach.cloneAndAdjustDataConfig(item, parentDataBinding, resolvedVersion);		
	}, parentViewScope);

};

com.ibm.bpm.coach.cloneAndAdjustDataConfig = function(viewElement, parentBinding, resolvedVersion ){
	
	var tempViewCfgBinding = viewElement.getAttribute("data-config");
	if(!tempViewCfgBinding){
		return;
	}
    var configObj = com.ibm.bpm.coach.GlobalAssets.options[tempViewCfgBinding];
	var clonedObjId = "config_" + viewElement.id;
	
	//clone the object and store it under GlobalAssets.options
	var clonedObj = {};
	var configValue;
	var bindingStr;
	for(var configKey in configObj) {
		configValue = configObj[configKey];
	    
		if (configKey == "_metadata"){
			//for meta data
			clonedObj[configKey] = {};
			for(var metaDataKey in configValue) {
			    if (!!configValue[metaDataKey].bindingStr){	
				    bindingStr = configValue[metaDataKey].bindingStr;
                    if (configValue[metaDataKey].bindingStr.indexOf("[]") > -1){	
						//adjust index				
						bindingStr = configValue[metaDataKey].bindingStr.replace(parentBinding, resolvedVersion);
					}
					clonedObj[configKey][metaDataKey] = com.ibm.bpm.coach.resolveOptionValue(clonedObjId, configKey + "." + metaDataKey, bindingStr, configValue[metaDataKey].updateFunction, true);
				}
				//simply copy over the value
				//case one: static value
				//case 2: none array bo: resolveOptionValue function is already called in initConfig
				else{
					clonedObj[configKey][metaDataKey] = configValue[metaDataKey];
				}
			}
		}
		//check if it points to an object
		else if (!!configValue.bindingStr){	
		    bindingStr = configValue.bindingStr;
			if(configValue.bindingStr.indexOf("[]") > -1){			
				//adjust index		
				bindingStr = configValue.bindingStr.replace(parentBinding, resolvedVersion);
			}
			clonedObj[configKey] = com.ibm.bpm.coach.resolveOptionValue(clonedObjId, configKey, bindingStr, configValue.updateFunction, true);
		}
		//simply copy over the value		
		else{
			clonedObj[configKey] = configObj[configKey];
		}
	}	 
	
	
	com.ibm.bpm.coach.GlobalAssets.options[clonedObjId] = clonedObj;
	
	//update the view    
	viewElement.setAttribute('data-config', clonedObjId);

	//go through the cloned object, if there are options that points to bo, then replace the array index
	//if not, then resolve the bo, if we resolve here, then we do not need to do it in init view
	//But if resolve it here, what about the one that are not repeated? it will not be resolved...
	//what if we resove the non array one at the beginning, resoveOptionValue, so that the value is set, which is a string
	//but at there, we do not know if the view using that config obj will be repeated or not
	//if it will be repeated, the whole config object should be cloned, then we should know the oringinal object, to set up callback
	
	//so the conclusion here is:
	//1: in initconfig: only produce the config with binding str
	//2: in adjust binding for repeating case: clone the object, ajust array index
	//3: in init view
	//get the option object
	//resolve the binding object if points to bo
	//register call back 
	//set this.option to the option object, which is the one cloned and stored in GlobalAssets
	//4: for update complex bo case
	//store/perserve the config binding string, and update the config binding when a bo is changed...
	
	//finally
	//1. initConfig: produce the stored object with bindingStr only for bo which is not array
	//2. in adjust binding for repeating case: clone the object
	     // if not array, store it with bindingStr 
		 // if array, adjust index, store it with bindingStr
    //3. in init view
	//get the option object and set it to this.options , which is the one cloned and stored in GlobalAssets
	//go through the stored one, resolve the object and register callback
	//4: for update complex bo case: to think
	     //go throug the stored one, and update it based on binding string
	
}

// rtDataBinding provides 3 helper method for resolving List related data-binding during runtime.
com.ibm.bpm.coach.rtDataBinding = function() {};
com.ibm.bpm.coach.rtDataBinding._get = function(viewElement) {
	var rtBinding = viewElement.getAttribute("data-bindingRT");
	if(!rtBinding) {
		return viewElement.getAttribute("data-binding");
	} else {
		return rtBinding;
	}
}
com.ibm.bpm.coach.rtDataBinding._set = function(viewElement, bindingValue) {
	viewElement.setAttribute('data-bindingRT', bindingValue);
}
com.ibm.bpm.coach.rtDataBinding._delete = function(viewElement) {	
	// viewElement.removeAttribute("data-bindingRT");
}

com.ibm.bpm.coach.isContentBox = function(node) {
	if(!!node) {
		var classAttribute = node.getAttribute("Class");
		if(!!classAttribute) {
			return classAttribute.indexOf("ContentBox") != -1;
		}
	}
	return false;
};

// only expect input className to be either "ContentBox" or "view"
// once find a node whose class satisfies, won't go deeper on that node. So the nodes in the return array don't have direct or indirect containership.
com.ibm.bpm.coach.shallowQueryByClass = function(node, className, immediateChildOnly) {
	if(immediateChildOnly == true) {
		return dojo.query('> .' + className, node);
	} else {
		var deepQueryResults = dojo.query(' .' + className, node);
		var shallowQueryResults = new Array();
		dojo.forEach(deepQueryResults, function(oneNode) {
			var parentNode = oneNode.parentNode;
			while(parentNode != node) {
				var classAttribute = parentNode.getAttribute("Class");
				if(!!classAttribute && (classAttribute.indexOf("ContentBox") != -1 || classAttribute.indexOf("view") != -1)) {
					// if parentNode is view or ContentBox, the node is not what we want
					return;
				}
				parentNode = parentNode.parentNode;
			}
			shallowQueryResults[shallowQueryResults.length] = oneNode;
		});
		return shallowQueryResults;
	}
};

com.ibm.bpm.coach._handleFwkManagedContentBox = function(parentView, contentBoxDiv) {
	var immediateViewList = com.ibm.bpm.coach.shallowQueryByClass(contentBoxDiv, 'view');
	dojo.forEach(immediateViewList, function(item, index) {
		com.ibm.bpm.coach.initView.apply(com.ibm.bpm.coach.GlobalAssets, [item, parentView]);
	});
	
	// find the direct ContentBox under this contentBox.
	var immediateContentBoxList = com.ibm.bpm.coach.shallowQueryByClass(contentBoxDiv, 'ContentBox');
	// only care about the framework managed ContentBox. view managed ContentBox will be built by its owner view.
	for(var k=0; k<immediateContentBoxList.length; k++) {
		if(immediateContentBoxList[k].getAttribute('data-view-managed') != "true") {
			// Framework managed
			com.ibm.bpm.coach._handleFwkManagedContentBox(parentView, immediateContentBoxList[k]);
		}
	}
}

// find the direct parent view for the viewDivNode
com.ibm.bpm.coach._findParentViewScope = function(rootViewScope, viewDivNode) {
	// we intend not to have robust checking on 2 while loop and 1 for loop. If exception occurs, we will find some scenarios that we haven't thought of. 
	
	var parentViewScopeNode = null;
	var rootDivNode = rootViewScope.context.element;
	var parentNode = viewDivNode.parentNode;
	while(!parentViewScopeNode) {
		if(parentNode == rootDivNode) {
			// well, rootViewScope is the direct owner
			return rootViewScope;
		}
		var classAttribute = parentNode.getAttribute("Class");
		if(!!classAttribute && classAttribute.indexOf("view") != -1) {
			parentViewScopeNode = parentNode;
			break;
		}
		parentNode = parentNode.parentNode;
	}
	
	// try to find the ViewScope object of parentViewScopeNode with the help of rootViewScope		
	var nodeChain = new Array();
	nodeChain[0] = parentViewScopeNode;
	parentNode = parentViewScopeNode.parentNode;
	while(parentNode != rootDivNode) {
		nodeChain[nodeChain.length] = parentNode;
		parentNode = parentNode.parentNode;			
	}
	
	return com.ibm.bpm.coach._findViewScopeByChain(rootViewScope, nodeChain);
}

com.ibm.bpm.coach._findViewScopeByChain = function(rootViewScope, nodeChain) {
	if(!rootViewScope || !nodeChain) {
		return null;
	}
	
	var tempViewScope = rootViewScope;
	for(var k=nodeChain.length -1; k>=0; k--) {
		var viewId = nodeChain[k].getAttribute("data-viewId");
		var divId = nodeChain[k].getAttribute("id");
		if(!!viewId && !!divId) {
			if(!tempViewScope.context.subview[viewId]) {
				console.debug("*****ERROR: Developer, viewId [" + viewId + "] may not the right one");
				return null;
			} else if(!tempViewScope.context.subview[viewId][divId]) {
				console.debug("*****ERROR: Developer, divId [" + divId + "] may not the right one");
				return null;
			} 
			tempViewScope = tempViewScope.context.subview[viewId][divId];
			if(!tempViewScope) {
				return null;
			}
		}
	}
	return tempViewScope;
}
    
com.ibm.bpm.coach._findViewScope = function(domId) {

	var domNode = document.getElementById(domId);
	
	var nodeChain = new Array();
	var classAttribute;
	while(!!domNode && domNode.tagName != "BODY") {
		classAttribute = domNode.getAttribute("class");
		if(!!classAttribute && classAttribute.indexOf("topLevel") != -1) {
			// now domNode is the topLevel view
			var topLevelViewId = domNode.getAttribute("data-viewId");
			var topViewScope = com.ibm.bpm.coach.GlobalAssets.topLevelViewScope[topLevelViewId];			
			return com.ibm.bpm.coach._findViewScopeByChain(topViewScope, nodeChain);
		}
		nodeChain[nodeChain.length] = domNode;
		domNode = domNode.parentNode;
	}
	
	return null;	
}

com.ibm.bpm.coach._createHoverBox = function(node, aContent) {
    var tooltipDialog = new dijit.TooltipDialog({
        content: aContent 
    });
try{
	dijit.popup.open({ 
        popup: tooltipDialog, 
        around: node, 
        orient: {TR: 'BR', BL: 'BR'}                           
    }); 
    dijit.popup.close(tooltipDialog);
} catch(e) {
console.debug("com.ibm.bpm.coach._createHoverBox: " + e);
}
    
    var dijitTooltipDialogPopups = dojo.query('.dijitTooltipDialogPopup');
    var hoverDiv;
    if(dijitTooltipDialogPopups.length == 1) {
    	hoverDiv = dijitTooltipDialogPopups[0];
    } else {
    	for(var k=0; k<dijitTooltipDialogPopups.length; k++) {
    		if(dojo.query('.hoverNode', dijitTooltipDialogPopups[k]).length > 0) {
    			hoverDiv = dijitTooltipDialogPopups[k];
    			break;
    		};
    	};
    }
    
    dojo.query('.dijitTooltipContainer', hoverDiv).addClass('hoverArea');
    dojo.query('.dijitTooltipFocusNode', hoverDiv).addClass('hoverArea');
    dojo.query('.dijitTooltipAbove', hoverDiv).addClass('hoverArea');
    dojo.query('.dijitTooltipConnector', hoverDiv).addClass('hoverArea');
    
    return tooltipDialog;
}
                             
com.ibm.bpm.coach._createBubbleBox = function(node, aContent) {
    var tooltipDialog = new dijit.TooltipDialog({
        content: aContent 
    });
try{
	dijit.popup.open({ 
        popup: tooltipDialog, 
        around: node, 
        orient: {TR: 'BR', BL: 'BR'}                           
    }); 
    dijit.popup.close(tooltipDialog);
} catch(e) {
console.debug("com.ibm.bpm.coach._createBubbleBox: " + e);
}
    
    var dijitTooltipDialogPopups = dojo.query('.dijitTooltipDialogPopup');
    var bubbleDiv;
    if(dijitTooltipDialogPopups.length == 1) {
    	bubbleDiv = dijitTooltipDialogPopups[0];
    } else {
    	for(var k=0; k<dijitTooltipDialogPopups.length; k++) {
    		if(dojo.query('.bubbleNode', dijitTooltipDialogPopups[k]).length > 0) {
    			bubbleDiv = dijitTooltipDialogPopups[k];
    			break;
    		};
    	};
    }
    
    dojo.query('.dijitTooltipContainer', bubbleDiv).addClass('bubbleArea');
    dojo.query('.dijitTooltipFocusNode', bubbleDiv).addClass('bubbleArea');
    dojo.query('.dijitTooltipAbove', bubbleDiv).addClass('bubbleArea');
    dojo.query('.dijitTooltipConnector', bubbleDiv).addClass('bubbleArea');
    
    return tooltipDialog;
}
                             


	
com.ibm.bpm.coach.updateViews = function (oldValue, newValue, property){
	;
	var path = oldValue._objectPath;
	if (!path){
	  throw new Error("should not happen: object does not have path");
	}
	var treeNode;
	var event;
	var pathArray = path.split(".");
	if (pathArray[0] == "local"){
	    treeNode = com.ibm.bpm.coach.GlobalAssets.boToViewMapping.local;
		for (var i=1; i<pathArray.length; i++) {
			treeNode = treeNode[pathArray[i]];
		}
	}
	
    if (!!treeNode){	
		com.ibm.bpm.coach.updateViewsForBo(treeNode, newValue);
	}
	
};

com.ibm.bpm.coach.updateViewsForBo = function (mapTreeNode, boTreeNode){
		
		var views = mapTreeNode._views;
		var viewsPerProperty;
		var viewScope;
		for(var property in views) {
			viewsPerProperty = views[property];				
			if (!!viewsPerProperty) {
				for(var i=0; i<viewsPerProperty.length; i++) {
					viewScope = viewsPerProperty[i];
					viewScope.context._updateBinding();	
                    //register call back on the new binding object
			var dataBinding = viewScope.context.binding;
			if (dataBinding != null  && dataBinding.boundObject != null) {
				if (dataBinding.boundObject instanceof com.ibm.bpm.coach.Stateful) {
					//register the callback on the parent(boundObject)
					handleToCallBack = dataBinding.boundObject.registerCallback(dataBinding.property, viewScope.system.changeObject);					
                    //newChangeEvent = {property: binding.property, oldVal: null, newVal: dataBinding.get("value"), removedFrom: null, insertedInto: null};					
				}
				//will never happen due to our design...
				// else {
					// handleToCallBack = dataBinding.boundObject.registerCallback(dataBinding.property, subViewScope.system.changeList);
					// newChangeEvent = {property: binding.property, oldVal: null, newVal: dataBinding.get("value"), removedFrom: null, insertedInto: null};					
				// }
				
			}
			dataBinding.handle = handleToCallBack;					
				}	  
			}	
		}	 
		
		for(var property in mapTreeNode) {
			if (property != "_views"){
				com.ibm.bpm.coach.updateViewsForBo(mapTreeNode[property], boTreeNode[property]);			
			}
		}
		
		//instead of doing this, we can also do a set of the bo
		
		
		for(var property in views) {
			viewsPerProperty = views[property];	
            			
			if (!!viewsPerProperty) {
				for(var i=0; i<viewsPerProperty.length; i++) {
					viewScope = viewsPerProperty[i];
					
					event = {property: property, oldVal: null, newVal: boTreeNode[property], removedFrom: null, insertedInto: null};					
					//viewScope.change(event);
					viewScope.context.binding.boundObject.set(property, boTreeNode[property]);
					break;
				}	  
			}	
		}	 

};
 
 
 
 
 
 
//##DataBinding start
	dojo.require("dojo.Stateful");
	dojo.require("dojo.store.Memory");
	dojo.require("dojo.store.Observable");

	//Extends dojo.Stateful.  Replaces the "get" method to handle wrapping embedded objects and lists to also make them
	//watchable.  It also caches the watchable objects so that subsequent get operations do not need to wrap new objects.
	dojo.declare("com.ibm.bpm.coach.Stateful", dojo.Stateful, {
		//cache of all the child objects and lists
		childrenCache: null,
		_systemCallbackHandle: null,

		//constructor
		constructor: function(baseObject, path) {
			this.inherited(arguments);
			this.childrenCache = {};
			this._objectPath = path;
			this._systemCallbackHandle = null;
			this.enableSystemNotifications();
		},

		disableSystemNotifications: function () {
			this._systemCallbackHandle.unwatch()
		},
		
		enableSystemNotifications: function () {
			//set system call back, for notifications to controller
			this._systemCallbackHandle = this.registerCallback(null, dojo.hitch(this, function(prop, oldVal, newVal) {
				    						com.ibm.bpm.coach.controller.systemChangeHandler(this._objectPath, prop, newVal);
				    					})
			);			
		},
		
		//overide the super's get to add caching and wrapping of embeded objects and lists
		get: function (name) {
			var result;
			
			var pathArray = name.split(".");
			if (pathArray.length > 1) {
				result = this.resolvePath(pathArray);
			} else {
				//first check the cache
				var property = this.childrenCache[name];
	
				if (typeof(property) == "undefined") {
					var obj = this.inherited(arguments);
					
					//check if it is a simple type
					var dataTypes = ["number", "string", "boolean"];
					dojo.forEach(dataTypes, function(dataType) {
						if (typeof obj == dataType) {
							result = obj;
						}
					});
	
					if (typeof result == "undefined") {
						//handle a list
						if (Object.prototype.toString.call( obj ) === "[object Array]") {
							result = new com.ibm.bpm.coach.List(obj,  this._objectPath + "." + name);
							//add result to the cache
							this.childrenCache[name] = result;
			        		} else if (typeof obj == "object") {
			        			//handle an object
							if (!obj.prototype) { // check that this is a literal
								result = obj;
							} else if (obj.constructor) {
								result = new obj.constructor();
							}
							result = new com.ibm.bpm.coach.Stateful(result, this._objectPath + "." + name);
		
							//add result to the cache
							this.childrenCache[name] = result;
						}
					} else {
						//handle a simple type
						result = obj;
					}
				} else {
					//cache hit
					result = property;
				}
			}
			return result;
		}, //function get
		
		//overide the super's set to add path
		set: function (name, value, disableSystemNotification) {

			var pathArray = name.split(".");
			if (pathArray.length > 1) {
				name = pathArray.pop();
				var object = this.resolvePath(pathArray);
				if (object instanceof com.ibm.bpm.coach.Stateful) {
					object.set(name, value);
				}
			} else {
			    var oldPlainValue = this[name];
			    var oldValue = this.childrenCache[name];
				//We do not cache value for leaf
				if (!!oldValue){
				    delete this.childrenCache[name];
					
					this[name] = value;
					com.ibm.bpm.coach._refreshViews(oldValue, value, name);
					this[name] = oldPlainValue;						
						
				}
				if (disableSystemNotification && disableSystemNotification == true) {
					this.disableSystemNotifications();
					this.inherited(arguments);
					this.enableSystemNotifications();
				} else {
					this.inherited(arguments);
				}
			}
		},

		//walks an array of strings that reprsent an object graph of Stateful objects
		//returns the leaf or null if the path is not valid
		resolvePath: function(objectArray) {
			var result = null;
			if (objectArray) {
				result = this.resolveIndexedProperty(objectArray[0]);					
				
				for (var i=1; i<objectArray.length; i++) {
					if (result instanceof com.ibm.bpm.coach.Stateful) {
						result = result.resolveIndexedProperty(objectArray[i]);
					} else {
						result = null;
						break;
					}
				}
			}
			return result;
		},
		
		resolveIndexedProperty: function (property) {
			var result = null
			var openIndex = property.indexOf("["); 
			if (openIndex != -1) {
				//refers to an array
				var closeIndex = property.indexOf("]");
				result = this.get(property.substring(0,openIndex)).get(property.substring(openIndex+1, closeIndex));
			} else {
				result = this.get(property);					
			}
			return result;
		},

		//registers a callback for this object that will be invoked if this object is modified
		registerCallback: function (property, callback) {
			if (callback) {
				if (property) {
					//com.ibm.bpm.coach.GlobalAssets["watchHandles"].push(this.watch(property, callback));
					return this.watch(property, callback);
				} else {
					//com.ibm.bpm.coach.GlobalAssets["watchHandles"].push(this.watch(callback));
					return this.watch(callback);
				}
			}
		}, 
		
	    bind: function(property, callback) {
	    	var obj = this;
	    	if (property) {
				var pathArray = property.split(".");
				if (pathArray.length > 1) {
					property = pathArray.pop();
					obj = this.resolvePath(pathArray);
				}
	    	}    	
			if (obj instanceof com.ibm.bpm.coach.Stateful) {
		    	obj.registerCallback(property, function(object, oldVal, newVal) {
					//Change handler for Stateful objects, calls the subclass change handler
					var event = {property: object, oldVal: oldVal, newVal: newVal};
					callback(event);
				});
			}
	    },

	    bindAll: function(callback) {
	    	this.bind(null, callback);
	    },
		
		unbind : function(handle){
		    handle.unwatch();		
		},
		
		toJson: function() {
			//first get all of the stateful properties and do a toJson on them and 
			//then print the simple properties
			var excludeList = ["_inherited", "childrenCache", "_objectPath", "declaredClass", "_systemCallbackHandle"];

			var resultList = [];
			for (var x in this.childrenCache) {
				resultList.push("\"" + x + "\":" + this.get(x).toJson());
				excludeList.push(x);
			}
			for (var x in this) {
				if (typeof this[x] != 'function'){
					if ((excludeList.join("").indexOf(x)) == -1) {
						if (typeof this[x] == 'object' ||
							Object.prototype.toString.call(this[x] ) === "[object Array]"
							) {
							resultList.push("\"" + x + "\":" + this.get(x).toJson());
						} else {
							resultList.push("\"" + x + "\":" + "\"" + this[x] + "\"");
						}
					}
				}
			}
			return "{" + resultList.join(",") + "}";
		}
		

  	}); //com.ibm.bpm.coach.Stateful
  	

	//List object that uses dojo.store.Observable to "watch" the list for updates.
	//Maintains an internal representation of the list to map ids, so the user of this list does not need to supply unique ids
	//for each list item
	dojo.declare("com.ibm.bpm.coach.List", null, {
		childrenCache: null,
		dataStore: null,
		propertyList: null,
		//index: 0,  //index is the index for the next available spot
		managedList: null,
		indexToId: null, // maitains a map from index to id. Think about remove from middle case; remove once, and remove again
		baseList: null,
		_systemCallbackHandle: null,
		
		constructor: function(listObj, path) {
		    
			this.childrenCache = {};
			this.managedList=[];
			this.indexToId = {};
			this._objectPath = path;

			//populate the internal managedList object with generated ids
			this.baseList = listObj;
			for (var index in listObj) {		
				this.managedList.push({"bpm_id":  this.updateId(index), "listItem": listObj[index]});
			};
			this.dataStore = dojo.store.Observable(new dojo.store.Memory({data: this.managedList, idProperty:"bpm_id"}));
			this.propertyList = this.dataStore.query();

			//register the system callback
			this._systemCallbackHandle = null,
			this.enableSystemNotifications();
		},
		
		disableSystemNotifications: function () {
			this._systemCallbackHandle.cancel()
		},
		
		enableSystemNotifications: function () {
			//set system call back, for notifications to controller
			this._systemCallbackHandle = this.registerCallback(null, dojo.hitch(this, function(changedObject, removedFrom, insertedInto) {
											com.ibm.bpm.coach.controller.systemListChangeHandler(this._objectPath, changedObject, removedFrom, insertedInto);
										}));			
		},
		
		updateId: function(index){
			//TODO: May need to use uuid
		    var id;
            if (this.length() == 0){
				id = 1;
			}
			else{
				id = this.indexToId[this.length()-1] + 1;
			}			
			this.indexToId[index] = id;
			return id;
		},
				
		//registers a callback for this object that will be invoked if this object is modified
		registerCallback: function (property, callback) {
			if (callback)	{
				//com.ibm.bpm.coach.GlobalAssets["watchHandles"].push(this.propertyList.observe(callback, true));
				var handle =  this.propertyList.observe(callback, true);
				//debugger;
				return handle;
			}
		},
		
		close: function () {
			this.propertyList.close();
		},
		//puts a new object on the list, replacing object at index
		put: function(index, object, options, disableSystemNotification) {
		    if (index < 0 || index > this.length() - 1){
				return;
			}						   
		    //update baselist too
			if (this.baseList){
				this.baseList.splice(index, 1, object);		
			}	
			if (!!this.childrenCache[index]){
				delete this.childrenCache[index];
			}

			var result = null;
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				result = this.dataStore.put({"bpm_id": this.indexToId[index], listItem: object}, options); 
				this.enableSystemNotifications();
			} else {
				result = this.dataStore.put({"bpm_id": this.indexToId[index], listItem: object}, options); 
			}
			//Use the same id that is used by the object in this position
			return result;
		},
		
		//adds a new object to the end of the list
		add: function(object, options, disableSystemNotification) {		
			var index = this.length();
			this.baseList.push(object);
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				this.dataStore.add({"bpm_id": this.updateId(index), listItem: object}, options)	;
				this.enableSystemNotifications();
			} else {
				this.dataStore.add({"bpm_id": this.updateId(index), listItem: object}, options)	;				
			}
			return index;
		},
		
		//Insert a new object to the specified index
		//To do: the only way make this work is to add another field called order  Can we use the bpmId??? 
		//Do we want to support this, then?
		addAt: function(index, object, options, disableSystemNotification) {	
            if (index < 0 || index > this.length()){
				return;
			}				
			if (this.baseList){
				this.baseList.splice(index, 0, object);		
			}	
			//This does not work. Need to enforce some ordering...
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				this.dataStore.put({"bpm_id": this.updateId(index), listItem: object}, options);
				this.enableSystemNotifications();
			} else {	
				this.dataStore.put({"bpm_id": this.updateId(index), listItem: object}, options);
			}
		},

		//removes the object at index from the list					
		remove: function(index, disableSystemNotification) {
			if (index < 0 || index >= this.length()){
				return;
			}		  
			if (this.baseList) {
				this.baseList.splice(index, 1);
			}
			var result = null;
			if (disableSystemNotification && disableSystemNotification == true) {
				this.disableSystemNotifications();
				result = this.dataStore.remove(this.indexToId[index]);
				this.enableSystemNotifications();
			} else {
				result = this.dataStore.remove(this.indexToId[index]);
			}
			
			//update all the index to Id mappings			
			for (var i=index; i < this.length(); i++){			   
				this.indexToId[i] = this.indexToId[i + 1];
				this.childrenCache[i] = this.childrenCache[i + 1];
			}
			delete this.indexToId[i];
			delete this.childrenCache[i];
			return result;
		},
		

		//gets the object at index
		//also maintains a cache of any embedded lists or obejcts and makes them watchable.
		get: function(index) {
			var result;
			
			//first check the cache
			var property = this.childrenCache[index];

			if (typeof(property) == "undefined") {
				var obj = this.baseList[index];  
				
				//will remove the use of baseList so that we do not need to maintain it
				//var obj = this.dataStore.get(this.indexToId[index]);  
				
				//check if it is a simple type
				var dataTypes = ["number", "string", "boolean"];
				dojo.forEach(dataTypes, function(dataType) {
					if (typeof obj == dataType) {
						result = obj;
					}
				});

				if (typeof result == "undefined") {
					if (Object.prototype.toString.call( obj ) === "[object Array]") {
						result = new com.ibm.bpm.coach.List(obj, this._objectPath + "[" +index + "]");
						//add result to the cache
						this.childrenCache[index] = result;
		        	} else if (typeof obj == "object") {
		        		if (!obj.prototype) { // check that this is a literal
		        			result = obj;
		        		} else if (obj.constructor) {
		        			result = new obj.constructor();
		        		}
		        		result = new com.ibm.bpm.coach.Stateful(result, this._objectPath + "[" +index + "]");

	        			//add result to the cache
	        			this.childrenCache[index] = result;
	        		}
				} else {
					result = obj;
				}
			} else {
				result = property;
			}
			return result;
		}, //get 
		
		//retuns the length of the list
		length: function() {
			return this.managedList.length; 
			//this can be replace by maintaina a length variable or check the length of managed list
		},
		
		bindAll: function(callback) {
		    var wrapperFunction = function(object, removedFrom, insertedInto) {
				var newVal, oldVal;		
				if (insertedInto > -1) {
					newVal = object.listItem;
	             }
				if (removedFrom > -1){
					oldVal = object.listItem;			 
				 }
				var event = {property: null, oldVal: oldVal, newVal: newVal, removedFrom: removedFrom, insertedInto: insertedInto};
				callback(event);
			}
			return this.registerCallback(null, wrapperFunction);
				
	    },
		
		unbind : function(){
				
		},

		toJson: function() {
			//first get all of the stateful properties and do a toJson on them and 
			//then print the simple properties

			var resultList = [];
			for (var x=0; x< this.baseList.length; x++) {
				if (typeof this.baseList[x] == 'object' || 
					Object.prototype.toString.call(this.baseList[x] ) === '[object Array]' ||
					typeof this.baseList[x] == 'com.ibm.bpm.coach.Stateful' ||
					typeof this.baseList[x] == 'com.ibm.bpm.coach.List') {
					resultList.push(this.get(x).toJson());
				} else {
					resultList.push(this.baseList[x]);
				}
			}
			if (resultList.length > 0){
				return "[" + "\"" +  resultList.join("\",\"") + "\"" + "]";
			}
			else{
			    return "[" + "]";
			}
		}

	}); //com.ibm.bpm.coach.List
//##DataBinding end



// controller start
com.ibm.bpm.coach.controller = new Object();

com.ibm.bpm.coach.controller.inCollaboration = false;

com.ibm.bpm.coach.controller.init = function() {
    com.ibm.bpm.coach.controller.initNotificationBus();
    com.ibm.bpm.coach.controller.initFocus();
    com.ibm.bpm.coach.controller.subscribeToBoundaryEvents();
    com.ibm.bpm.coach.controller.initProgressIcon();
}

com.ibm.bpm.coach.controller.initProgressIcon = function() {
    var img = document.createElement('img');
    img.src = '/portal/js/com/ibm/bspace/common/util/ui/EmbeddedMsg/img/messageIndicators/loading.gif';
    com.ibm.bpm.coach.controller.progressIcon = document.createElement('div');
    com.ibm.bpm.coach.controller.progressIcon.style.position = 'fixed';
    com.ibm.bpm.coach.controller.progressIcon.style.top = '10px';
    com.ibm.bpm.coach.controller.progressIcon.style.right = '10px';
    com.ibm.bpm.coach.controller.progressIcon.visibility = "hidden";
    com.ibm.bpm.coach.controller.progressIcon.style.display = "none";
    com.ibm.bpm.coach.controller.progressIcon.appendChild(img);
    document.body.appendChild(com.ibm.bpm.coach.controller.progressIcon);
}

com.ibm.bpm.coach.controller.showProgressIcon = function() {
    com.ibm.bpm.coach.controller.progressIcon.style.display = "block";
    com.ibm.bpm.coach.controller.progressIcon.style.visibility = "visible";
}

com.ibm.bpm.coach.controller.hideProgressIcon = function() {
    com.ibm.bpm.coach.controller.progressIcon.style.display = "none";
    com.ibm.bpm.coach.controller.progressIcon.style.visibility = "hidden";
}

com.ibm.bpm.coach.controller.initNotificationBus = function() {
    // should be changed to sessionId, provided by codegen
    com.ibm.bpm.coach.controller.sessionId = com.ibm.bpm.coach.GlobalAssets.systemData.zTaskId;
    // joining session with the given session id, if session does not exist, it is being initialized
    com.ibm.bpm.social.portal.util.collaboration.RegisterForCollaboration(com.ibm.bpm.coach.controller.sessionId,
                                                  com.ibm.bpm.coach.GlobalAssets.systemData.taskOwnerId,
                                                  // fullSynchReceivedCallbackMethod
                                                  dojo.hitch(this, "handleFullSync"),
                                                  // publishFullSynchCallbackMethod
                                                  dojo.hitch(this, "publishFullSync"),
                                                  // incomingChangesetCallbackMethod
                                                  dojo.hitch(this, "handleBusNotifications"),
                                                  // rosterChangeCallbackMethod
                                                  dojo.hitch(this, "handleRosterChange"));
}

com.ibm.bpm.coach.controller.publishFullSync = function() {
    com.ibm.bpm.coach.controller.inCollaboration = true;
    return com.ibm.bpm.coach.GlobalAssets.local.toJson();
}

com.ibm.bpm.coach.controller.handleFullSync = function(data) {
    // we assume that we are in collaboration if we are handling full sync
    com.ibm.bpm.coach.controller.inCollaboration = true;
    var baseObject = dojo.fromJson(data);
    // setting new properties and refreshing views at the same time
    for (var x in baseObject) {
        com.ibm.bpm.coach.GlobalAssets.local.set(x, baseObject[x], true);
    }
}

com.ibm.bpm.coach.controller.handleBusNotifications = function(data) {
    var event = dojo.fromJson(data.data);
    com.ibm.bpm.coach.controller.processNotification(event);
}

com.ibm.bpm.coach.controller.handleRosterChange = function(sessionRoster) {
    if (sessionRoster.allParticipants.length < 2) {
        com.ibm.bpm.coach.controller.inCollaboration = false;
        return;
    }
    for (var i = 0; i < sessionRoster.allParticipants.length; i++) {
        if (sessionRoster.allParticipants[i].fullName == system.user_fullName) {
            com.ibm.bpm.coach.controller.inCollaboration = true;
            return;
        }
    }
}

com.ibm.bpm.coach.controller.processBoundaryEvent = function(boundaryEvent) {
    var event = boundaryEvent.event;
    if (event == 'taskCompleted') {
        com.ibm.bpm.coach.TaskCompleted(boundaryEvent.url, boundaryEvent.taskId);
    }
    if (event == 'taskPostponed') {
        com.ibm.bpm.coach.TaskCompleted(boundaryEvent.url);
    }
    if (event == 'taskStateChanged') {
        com.ibm.bpm.coach.StateChange(boundaryEvent.url);
    }
    if (event == 'taskError') {
        com.ibm.bpm.coach.ErrorBoundaryEvent(boundaryEvent.errorMessage);
    }
}

com.ibm.bpm.coach.controller.subscribeToBoundaryEvents = function() {
    dojo.subscribe("com.ibm.bpm.coach.BoundaryEventTriggered", function(obj) {
        com.ibm.bpm.coach.controller.handleBoundaryEvent(obj.boundaryEventId, obj.boundaryEventEndStateId, obj.callback);
    });
}

com.ibm.bpm.coach.controller.publishNotification = function(notificationObject) {
    if (com.ibm.bpm.coach.controller.inCollaboration) {
        com.ibm.bpm.social.portal.util.collaboration.publishData(notificationObject);
    }
}

com.ibm.bpm.coach.controller.handleBoundaryEvent = function(boundaryEventId, boundaryEventEndStateId, callback) {
    var localData = "{}";
    // we need to check if GlobalAssets.local is not empty
    if (!!com.ibm.bpm.coach.GlobalAssets.local.toJson) {
	    localData = com.ibm.bpm.coach.GlobalAssets.local.toJson();
    }
	var coachData = "{\"boundaryEventInfo\":{" +
                    "\"boundaryEventId\":\"" + boundaryEventId + "\"," +
                    "\"boundaryEventEndStateId\":\"" + boundaryEventEndStateId + "\"}," +
                    "\"stateInfo\":{" +
                    "\"zComponentId\":\"" + com.ibm.bpm.coach.GlobalAssets.systemData.zComponentId + "\"," +
                    "\"zComponentName\":\"CoachNG\"," +
                    "\"zTaskId\":\"" + com.ibm.bpm.coach.GlobalAssets.systemData.zTaskId + "\"," +
                    "\"zWorkflowState\":\"" + com.ibm.bpm.coach.GlobalAssets.systemData.zWorkflowState + "\"," +
                    "\"applicationId\":\"" + com.ibm.bpm.coach.GlobalAssets.systemData.applicationId + "\"," +
                    "\"applicationInstanceId\":\"" + com.ibm.bpm.coach.GlobalAssets.systemData.applicationInstanceId + "\"," +
                    "\"zActType\":\"" + com.ibm.bpm.coach.GlobalAssets.systemData.zActType + "\"," +
                    "\"zDbg\":\"" + com.ibm.bpm.coach.GlobalAssets.systemData.zDbg + "\"}," +
                    "\"data\":{\"local\":" + localData + "}}";
    console.debug("coach data: " + coachData);
    com.ibm.bpm.coach.controller.showProgressIcon();
    var xhrArgs = {
        url: "/teamworks/ajaxCoach",
        handleAs: "json",
        headers: { 
            "Content-Type":"application/json; charset=utf-8" 
        },
        postData: coachData,
        load: function(response) {
            com.ibm.bpm.coach.controller.hideProgressIcon();
            console.debug("Success in calling ajax servlet");
            if (!!callback) {
                callback();
            }
            if (response) {
                console.debug(response);
                var notificationEvent;
                var event = response.event;
                var system = com.ibm.bpm.coach.GlobalAssets.system;
                if (event == 'taskCompleted') {
                    console.debug("task is completed");
                    var nextUrl = response.url;
                    var nextTaskId;
                    if (!!response.taskId) {
                        nextTaskId = response.taskId;
                    }
                    notificationEvent = {type: 'boundary', event: 'taskCompleted', url: nextUrl, taskId: nextTaskId, userName: system.user_fullName};
                    // sending boundary event notification
                    com.ibm.bpm.coach.controller.publishNotification(dojo.toJson(notificationEvent));
                    com.ibm.bpm.coach.TaskCompleted(nextUrl, nextTaskId);
                }
                if (event == 'taskPostponed') {
                    console.debug("task is postponed");
                    var nextUrl = response.url;
                    notificationEvent = {type: 'boundary', event: 'taskPostponed', url: nextUrl, userName: system.user_fullName};
                    // sending boundary event notification
                    com.ibm.bpm.coach.controller.publishNotification(dojo.toJson(notificationEvent));
                    com.ibm.bpm.coach.TaskCompleted(nextUrl);
                }
                if (event == 'taskStateChanged') {
                    console.debug("task state is changed");
                    var nextUrl = response.url;
                    notificationEvent = {type: 'boundary', event: 'taskStateChanged' ,url: nextUrl, userName: system.user_fullName};
                    // sending boundary event notification
                    com.ibm.bpm.coach.controller.publishNotification(dojo.toJson(notificationEvent));
                    com.ibm.bpm.coach.StateChange(nextUrl);
                }
                if (event == 'taskError') {
                    var message = response.errorMessage;
                    console.debug("error has occured: " + message);
                    notificationEvent = {type: 'boundary', event: 'taskError', url: nextUrl, userName: system.user_fullName};
                    // sending boundary event notification
                    com.ibm.bpm.coach.controller.publishNotification(dojo.toJson(notificationEvent));
                    com.ibm.bpm.coach.ErrorBoundaryEvent(message);
                }
            }
        },
        error: function(error) {
            com.ibm.bpm.coach.controller.hideProgressIcon();
            console.debug("An unexpected error occurred: " + error);
            com.ibm.bpm.coach.ErrorBoundaryEvent(error);
        }
    }

    dojo.xhrPost(xhrArgs); 
}

com.ibm.bpm.coach.controller.processNotification = function(event) {
	//router for notification events from the bus
	if (event.type) {
        // we ignore event from the same user
        if (event.userName != com.ibm.bpm.coach.GlobalAssets.system.user_fullName) {
            if (event.type == 'focus') {
                com.ibm.bpm.coach.controller.processFocusEvent(event);
            } else if (event.type == 'data') {
                com.ibm.bpm.coach.controller.processDataEvent(event);
            } else if (event.type == 'boundary') {
                com.ibm.bpm.coach.controller.processBoundaryEvent(event);
            } else {
                console.debug("could not process notification event");
                console.dir(event);
            }
        }
	} else {
		console.debug("could not process notification event");
		console.dir(event);
	}
}
com.ibm.bpm.coach.controller.initFocus = function() {
	com.ibm.bpm.coach.controller.setFocusHandlers();
	
	//init data structures to handle focus collaboration
	com.ibm.bpm.coach.controller.currentUserFocus = {};
	com.ibm.bpm.coach.controller.focusTextColor = "#ffffff";
	com.ibm.bpm.coach.controller.colorList = ["#07e5e2", "#ec8705", "#59a043", "#1b75bc", "#8d0ee6", "#0818f1", "#e5c307", "#e50707", "#cd462f"];

	com.ibm.bpm.coach.controller.userColorMap = {};
	com.ibm.bpm.coach.controller.userFocusMap = {};
	com.ibm.bpm.coach.controller.focusElementToUserMap = {};	
}


com.ibm.bpm.coach.controller.focusHandler = function (event) {
	//finds the id of the focus target and the id of the view it belongs to 
	//and passes the ids and focus event to the controller

	//IE
	if (!event.target) {
		event.target = event.srcElement;
	}
	
	var id = event.target.id;
	var viewId = null;
	var parent = event.target.parentNode;
	while (parent) {
		if (parent.className && parent.className.indexOf("view") != -1) {
			viewId = parent.id;
			break;
		}
		parent = parent.parentNode;
	}
	if (!id) {
		id = viewId;
	}
	e = {'type': 'focus', 'domId': id, 'viewId': viewId, 'userName': this.system.user_fullName};
	com.ibm.bpm.coach.controller.currentUserFocus.domId = id;
	com.ibm.bpm.coach.controller.currentUserFocus.viewId = viewId;
	console.debug ("sending focus collab event on bus: ");
	console.debug (e);
	
	// publishing focus notification
    com.ibm.bpm.coach.controller.publishNotification(dojo.toJson(e));
	
	//aalim test - start
//	console.debug("simulating a focus change from another coach");
//	com.ibm.bpm.coach.controller.processNotification(e);
	//aalim end - end
	
}	

com.ibm.bpm.coach.controller.mouseOnHandler = function (event) {
	//detected a hover event.  Need to check if the current domId+viewId has any users who are focused on it.
	//If so, notify the framework to show a bubble on the element.

	//IE
	if (!event.target) {
		event.target = event.srcElement;
	}
	
	if (!!event && !!event.target && event.target.id != "mainBody") {
		var id = event.target.id;
		var viewId = null;
		var parent = event.target.parentNode;
		while (parent) {
			if (parent.className && parent.className.indexOf("view") != -1) {
				viewId = parent.id;
				break;
			}
			parent = parent.parentNode;
		}
		if (!id) {
			id = viewId;
		}
		var focusedUsers = com.ibm.bpm.coach.controller.focusElementToUserMap[id+viewId];
		if (focusedUsers && focusedUsers.length >0) {
			var e = 	{viewId: viewId, domId: id};
			e.color = com.ibm.bpm.coach.controller.focusTextColor;
			e.bgColor = com.ibm.bpm.coach.controller.userColorMap[focusedUsers[0]];
			if (focusedUsers.length >1) {
				e.message = focusedUsers[0] + ",...";
			} else {
				e.message = focusedUsers[0];
			}
			
			console.debug("Calling framework function: com.ibm.bpm.coach.handleMouseOnEvent(e) with values e.viewId: "+e.viewId+
			        ", e.domId: "+e.domId+", e.color: "+e.color+", e.bgColor: "+e.bgColor+", e.message: "+e.message);
			com.ibm.bpm.coach.handleMouseOnEvent(e);
		}
	}
}

com.ibm.bpm.coach.controller.mouseOffHandler = function (event) {
	//detected a hover leave event.  Need to check if the current domId+viewId has any users who are focused on it.
	//If so, notify the framework to hide the bubble on the element.

	//IE
	if (!event.target) {
		event.target = event.srcElement;
	}
	
	if (!!event && !!event.target && event.target.id != "mainBody") {
		var id = event.target.id;
		var viewId = null;
		var parent = event.target.parentNode;
		while (parent) {
			if (parent.className && parent.className.indexOf("view") != -1) {
				viewId = parent.id;
				break;
			}
			parent = parent.parentNode;
		}
		if (!id) {
			id = viewId;
		}
		var focusedUsers = com.ibm.bpm.coach.controller.focusElementToUserMap[id+viewId];
		if (focusedUsers && focusedUsers.length >0) {
			var e = 	{viewId: viewId, domId: id};

			console.debug("Calling framework function: com.ibm.bpm.coach.handleMouseOffEvent(e) with values e.viewId: "+e.viewId+
			        ", e.domId: "+e.domId);
			com.ibm.bpm.coach.handleMouseOffEvent(e);
		}
	}
}


com.ibm.bpm.coach.controller.setFocusHandlers = function () {
	//sets a focus handler on the mainBody element to process all focus changes on the coach
	var mainBody = dojo.byId("mainBody");
	
	if (mainBody.addEventListener) { //non-IE
		mainBody.addEventListener('focus', dojo.hitch(com.ibm.bpm.coach.GlobalAssets, com.ibm.bpm.coach.controller.focusHandler), true); 
		mainBody.onmouseover = com.ibm.bpm.coach.controller.mouseOnHandler;
		mainBody.onmouseout = com.ibm.bpm.coach.controller.mouseOffHandler;
	} else {  // IE
	  //mainBody.onfocusin = com.ibm.bpm.coach.controller.focusHandler;
		mainBody.attachEvent('onfocusin', dojo.hitch(com.ibm.bpm.coach.GlobalAssets, com.ibm.bpm.coach.controller.focusHandler));
		mainBody.attachEvent('onmouseover', com.ibm.bpm.coach.controller.mouseOnHandler);
		mainBody.attachEvent('onmouseout', com.ibm.bpm.coach.controller.mouseOffHandler);
	}	
}

com.ibm.bpm.coach.controller.processFocusEvent = function(e) {
	//a focus event has come in, need to check if the user already is assigned a color, if not, assign one
	if (!com.ibm.bpm.coach.controller.userColorMap[e.userName]) {
		com.ibm.bpm.coach.controller.userColorMap[e.userName] = com.ibm.bpm.coach.controller.colorList.pop();
	}
	
	//check the user focus map to see if this user has an existing focus
	var userFocus = com.ibm.bpm.coach.controller.userFocusMap[e.userName];
	if (userFocus) {
		//use this key to remove the user from the focusElementToUserMap
		var listOfUsers = com.ibm.bpm.coach.controller.focusElementToUserMap[userFocus.domId+userFocus.viewId];
		for (var x=0; x<listOfUsers.length; x++) {
			if (listOfUsers[x] == e.userName) {
				listOfUsers.splice(x,1);
				break;
			}
		}
		//if there are any other users who are currently focused on this element, send a focus event to the framework on their behalf,
		//otherwise ask the framework to blur this element
		if (listOfUsers.length > 0) {
			var altColor = com.ibm.bpm.coach.controller.userColorMap[listOfUsers[0]];
			com.ibm.bpm.coach.handleFocusEvent({viewId: userFocus.viewId, domId: userFocus.domId, color: altColor});
		} else {
			com.ibm.bpm.coach.handleBlurEvent({viewId: userFocus.viewId, domId: userFocus.domId});
		}
	}
	//change the users new focus
	com.ibm.bpm.coach.controller.userFocusMap[e.userName] = {domId: e.domId, viewId: e.viewId};
	//add the user to the focus map of the new dom
	if (!com.ibm.bpm.coach.controller.focusElementToUserMap[e.domId+e.viewId]) {
		com.ibm.bpm.coach.controller.focusElementToUserMap[e.domId+e.viewId] = [];
	}
	com.ibm.bpm.coach.controller.focusElementToUserMap[e.domId+e.viewId].push(e.userName);
	
	console.debug("Calling framework function: com.ibm.bpm.coach.handleFocusEvent(e) with values e.viewId: "+e.viewId+
			        ", e.domId: "+e.domId+", color: "+com.ibm.bpm.coach.controller.userColorMap[e.userName]+")");
	com.ibm.bpm.coach.handleFocusEvent(e);
}

com.ibm.bpm.coach.controller.processDataEvent = function(e) {
	
	//process data event
	var statefulObject = null; 
	if (e.path == "local") {
		statefulObject = com.ibm.bpm.coach.GlobalAssets.local;
	}
	else {
		var path = e.path.substring("local.".length);
		statefulObject = com.ibm.bpm.coach.GlobalAssets.local.get(path);
	}
		
	if(statefulObject instanceof com.ibm.bpm.coach.Stateful) {
		console.debug("find the local object and set the new value on it: " + e.path+"." +e.property + " = " + e.value);
		var processedValue = e.value;
		if (e.value[0] == "{" || e.value[0] == "[")
		{
			processedValue = eval('('+e.value+')'); 
		}
		console.debug("set the object with the new value");
		console.debug(processedValue);

		//disable system notifications before doing the set to prevent an infinite
		//loop of notifications
		statefulObject.set(e.property, processedValue, true);			
	} else {
		//must be a list
		if (e.insertedInto == -1) {
			//must be a delete
			if (e.removedFrom != -1) {
				statefulObject.remove(e.removedFrom, true)
			} else {
				console.debug("error could not process update: ");
				console.dir(e);
			}
		} else {
			//either an insert or a update
			processedValue = e.value;
			if (e.value[0] == "{" || e.value[0] == "[")
			{
				processedValue = eval('('+e.value+')'); 
			}
			if (e.removedFrom != -1) {
				//update
				statefulObject.put(e.insertedInto, processedValue.listItem, null, true);
			} else {
				//add
				statefulObject.addAt(e.insertedInto, processedValue.listItem, null, true);
			}
		}
	}	
	e.color = com.ibm.bpm.coach.controller.focusTextColor;
	e.bgColor = com.ibm.bpm.coach.controller.userColorMap[e.userName];
	e.message = e.userName + " changed this (needs NL)"
	console.debug("Calling framework function: com.ibm.bpm.coach.handleCollabDataChangeEvent(e) " +
			           "where e.domId="+e.domId+", e.viewId=" +e.viewId + 
			           ", e.color="+ e.color + ", e.bgColor="+e.bgColor + ", e.message=" +e.message);
	com.ibm.bpm.coach.handleCollabDataChangeEvent(e);
}

//system change handlers
com.ibm.bpm.coach.controller.systemChangeHandler = function (path, prop, newValue) {
	var type = typeof newValue;
	if (typeof newValue == 'object' || 
		  Object.prototype.toString.call(newValue) === '[object Array]') {
		var shortPath = path;
		if (prop && prop != "") {
			shortPath = shortPath + "." + prop;
		}
		if (shortPath.indexOf("local.") != -1)
		{
			shortPath = shortPath.substring("local.".length);
		}
		newValue = com.ibm.bpm.coach.GlobalAssets.local.get(shortPath).toJson();
	}
	var system = com.ibm.bpm.coach.GlobalAssets.system;
	console.debug("sending task collab change event on bus");
	var event = {type: 'data', path: path, property: prop, value: newValue, userName: system.user_fullName, 
				 domId: com.ibm.bpm.coach.controller.currentUserFocus.domId, viewId: com.ibm.bpm.coach.controller.currentUserFocus.viewId };
	console.dir(event);
	
	// publishing data change (collaboration) notification
    com.ibm.bpm.coach.controller.publishNotification(dojo.toJson(event));
	
//aalim test -start
//	console.debug("simulate a data change event from the bus");
//	com.ibm.bpm.coach.controller.processNotification(event);
// aalm test -end
}

com.ibm.bpm.coach.controller.systemListChangeHandler = function (path, object, removedFrom, insertedInto) {
	console.debug("TODO: send task collab change event for list on bus");

	var objectString = ""
	if (insertedInto != -1) {
		//this is not a remove operation
		objectString = dojo.toJson(object);
	}
	var system = com.ibm.bpm.coach.GlobalAssets.system;
	var event = {type: 'data', path: path, removedFrom: removedFrom, insertedInto: insertedInto, value: objectString, userName: system.user_fullName, 
			 domId: com.ibm.bpm.coach.controller.currentUserFocus.domId, viewId: com.ibm.bpm.coach.controller.currentUserFocus.viewId };
	console.dir(event);
	
	// publishing data change notification (collaboration)
    com.ibm.bpm.coach.controller.publishNotification(dojo.toJson(event));

//aalim test -start
//	console.debug("simulate a list data change event from the bus");
//	com.ibm.bpm.coach.controller.processNotification(event);
// aalm test -end
}

com.ibm.bpm.coach.controller.destroy = function() {
}

// controller end


com.ibm.bpm.coach.findEndStateId = function(bdEventId) {
    var systemData = com.ibm.bpm.coach.GlobalAssets.systemData;
    if (systemData.boundaryEventInformation) {
        for (var i=0; i<systemData.boundaryEventInformation.length; i++) {
            if (systemData.boundaryEventInformation[i].boundaryEventID == bdEventId) {
                return systemData.boundaryEventInformation[i].endStateID;
            }
        }
    }
    return "";
}


// Helper API for handling boundary event response
com.ibm.bpm.coach.TaskCompleted = function(nextURL, nextTaskId) {
	com.ibm.bpm.coach.onCoachUnload.call(com.ibm.bpm.coach.GlobalAssets);
	dojo.require("dojo.cookie");
	var parentObjectName = dojo.cookie("com.ibm.bpm.coach.parentObjectName");
	if ((!!parentObjectName) && (!!window.parent[parentObjectName])) {
		window.parent[parentObjectName].onCompleted(nextTaskId);
	} else {
		// redirect to the next URL
		window.location.assign(nextURL);
	}
}

com.ibm.bpm.coach.TaskPostponed = function(nextURL) {
	com.ibm.bpm.coach.onCoachUnload.call(com.ibm.bpm.coach.GlobalAssets);
	dojo.require("dojo.cookie");
	var parentObjectName = dojo.cookie("com.ibm.bpm.coach.parentObjectName");
	if ((!!parentObjectName) && (!!window.parent[parentObjectName])) {
		window.parent[parentObjectName].onPostponed();
	} else {
		// redirect to the next URL
		window.location.assign(nextURL);
	}
}


com.ibm.bpm.coach.StateChange = function(nextURL) {
	com.ibm.bpm.coach.onCoachUnload.call(com.ibm.bpm.coach.GlobalAssets);
	// redirect to the next URL
	window.location.assign(nextURL);
}


com.ibm.bpm.coach.ErrorBoundaryEvent = function(message) {
	dojo.require("dojo.cookie");
	var parentObjectName = dojo.cookie("com.ibm.bpm.coach.parentObjectName");
	if ((!!parentObjectName) && (!!window.parent[parentObjectName])) {
		window.parent[parentObjectName].onError(message);
	} else {
		// Show error dialog
		// TODO: Use the common UI dialog
		alert(message);
	}
}

// Helper API for handling collab related event
com.ibm.bpm.coach.handleFocusEvent = function(event) {
	// event.viewId means the dom id of the closest viewScope node
	if(!event.viewId) {
		return;
	}
	var viewScope = com.ibm.bpm.coach._findViewScope(event.viewId);
	if(!!viewScope) {
		var domId = !!event.domId ? event.domId : event.viewId;
		var color = !!event.color ? event.color : "red"; // now use "red" as default color. refine later
		viewScope.collaboration({"type": "focus", "domId": domId, "color": color});
	}
}

com.ibm.bpm.coach.handleBlurEvent = function(event) {
	// event.viewId means the dom id of the closest viewScope node
	if(!event.viewId) {
		return;
	}
	var viewScope = com.ibm.bpm.coach._findViewScope(event.viewId);
	if(!!viewScope) {
		var domId = !!event.domId ? event.domId : event.viewId;
		viewScope.collaboration({"type": "blur", "domId": domId});
	}
}

com.ibm.bpm.coach.handleMouseOnEvent = function(event) {
	// event.viewId means the dom id of the closest viewScope node
	if(!event.viewId) {
		return;
	}
	var viewScope = com.ibm.bpm.coach._findViewScope(event.viewId);
	if(!!viewScope) {
		var domId = !!event.domId ? event.domId : event.viewId;
		var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
		var bgColor = !!event.bgColor ? event.bgColor : "red"; // now use "red" as default background color. refine later
		var message = !!event.message ? event.message : "Unknow";
		viewScope.collaboration({"type": "mouseOn", "domId": domId, "color": color, "bgColor": bgColor, "message": message});
	}
}

com.ibm.bpm.coach.handleMouseOffEvent = function(event) {
	// event.viewId means the dom id of the closest viewScope node
	if(!event.viewId) {
		return;
	}
	var viewScope = com.ibm.bpm.coach._findViewScope(event.viewId);
	if(!!viewScope) {
		var domId = !!event.domId ? event.domId : event.viewId;
		viewScope.collaboration({"type": "mouseOff", "domId": domId});
	}
}

com.ibm.bpm.coach.handleCollabDataChangeEvent = function(event) {
	// event.viewId means the dom id of the closest viewScope node
	if(!event.viewId) {
		return;
	}
	var viewScope = com.ibm.bpm.coach._findViewScope(event.viewId);
	if(!!viewScope) {
		var domId = !!event.domId ? event.domId : event.viewId;
		var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
		var bgColor = !!event.bgColor ? event.bgColor : "red"; // now use "red" as default background color. refine later
		var message = !!event.message ? event.message : "Unknow";
		var userName = !!event.userName ? event.userName : "Unknow user";
		var value = !!event.value ? event.value : "Unknow value";
		viewScope.collaboration({"type": "dataChange", "domId": domId, "color": color, "bgColor": bgColor, "message": message, "userName": userName, "value": value});
	}
}

// Main modules for the framework starts
com.ibm.bpm.coach.GlobalAssets = {"topLevelViewScope": {}, "watchHandles": [], "local" : {}, "epv": {},
				  "system": {}, "systemData": {}, "resource": {}, "options": {}};
				  
// merge those 2 to com.ibm.bpm.coach.GlobalAssets later if needed
var hoverBox;
var bubbleBox;

dojo.addOnLoad(function() {
	com.ibm.bpm.coach.initData.call(com.ibm.bpm.coach.GlobalAssets);
	com.ibm.bpm.coach.initConfig.call(com.ibm.bpm.coach.GlobalAssets);
	com.ibm.bpm.coach.controller.init();
	com.ibm.bpm.coach.initViews.call(com.ibm.bpm.coach.GlobalAssets);
	com.ibm.bpm.coach.prepareView.call(com.ibm.bpm.coach.GlobalAssets);
	com.ibm.bpm.coach.completeInitialization();
});

com.ibm.bpm.coach.completeInitialization = function() {
	// Notify task management if we are running inside them
	dojo.require("dojo.cookie");
	var parentObjectName = dojo.cookie("com.ibm.bpm.coach.parentObjectName");
	if ((!!parentObjectName) && (!!window.parent[parentObjectName])) {
		window.parent[parentObjectName].onInitialized();
	}
}

com.ibm.bpm.coach.onWindowClose = function() {
	com.ibm.bpm.coach.onCoachUnload.call(com.ibm.bpm.coach.GlobalAssets);
    com.ibm.bpm.coach.controller.destroy();
}

com.ibm.bpm.coach.onCoachUnload = function() {
	for (var viewId in this.topLevelViewScope) {
		   var viewScope = this.topLevelViewScope[viewId];
		   com.ibm.bpm.coach.callUnload(viewScope);
	}
}

com.ibm.bpm.coach.callUnload = function(viewScope) {
	if(!viewScope) return;
	
	// Call nested view unload first
	if (!!viewScope.context.subview) {
		for (var viewId in viewScope.context.subview) {

			var subviews = viewScope.context.subview[viewId];
			for(var divId in subviews) {
				var subViewScope = subviews[divId];
				com.ibm.bpm.coach.callUnload(subViewScope);
			}
		}		
	}
	
    // Call the view unload lifecycle method
	viewScope.unload();
}

window.onbeforeunload = com.ibm.bpm.coach.onWindowClose;
// Main modules for the framework ends
