/*
Licensed Materials - Property of IBM
5725-C95
(C) Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach view super class js

define(["dojo/_base/lang", "dojo/_base/array", "dojo/dom", "dojo/dom-class", "./CONSTANTS", "./commonHelper", "./domHelper", "./engine", "./collaborationPopup","./logger", "./dataBindingHelper", "./multiBindingListsHelper", "./validationHelper"], function(lang, array, dom, domClass, CONSTANTS, commonHelper, domHelper, engine, collaborationPopup, logger, dataBindingHelper, multiBindingListsHelper, validationHelper) {
	return function (context, element, binding, option, readonly) {
		
		if(!!context) {
			this.context = context;
			this.context._init(this, engine.GlobalAssets.system, element, binding, option, readonly);
		}
		
		// if the view binds with a list and is framework managed, use following templates to handle the dynamic adding (e.g. list has been inserted a new item)
		this._contentBoxTemplates = {};
		// use this counter to work out the unique id (div id) for the content in the repeating ContentBox
		this._counter = 0;
				
		// hold all the handles of binding
		this._bindingHandles = {};
		
		this._impactedByMultipleLists = undefined;
		this._domIdToBindingListMap = {};
		// we monitor those list objects
		this._bindingStrToListMap = {};
		
	    this.view = function() {
	    	// Super-class lifeCycle view method
	    	var visibility = (!!this.context.options  &&  (!!this.context.options._metadata) && !!(this.context.options._metadata.visibility)) ? this.context.options._metadata.visibility.get("value"): "DEFAULT";
	    	if (visibility == "DEFAULT") {
	    		visibility = this.context.getInheritedVisibility();
	    	}
	    	if (visibility == "NONE") {
	    		this.context.setDisplay(false);
	    	} else {
	    		this.context.setDisplay(true);
	    	}

	    };
	    
	    // Internal method
	    this._preLoad = function () {
	    	this.context.viewid = this.context.element.getAttribute(engine.CONSTANTS.VIEW_ID);
	    }
	    
	    this.load = function() {
	    	// Super-class lifeCycle load method
	    };
	    
	    // Internal method
	    this._preUnload = function() {
	    	for(var k in this._bindingHandles) {
	    		if(!!this._bindingHandles[k].unbind){
	    			this._bindingHandles[k].unbind();
	    		}
	    		else if(!!this._bindingHandles[k].cancel){
	    				this._bindingHandles[k].cancel();
	    		}
	    	}
	    	this._bindingHandles = null;
	    }

	    this.unload = function() {
	    	// Super-class lifeCycle unload method
	    	// console.debug("unload() is called on viewid " + this.context.viewid + ", Class is " + this.context.element.getAttribute("Class"));
	    };
	    
	    // Internal method to make the CoachView object ready for garbage collection
	    this._postUnload = function() {
	    	// considering this.context could be referenced by something, set the content of context to null firstly then set context to null
	    	this.context.element = null;
	    	this.context.binding = null;
	    	this.context.options = null;
	    	this.context.subview = null;
	    	this.context.bpm = null;
	    	this.context._parent = null;
	    	this.context._parentView = null;	    	
	    	this.context = null;	    	
	    }

		this._change = function(event){
			
			//if the change is for data-binding
			if (event.type !== "config" && !!this.context.binding && this.context.binding.get("value") instanceof com.ibm.bpm.coach.List){
				var bindingHandle = this._bindingHandles["_listUpdate"];
				if (!!bindingHandle && bindingHandle.unbind){
					bindingHandle.unbind();
				}
				engine.refreshRepeatableView(this, event.oldVal, event.newVal, this.context.binding.boundObject._objectPath + "." + this.context.binding.property);
				this._bindingHandles["_listUpdate"] = this.context.binding.get("value").bindAll(lang.hitch(this, this._listUpdated));
			}
			
			engine.callViewLifeCycleAPI(this, "change", CONSTANTS.ELAPSED_TIME_THRESHOLD_CHANGE, event);
			
			//post change,
			//one thing needs to do is to check if we need to update the visibility of children
			if (event.type == "config" && event.property == "_metadata.visibility"){
				var visibility;
				//go through its subview, if the value is default, then..
				var subviewScope;
				var subviewScopes;
				for (var subviewId in this.context.subview){
					subviewScopes = this.context.subview[subviewId];
					for (var realViewId in subviewScopes){
						subviewScope = subviewScopes[realViewId];
						if (subviewScope.context.options && subviewScope.context.options._metadata && subviewScope.context.options._metadata.visibility.get('value') == "DEFAULT"){						
							subviewScope._change(event);
						}			
					}
				}				
			}		
		}
				
	    this.change = function(event) {
			// Default behavior.  Call view method if subview do not have any change logic
	    	this.view();
	    };	
		
	    this.collaboration = function(event) {
	        
			var node = dom.byId(event.domId);
			
			/* now engine guarantee that event.domId is the right one. So we don't need to have following logic here any more. 
			if(!node || !dom.isDescendant(node, this.context.element)) {
				// notice that sometimes event.domId (detected in editor side) is the id generated by dojo, that id may not exist in the dom node of this coachView in viewer side.
				// For those cases, use coachView's dom node instead.
				node = this.context.element;
			}
			*/
			
	    	if(event.type == "focus") {
				if(!this.originalOutline && !!node.style.outline && node.style.outline != "") {
					this.originalOutline = node.style.outline;
				}
				// node.style.border = "solid " + event.outlineColor;
				node.style.outline = "1px solid " + event.outlineColor;
				
				collaborationPopup.showCollaboratorInfo(node, event.color, event.bgColor, event.message, this.context.element);				
	    	} else if(event.type == "blur") {
	 			if(!!this.originalOutline) {
					node.style.outline = this.originalOutline;
					delete this.originalOutline;
				} else {
	   				node.style.outline = "none";
				}
	    		
	    	} else if(event.type == "mouseOn") {
				collaborationPopup.showCollaboratorInfo(node, event.color, event.bgColor, event.message, this.context.element);
	    	} else if(event.type == "mouseOff") {
				collaborationPopup.hideCollaboratorInfo(node);	    		
	    	} else if(event.type == "dataChange") {
				collaborationPopup.showDataChange(node, event.message, this.context.element);
	    	}    	
	    };
	    
		/**
		The default implementation provided by framework does nothing, it's each view's responsibility 
		to decide what to do when it receives the validation event.
		
		If validation errors occur during the process of boundary event, coach framework calls validate(event) 
		on all the involved views. When both parent and children views are involved, parent view is called 
		prior to children views. Framework guarantees that involved views are only called once per boundary event.
		
		Details about the input parameter event:		
		event.type can be either "error" or "clear".
		
		If event.type is "clear", event doesn't have any other property.		
		If event.type is "error", event has 2 more properties as following:
		
		- event.errors: list of errors. This view receives those errors because its data binding or part of data binding 
		  has validation errors. Each error object has 3 properties: binding, message and view_dom_ids.		
			a) binding is the path relatives to the data binding of current view.
			b) message is the localized message.
			c) view_dom_ids is the list of DOM id of the views that are either current view or the descendant views of current view and also receive the error message.		
		- errors_subcription: list of errors. This view receives those errors because it has called subscribeValidation() 
		  and some of its descendant views have validation errors on them. If the error of one view has been described 
		  in event.errors, it won't show in errors_subcription. Each error object has 2 properties: view_dom_id and messages.		
			a) view_dom_id is the DOM id of the view that contains the error.
			b) messages is a list of the localized messages.
		
		event example: 
		{		
		   type: "error",
		   errors: 
		           [
		            {
						binding: "ownerBirthday.year"
						message: "2030 is after current year of 2012",
						view_dom_ids: ["domId_PhoneBookView", "domId_yearView"]
		            }
		          ]
		   errors_subcription: 
		          [
		            {
						view_dom_id: "domId_advertisementView",
						messages: ["something wrong"]
		            }
		          ]
		}
		
		Since coach framework calls validate() on all involved views synchronously, the execution of this method 
		is expected to complete shortly.
		
		*/
	    this.validate = function(event) {
	    }
	    
		this._buildChildren = function() {
			var currentViewScope = this;
			var currentDiv = currentViewScope.context.element;
		
			var binding = currentViewScope.context.binding;
			var bindToArray = (!!binding && binding.get("value") instanceof com.ibm.bpm.coach.List);			

			// view could hold subviews directly, those direct subview won't repeat even if parent view binds with a list
			var directViewList = domHelper.shallowQueryByClass(currentDiv, engine.CONSTANTS.COACH_VIEW, true);
			var oneDirectView;
			array.forEach(directViewList, function(item, index) {
				oneDirectView = engine.initView.apply(engine.GlobalAssets, [item, currentViewScope]);
			});			
		
			// find the direct ContentBox under this view
			var immediateContentBoxList = domHelper.shallowQueryByClass(currentDiv, engine.CONSTANTS.CONTENT_BOX, true);
			
			if(bindToArray) {
				if(!!binding && binding.get("value").length() == 0) {
					// view binds an empty list, no repeat will occurs, so no subview will call _addRelatedList() to report.
					// we have to simulate to collection information for the multiple binding list support here.
					this._simulateRepeatingToDetectMultiLists(immediateContentBoxList);
				}

				// Register listener for List changes to watch if rtDataBinding is dirty. Do it for both view managed and framework managed. 
				var oneHandler = binding.get("value").bindAll(lang.hitch(currentViewScope, currentViewScope._watchRtDataBinding));
				this._bindingHandles["_watchRtDataBinding"] = oneHandler;
			}			
			
			// only care about the framework managed ContentBox. view managed ContentBox will be built by its parent view.
			var fwkManagedContentBoxs = new Array();
			for(var k=0; k<immediateContentBoxList.length; k++) {
				if(!engine.isViewManagedContentBox(immediateContentBoxList[k])) {
					// Framework managed
					fwkManagedContentBoxs[fwkManagedContentBoxs.length] = immediateContentBoxList[k];
				}
			}
			
			if(fwkManagedContentBoxs.length == 0) {
				// no Framework managed contentBox, so job is done
				return;
			}
			
			if(bindToArray) {
				// save the template in the viewScope
				this._contentBoxTemplates = fwkManagedContentBoxs;
				
				// keep the templates in the dom since we need them to indicate where to insert 
				// when dynamically adding from nothing (the list was empty before adding).			
				// Hide those templates by changing its css class from "ContentBox" to "ContentBoxTemplate" so they don't take the space
				for(var i=0; i<this._contentBoxTemplates.length; i++) {
					domClass.replace(this._contentBoxTemplates[i], engine.CONSTANTS.CONTENTBOX_TEMPLATE, engine.CONSTANTS.CONTENT_BOX);
				}
				if(!!binding) {
					// Register listener for List changes
					var bindingHandle = binding.get("value").bindAll(lang.hitch(currentViewScope, currentViewScope._listUpdated));
					this._bindingHandles["_listUpdate"] = bindingHandle;
					
					// need to repeat the framework managed contentBox based on the data-binding						
					for(var k=0; k<binding.get("value").length(); k++) {
						this._repeatContentBoxs(k, false);
					}
				}
			} else {
				for(var i=0; i<fwkManagedContentBoxs.length; i++) {
					engine.buildContentBox(currentViewScope, fwkManagedContentBoxs[i]);
				}
			}	
		}
		
		// when binding list is empty, or ViewContext.containDiffLoopingContext() is called before view is fully initialized,
		// have to simulate to collect enough information to decide if this view involves multiple lists
		// this._impactedByMultipleLists is assigned in the function
		this._simulateRepeatingToDetectMultiLists = function(immediateContentBoxList) {
			this._impactedByMultipleLists = false;

			if(multiBindingListsHelper.isBindingObjectList(this)) {
				if(!immediateContentBoxList) {
					immediateContentBoxList = domHelper.shallowQueryByClass(this.context.element, engine.CONSTANTS.CONTENT_BOX, true);			
				}
				
				for(var k=0; k<immediateContentBoxList.length; k++) {
					var oneDiv = immediateContentBoxList[k].cloneNode(true);						
					engine.adjustIdAndBinding(oneDiv, this, 0);		
					// this.context.element.parentNode.appendChild(oneDiv); // only for debug purpose
				}

		    	var listBindingStrArray = multiBindingListsHelper.analyzeRelatedListBindingStrs(this);
		    	this._impactedByMultipleLists = (listBindingStrArray.length > 0);
			}
		}
		
		// Repeat all the contentBox templates (of framework managed contentBoxs) for the specified bindingIndex.
		this._repeatContentBoxs = function(bindingIndex, callView) {
			for(var i=0; i<this._contentBoxTemplates.length; i++) {
				var templateNode = this._contentBoxTemplates[i];
				var oneDiv = templateNode.cloneNode(true);
				// ContentBox template's css class is "ContentBoxTemplate", change it to "ContentBox"						
				domClass.replace(oneDiv, engine.CONSTANTS.CONTENT_BOX, engine.CONSTANTS.CONTENTBOX_TEMPLATE);
				oneDiv.setAttribute(engine.CONSTANTS.TEMPLATE_ID, templateNode.id);

				// content boxes are not always directly under their CoachView's element
				templateNode.parentNode.insertBefore(oneDiv, templateNode);

				engine.adjustIdAndBinding(oneDiv, this, bindingIndex);
				var newViewList = new Array();
				engine.buildContentBox(this, oneDiv, newViewList);
				
				if(callView == true) {
					for(var k=0; k<newViewList.length; k++) {
						engine.callView(newViewList[k]);
					}
				}
			}
		}
		
		// If bindingIndex is specified, delete the framework managed contentBox instances according to the binding index
		// If bindingIndex is not specified, delete all framework managed contentBox instances
		this._deleteContentBoxs = function(bindingIndex) {
			var contentBoxGroup = this._findContentBoxGroup(bindingIndex);
			for(var k=0; k<contentBoxGroup.length; k++) {
				// delete the contentBox 
				engine.deleteContentBox(contentBoxGroup[k], this);
				// content boxes are not always directly under their CoachView's element
				contentBoxGroup[k].parentNode.removeChild(contentBoxGroup[k]);
			}
		}
		
		// refresh the framework managed contentBox instances according to specified binding index
		this._refreshContentBoxs = function(bindingIndex) {
			var contentBoxGroup = this._findContentBoxGroup(bindingIndex);
			for(var k=0; k<contentBoxGroup.length; k++) {
				// refresh the contentBox 
				engine._refreshContentBox(contentBoxGroup[k], this);
			}
		}
		
		this._watchRtDataBinding = function(changeEvent) {			
			if(changeEvent.removedFrom >= 0 && changeEvent.insertedInto >= 0 && changeEvent.removedFrom == changeEvent.insertedInto) {
				// event is "set", we don't care about it
				return;
			} else {
				if(changeEvent.removedFrom >= 0) { // one item has been removed from the list.	
					this._markRtDataBindingDirty(true);
					
					// notify validationHelper
					var dataBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
					
					validationHelper.listItemRemoved(dataBindingStr, changeEvent.removedFrom, changeEvent.oldVal);
				}
				
				if(changeEvent.insertedInto >= 0) {
					// when we support insert in the middle later, we need to uncomment following line
					// this._markRtDataBindingDirty(true);
				}	
			}	
	    };
		

	    this._listUpdated = function(changeEvent) {
	    	/*
			console.debug("list change detected, object: ", changeEvent.object, 
			"removedFrom: ",changeEvent.removedFrom, 
			"insertedInto: ",changeEvent.insertedInto);
			*/
			
			if(changeEvent.removedFrom >= 0 && changeEvent.insertedInto >= 0 && changeEvent.removedFrom == changeEvent.insertedInto) {
				this._refreshContentBoxs(changeEvent.removedFrom);
			} else {
				// if both changeEvent.removedFrom and changeEvent.insertedInto exist, execute both
				if(changeEvent.removedFrom >= 0) { // one item has been removed from the list.	
					this._deleteContentBoxs(changeEvent.removedFrom);
				}
				
				if(changeEvent.insertedInto >= 0 && this._contentBoxTemplates.length != 0) {
					// now we only support to add to the end of the list, figure out the proper position to insert when list supports to add item at any position.
					this._repeatContentBoxs(changeEvent.insertedInto, true);
				}	
			}	
	    };
	    
	    // If bindingIndex is specified, find one contentBox instance for each template based on the bindingIndex.
	    // If bindingIndex is NOT specified, find all the contentBox instances for all the contentBox templates owned by this CoachView
	    this._findContentBoxGroup = function(bindingIndex) {
 			var contentBoxGroup = new Array();
	    	var rootNode = this.context.element;
			for(var i=0; i<this._contentBoxTemplates.length; i++) {
				// find the contentBox who was created using current template
				var contentBoxList = rootNode.querySelectorAll(' .' + engine.CONSTANTS.CONTENT_BOX + '[' + engine.CONSTANTS.TEMPLATE_ID + '=' + this._contentBoxTemplates[i].id + ']'); 
				if(bindingIndex == undefined) {
					// all instances qualify
					for(var k=0; k<contentBoxList.length; k++) {
						contentBoxGroup.push(contentBoxList[k]);
					}
				} else if(contentBoxList.length > bindingIndex) {
					contentBoxGroup.push(contentBoxList[bindingIndex]);
				}
			}
			return contentBoxGroup;
	    }
	    
	    this._addRelatedList = function(viewDomId, relatedList) {
	    	this._domIdToBindingListMap[viewDomId] = relatedList;
	    }
	    
	    this._monitorRelatedListsIfNeeded = function() {
	    	this._impactedByMultipleLists = false;
	    	
			if(!multiBindingListsHelper.isBindingObjectList(this)) {
	    		return;
			}

			var myDataBindingObject = this.context.binding.get("value");			
	    	var listBindingStrArray = multiBindingListsHelper.analyzeRelatedListBindingStrs(this);
	    	if(listBindingStrArray.length > 0) {
	    		this._impactedByMultipleLists = true;
	    		var listStr;
				for(var k=0; k<listBindingStrArray.length; k++) {
					listStr = listBindingStrArray[k];
	    			if(!this._bindingStrToListMap[listStr]) {
	    				var resolvedList = engine.resolveBinding.call(engine.GlobalAssets, listStr).get("value");
	    				this._bindingStrToListMap[listStr] = resolvedList;
	    				
						var bindingHandle = resolvedList.bindAll(lang.hitch(this, this._monitorListsLengthChange));
						this._bindingHandles["_monitorLengthOf_" + listStr] = bindingHandle;
	    			}
				}
				this._bindingHandles["_monitorLengthOf_myOwnBinding"] = myDataBindingObject.bindAll(lang.hitch(this, this._monitorListsLengthChange));		    		
	    		multiBindingListsHelper.processMonitoredLists(this, engine);
	    	}
	    }
	    
	    this._monitorListsLengthChange = function(changeEvent) {
	    	// we only care the length change, so either add or delete, not set
	    	if(changeEvent.removedFrom != changeEvent.insertedInto) {
	    		multiBindingListsHelper.processMonitoredLists(this, engine);
	    	}
	    }
	    
	    this._checkIfImpactedByMultipleLists = function() {
			if(this._impactedByMultipleLists == undefined) {
			
				// this._impactedByMultipleLists is assigned in _simulateRepeatingToDetectMultiLists() or _monitorRelatedListsIfNeeded()
				// this._impactedByMultipleLists is still undefined, means that _monitorRelatedListsIfNeeded() is not called. So this view is not fully rendered yet.
				// As the result, maybe this view has some List binding that has not be discovered yet. 
				// So, have to trigger framework to evaluate the contentBox templates of this view to discover all the related List bindings.
				
				// have to simulate to collection information for the multiple binding list support here.
				this._simulateRepeatingToDetectMultiLists();
			}
			
			return this._impactedByMultipleLists;
	    }
	    
	    this._isReady = function() {
	    	return !!this.context;
	    }
	    
		this._getTopLevelView = function() {
			var aView = this;
			while(!!aView.context.parentView()) {
				aView = aView.context.parentView();
			}
			return aView;
		}
	    
		this._markRtDataBindingDirty = function(value) {
			var topLevelParentView = this._getTopLevelView();
			topLevelParentView._rtBindingDirty = value;		
		}
		
		this._isRtDataBindingDirty = function() {
			var topLevelParentView = this._getTopLevelView();
			return !!topLevelParentView._rtBindingDirty;		
		}

		this._getBindingStrings = function() {
			var bindingStrArray = new Array();
			// make sure rtDataBinding is up-to-date before we call it	
			var dataBindingStr = dataBindingHelper.rtDataBinding._get(this.context.element);
			bindingStrArray.push(dataBindingStr);
					
			return bindingStrArray;
		}

	    	    
	//##Data Binding start
	    this._changeObject = lang.hitch(this, function(property, oldVal, newVal) {
	    	//Change handler for Stateful objects, calls the subclass change handler
	    	var event = {property: property, oldVal: oldVal, newVal: newVal, removedFrom: null, insertedInto: null};
	    	this._change(event);
	    });
	//##Data Binding -end
	}
});

