/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

define(["require", "dojo/_base/array", "dojo/_base/connect", "dojo/dom-class", "./logger", "./validationHelper", "./coachViewHelper", "dojo/_base/config"], function(require, array, connect, domClass, logger, validationHelper, coachViewHelper, dojoConfig) {

	var ViewContext = function () {	
		this._init = function(ownerView, system, element, binding, option, readonly) {
			// Establish the context object
			this.element = element;
			this.binding = binding;
			this.options = option;
			this.readonly = readonly;
			this.subview = {};
			this.bpm = {"system": system};
			this._parent = ownerView;
			this._parentView = null;
			this._validationSubscribed = false;
			this.contextRootMap = dojoConfig.App._bpmContextRootMap;
		};
	}		
	
	// do we really need this? investigate later
	ViewContext.prototype.constructor = ViewContext;

	/**
	 * summary:
	 *		escape the text to clean xss attacks
	 *		using .createTextNode to utilize the browser escaping capabilities
	 * txt: the string to escape
	 */
	ViewContext.prototype.htmlEscape = function (txt) {
		var element = document.createElement('span');
		var content = document.createTextNode(txt);
		element.appendChild(content);
		return element.innerHTML;
	};
	
	ViewContext.prototype.parentView = function(){
		return this._parentView;
	};
	
	ViewContext.prototype.triggerCollaboration = function(payload) {
        var element = this.element;
        require(["./controller"], function(controller) {
            controller.customCollaborationHandler(element.getAttribute("id"), payload);
        });
	};
	
	ViewContext.prototype.getInheritedVisibility = function(){
		//find its parent and get parent's visibility
		var visibility = "EDITABLE";
		var parentScope = this.parentView();
		
		if (!parentScope || !parentScope.context || !parentScope.context.options || !parentScope.context.options._metadata)
		      return visibility;
			  
		//if it is null, goes up
		visibility = parentScope.context.options._metadata.visibility.get("value");
		if (visibility == "DEFAULT"){
			return parentScope.context.getInheritedVisibility();
		}
		return visibility;
	};
	
	ViewContext.prototype.bindingType = function() {
		return this.element.getAttribute("data-bindingType");
	};
			
	//Private function which will be used by the framework to remove the old binding and bind to the new object
	ViewContext.prototype._updateBinding = function (bindingStr) {
		var engine = this._getEngine();

		if(!bindingStr) {
			bindingStr = engine.rtDataBinding._get(this.element);
		}
		
		if (!bindingStr){
			return;
		}
				
		//cancel the callbacks on the old binding		
		if(this.binding) {
			var handle = this._parent._bindingHandles[this.binding.property];
			if (!!handle && !!handle.unbind){
				handle.unbind();
			}
		}
		
		//update binding to the new object
		this.binding = engine.resolveBinding.call(engine.GlobalAssets, bindingStr);
		
	};
	
	ViewContext.prototype.trigger = function(callbackfn) {
		var _this = this;
		setTimeout(function() {
			var bdEventId = _this.element.getAttribute('data-eventid');
			if (!!bdEventId) {
				var engine = _this._getEngine();
				var boundaryEventInfo = engine.getBoundaryEventInfo(bdEventId);
				var validateBoundaryEventInfo = engine.getValidateBoundaryEventInfo();
				connect.publish("com.ibm.bpm.coach.BoundaryEventTriggered", [{
					boundaryEventInfo: boundaryEventInfo, validateBoundaryEventInfo: validateBoundaryEventInfo, callback:callbackfn
				 }]);
			}
		}, 1);
	};
	
	// Public API for view developer.
	/*
	If domNode is the node of any view, a single instance of view will be created and returned.
	If domNode is the node of contextBox, framework creates view instances for all the views owned 
	by this contentBox, and recursively create views for all the framework managed contentBoxs 
	that are owned by this contentBox. An array of view instance will be returned.
	*/
	ViewContext.prototype.createView = function(divNode, index, parentView) {
		logger.log("CoachView", "context.createView", "divNode: " + divNode.id + ", index: " + index + ", parentView: " + parentView);

		/* parentViewScope may not be this._parent, for example,
				viewX
					viewB
						ViewA
							CB_1 (fwk managed)
								ViewC
								CB_2 (view managed, constructed by ViewB)
									ViewD
								CB_3 (fwk managed)
									ViewE
		
		CB_2 is constructed by ViewB, but viewB is not the parentViewScope of the subview (ViewD) in CB_2
		Parent of viewD is View A.

        Parent-child relationship is based on dom structure. ie, your most immediate outer view scope is your parent.
		vice versa: your immediate inner view scope is your child
		
		But at creation time, the view scope thats call viewscope.context.createView  may not be the parent of the view to be created
        This happens to view managed content box		
		
		*/	
		
		var engine = this._getEngine();
		var parentViewScope = (!!(parentView)) ? parentView : coachViewHelper.findParentViewScope(this._parent, divNode);
		
		if(!!index || index == 0) {
			engine.adjustIdAndBinding(divNode, parentViewScope, index);
		}
		
		if(engine.isContentBox(divNode)) {
			var newViewList = new Array();			
			engine.buildContentBox(parentViewScope, divNode, newViewList);
			
			for(var k=0; k<newViewList.length; k++) {
				engine.callView(newViewList[k]);
			}
			
						
			// check if it's needed to deliver the validation event to the newly created views
			var foundNewValidationEvent = false;
			for(var k=0; k<newViewList.length; k++) {
				foundNewValidationEvent = foundNewValidationEvent || validationHelper.validateView(newViewList[k]);
			}
			if(foundNewValidationEvent) {
				// the new view or its descendant views own validation event
				// just tell engine to deliver the new validation event. It's up to engine to decide when to deliver
				engine.deliverValidationEvents();
			}

			return newViewList;			
		} else {
			// this divNode is expected to be a view
	    	var viewScope = engine.initView.apply(engine.GlobalAssets, [divNode, parentViewScope]);
			engine.callView(viewScope);
			
			// check if it's needed to deliver the validation event to the newly created views
			var foundNewValidationEvent = validationHelper.validateView(viewScope);
			if(foundNewValidationEvent) {
				// the new view or its descendant views own validation event
				// just tell engine to deliver the new validation event. It's up to engine to decide when to deliver
				engine.deliverValidationEvents();
			}
			
			return viewScope;
		}				
	};
	
	// Public API for view developer. 		
	ViewContext.prototype.deleteView = function(divNode) {
		logger.log("CoachView", "context.deleteView", "divNode: " + divNode.id);
		if(!divNode) 	return;
		
		var engine = this._getEngine();
		if(engine.isContentBox(divNode)) {
			engine.deleteContentBox(divNode, this._parent);				
		} else {
			// this divNode is expected to be a view
			engine.deleteView(divNode, this._parent);				
		}
	};
	
	ViewContext.prototype.setDisplay = function(isDisplay, domNode) {
		logger.log("CoachView", "context.setDisplay", "domNode: " + domNode + ", isDisplay: " + isDisplay);
		var engine = this._getEngine();
		var domNode1 = (!!domNode)?domNode:this.element;
    	if (!isDisplay) {
    		domClass.add(domNode1, engine.CONSTANTS.COACH_VIEW_HIDDEN);
    	} else {
    		domClass.remove(domNode1, engine.CONSTANTS.COACH_VIEW_HIDDEN);
    	}
	};
	
	ViewContext.prototype.log = function(method, message) {
		var engine = this._getEngine();
		var viewName = engine.CONSTANTS.COACH_VIEW;
		var dataType = this.element.getAttribute(engine.CONSTANTS.VIEW_DATA_TYPE);
		if (!!dataType) {
			dataType = dataType.split(".");
			if (!!dataType) {
				viewName = dataType.pop();
			}
		}
		logger.log(viewName, method, message);
	};		
	
	// sortResult is optional, this API returns unsorted array unless sortResult is set to true
	ViewContext.prototype.getSubview = function(viewId, sortResult) {
		logger.log("CoachView", "context.getSubview", "viewId: " + viewId + ", sortResult: " + sortResult);
		var subviewsArray = new Array();
		var subviewsObject = this.subview[viewId];
		for(var divId in subviewsObject) {
			if(!!subviewsObject[divId]) { // there might be "hole" in subviewsObject as the result of deleteView
				subviewsArray.push(subviewsObject[divId]);
			}
		}
		
		if(sortResult && subviewsArray.length >= 2) {
			var testNode = subviewsArray[0].context.element;
			if(!!testNode.sourceIndex) {
				subviewsArray.sort(function (viewScopeA, viewScopeB) {
					return viewScopeA.context.element.sourceIndex - viewScopeB.context.element.sourceIndex;
				});
			}else if(!!testNode.compareDocumentPosition) {
				subviewsArray.sort(function (viewScopeA, viewScopeB) {
					var nodeA = viewScopeA.context.element;
					var nodeB = viewScopeB.context.element;
					return 3 - (nodeA.compareDocumentPosition(nodeB) & 6);
				});
			}				
		}
		
		return subviewsArray;
	}

	ViewContext.prototype.refreshView = function(){
		logger.log("CoachView", "context.refreshView", "is called");
		var engine = this._getEngine();
		engine._refreshView(this._parent);
	};
	
	/**
	Calls this API to indicate that it is interested in any validation errors that happen on its descendant views.
	When validation error occurs, validate() method of this view is called prior to that of descendant views.
	
	Views that have data binding are titled to validation event implicitly, namely their validate() method 
	are called when validation event occurs (either "clear" type or "error" type). 
	For the views that don't have data binding, e.g. Tab view, if those views are interested in 
	any validation events that happen on its descendant views, those views have to call this.context.subscribeValidation() 
	to register explicitly.
		
	A view who has data binding can also call this.context.subscribeValidation() to receive the validation events of 
	its descendant views, beside of its own validation event. In this case, after the view calls 
	this.context.unsubscribeValidation(), it can still receive its own validation event.	
	*/
	ViewContext.prototype.subscribeValidation = function(){
		logger.log("CoachView", "context.subscribeValidation", "is called");
		this._validationSubscribed = true;
	};
		
	/**
	Calls this API to unsubscribe.
	Nothing happens if subscribeValidation() was not called previously.
	*/
	ViewContext.prototype.unsubscribeValidation = function(){
		logger.log("CoachView", "context.unsubscribeValidation", "is called");
		this._validationSubscribed = false;
	};
	
	/**
	 * Return true if the view has chance to fall into the troubled situation about different lengths of lists, 
	 * otherwise return false.
	 * For example, table view calls context.containDiffLoopingContext() to determine if table needs to disable its 
	 * add/delete capability during the runtime even if customer has specified to enable add/delete through configuration.
	 * 
	 * Example scenario of different length of lists: Let's say we have following View structure -
	 * 			   TableView (bind to ListA[])
	 * 			       ContentBox
	 * 			           TextView (bind to ListB.currentItem)
	 * 
	 * Case 1: When the length of ListA is shorter than that of ListB, some items in ListB won't have chance to be displayed. 
	 * Case 2: When the length of ListA is longer than that of ListB, some instances of TextView won't have valid data binding. 
	 * Case 3: when he length of ListA and ListB are different, TableView has difficulty to support add/delete. 
	 * 
	 * Above 3 cases are what we mean by "troubled situation about different lengths of lists".
	 * Coach framework handles case 1 and case 2 automatically by showing warning message once the length of involved lists 
	 * are different.
	 * 
	 * TableView can call context.containDiffLoopingContext() to handle case 3. Based on TableView's structure and 
	 * binding information, the return of containDiffLoopingContext() is true. So TableView can disable its 
	 * add and delete capacities accordingly.
	 */
	ViewContext.prototype.containDiffLoopingContext = function(){
		logger.log("CoachView", "context.containDiffLoopingContext", "is called");
		return this._parent._checkIfImpactedByMultipleLists();
	};	
	
	ViewContext.prototype.getOptions = function(divNode) {
		var engine = this._getEngine();
		var tempViewCfgBinding = divNode.getAttribute(engine.CONSTANTS.DATA_CONFIG);
		
		//config
		var options = engine.GlobalAssets.options[tempViewCfgBinding]; 
		if (!!options) {
			for(var optionKey in options) {	
				if (optionKey == "_metadata"){
					for (var metadataKey in options[optionKey]){
						engine._resolveAndRegisterOptionBindingCallBack(null, options, metadataKey, true);
					}
				}
				else{			
					engine._resolveAndRegisterOptionBindingCallBack(null, options, optionKey, false);
				}
			}
		}		
		
		return options;
	}
	
	// private API
	// get the value of an option
	ViewContext.prototype._getOptionValue = function(configId, optionKey, isMetadata) {
		logger.log("CoachView", "context._getOptionValue", "is called");
		var engine = this._getEngine();
		return engine.getOptionValue(configId, optionKey, isMetadata);
	}

	ViewContext.prototype._getEngine = function(){
		// we know engine module has been loaded.
		return require("./engine");
	};
				
	return ViewContext;

});