/*
Licensed Materials - Property of IBM
5725-C95
 Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// Contains all utilities that Coach runtime supports validation

define(["require", "./CONSTANTS"], function(require, CONSTANTS) {
				
	var dataBindingHelper = new Object();
	
	// rtDataBinding provides 3 helper method for resolving List related data-binding during runtime.
	dataBindingHelper.rtDataBinding = new Object();
	dataBindingHelper.rtDataBinding._get = function(viewElement) {
		var rtBinding = viewElement.getAttribute(CONSTANTS.DATA_BINDING_RT);
		if(!rtBinding) {
			return viewElement.getAttribute(CONSTANTS.DATA_BINDING);
		} else {
			return rtBinding;
		}
	}	
	dataBindingHelper.rtDataBinding._set = function(viewElement, bindingValue) {
		viewElement.setAttribute(CONSTANTS.DATA_BINDING_RT, bindingValue);
	}
	dataBindingHelper.rtDataBinding._delete = function(viewElement) {	
		// viewElement.removeAttribute(CONSTANTS.DATA_BINDING_RT);
	}
	
	dataBindingHelper.resolveBinding = function(bindingStr) {
		var engine = dataBindingHelper._getEngine();
		return engine.resolveBinding.call(engine.GlobalAssets, bindingStr);
	}


	dataBindingHelper.collectBpmIds = function(bindingStr) {
		var bpmIds = null;
		if(bindingStr.indexOf("[") == -1) {
			return bpmIds;
		}
		var pathArray = bindingStr.split(".");
		var tempBindingStr = null;
		for (var i=0; i<pathArray.length; i++) {
			var aPath = pathArray[i];
			var leftIndexPosition = aPath.indexOf("[");
			if (leftIndexPosition != -1) {
				var rightIndexPosition = aPath.indexOf("]");
				if(rightIndexPosition - leftIndexPosition > 1) {
					// encounters abc[n], need to find out bpm_id from the index (n)
					var indexValue = aPath.substring(leftIndexPosition+1, rightIndexPosition);
					var listShortName = aPath.substring(0, leftIndexPosition);
					var listBindingStr = !tempBindingStr ? listShortName : tempBindingStr + "." + listShortName;
					var resolvedBinding = dataBindingHelper.resolveBinding(listBindingStr);
					if(!resolvedBinding) {
						// can't resolve this binding. The rest of bindings won't be resolved for sure. So stop here.
						break;
					}
					var list = resolvedBinding.get("value");
					var bpmId = list.indexToId[indexValue];
					if(!bpmIds) {
						bpmIds = new Array();
					}
					bpmIds.push(bpmId);
				}
			}
			
			if(!tempBindingStr) {
				tempBindingStr = aPath;
			} else {
				tempBindingStr += "." + aPath;
			}
		}
		
		return bpmIds;
	}

	// update RTDataBinding for specified view and its all descendants views.
	dataBindingHelper.updateAllRtDataBinding = function(viewScope, bindingStrToListMap) {
    	if(!!viewScope._bpmIds) {
    		if(!bindingStrToListMap) {
    			bindingStrToListMap = {};
    		}
    		var updatedBindingStr = viewScope.context.element.getAttribute(CONSTANTS.DATA_BINDING);
			var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
    		for(var k=0; k<viewScope._bpmIds.length; k++) {
    			var position = updatedBindingStr.indexOf(indexWithBracket);
    			var listBindingStr = updatedBindingStr.substring(0, position);
				var list = bindingStrToListMap[listBindingStr];
				if(!list) {
					list = dataBindingHelper.resolveBinding(listBindingStr).get("value");
					bindingStrToListMap[listBindingStr] = list;
				}
				var currentIndex = list.idToIndex(viewScope._bpmIds[k]);
    			var replaceValue = "[" + currentIndex + "]";
    			updatedBindingStr = updatedBindingStr.replace(indexWithBracket, replaceValue);
    		}
    		
    		dataBindingHelper.rtDataBinding._set(viewScope.context.element, updatedBindingStr);
    	}
		
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					dataBindingHelper.updateAllRtDataBinding(subviews[divId], bindingStrToListMap);
				}
			}		
		}			
	}
		
	// it's array if end with "[]", otherwise resolve the binding to check if it is type of com.ibm.bpm.coach.List
	dataBindingHelper.isBindingArray = function(bindingStr) {
		var refArray = bindingStr.split(".");
		if(refArray[0] == "local") {
			return (bindingStr.substring(bindingStr.length - 2) == "[]");
		} else {
			// could be config option
			var binding = dataBindingHelper.resolveBinding(bindingStr);
			if(!!binding && binding.get("value") instanceof com.ibm.bpm.coach.List) {
				return true;
			}
		}
			
		return false;
	}	
	
	dataBindingHelper.isNumber = function(stringValue) {
		var convertToNumber = Number(stringValue);
		return !isNaN(convertToNumber);
	}
	
	dataBindingHelper.getNumberOfIndex = function(bindingStr) {
		var pathArray = bindingStr.split(".");
		var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
		var numOfIndex = 0;
		for(var k=0; k<pathArray.length; k++) {
			if(pathArray[k].indexOf(indexWithBracket) != -1) {
				numOfIndex++;
			}			
		}		
		return numOfIndex;
	}
	
	dataBindingHelper.updateBindingStr = function(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating, numOfIndex) {
		if(!numOfIndex) {
			numOfIndex = dataBindingHelper.getNumberOfIndex(bindingStr);
		}
		
		if(numOfIndex >= maxNumOfUnresolvedRepeating) {
			var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
			var replaceValue = "[" + bindingIndex + "]";
			var newBinding;
			if(maxNumOfUnresolvedRepeating == 1 && numOfIndex > 1) {
				// assign all the outstanding "[index]" with current bindingIndex since this is the last repeating index
				newBinding = bindingStr;
				for(var k=0; k<numOfIndex; k++) {
					newBinding =newBinding.replace(indexWithBracket, replaceValue);
				}
			} else {
				newBinding = bindingStr.replace(indexWithBracket, replaceValue); // replace very first "[index]"
			}
			
			return newBinding; 
		} else {
			// now numOfIndex < maxNumOfUnresolvedRepeating. the unresolved "[index]" should be reserved for the following repeating
			return bindingStr;
		}				
	
	}
	
	/*
	 Check if bindingStrA starts with bindingStrB in term of path.
	 If the data represented by bindingStrA is sub-object of the data represented by bindingStrB, return true. Otherwise false.
	 Note: if bindingStrA == bindingStrB, the return is false
	 
	 Example:
	 	bindingStrA			bindingStrB		return
	 	-----------------------------------------------
	 	local.abc			local.abc		false
	 	*					null/undefined	false;
	 	null/undefined		*				false;
	 	local.def			local.abc		false
	 	local.abcdef		local.abc		false
	 	local.abc.def		local.abc		true
	 	local.abc.d.e		local.abc		true
	 	local.abc[2]		local.abc		true
	*/
	dataBindingHelper.startsWithPath = function(bindingStrA, bindingStrB) {
		if(bindingStrA != bindingStrB && !!bindingStrA && !!bindingStrB && (bindingStrA.indexOf(bindingStrB) == 0)) {
			// still need fine check, because ("local.abcdef", "local.abc") also passes above test
			var difference = bindingStrA.replace(bindingStrB, "");
			// difference is not blank string because bindingStrA != bindingStrB
			var firstCharacter = difference.substring(0, 1);
			
			if(firstCharacter == "." || firstCharacter == "[") {
				//("local.abc.def", "local.abc"), or ("local.abc[2]", "local.abc")
				return true;
			}
		}
		
		return false;
	}	

	// consider refactoring the source code, move data binding functions (e.g. engine.resolveBinding) out from engine. then we don't need engine here
	dataBindingHelper._getEngine = function(){
		// we know engine module has been loaded.
		return require("./engine");
	};
				
		
	return dataBindingHelper;
});
	