/*
Licensed Materials - Property of IBM
5725-C95
� Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

// coach runtime engine main js

define(["dojo/parser", "dojo/_base/json", "dojo/_base/lang", "dojo/json", "dojo/_base/array", "dojo/query", "dojo/dom", "dojo/dom-class", "dojo/_base/xhr", "dojo/_base/connect", "./CONSTANTS", "./commonHelper", "./domHelper", "./dataBindingHelper", "./validationHelper", "./coachViewHelper", "./Stateful", "./Shimmer", "./logger", "./ViewContext", "./templateStore"], function(parser, json, lang, json1, array, query, dom, domClass, xhr, connect, CONSTANTS, commonHelper, domHelper, dataBindingHelper, validationHelper, coachViewHelper, stateful, Shimmer, logger, ViewContext, templateStore) {
	var engine = new Object();

	// Main modules for the framework starts
	
	// backward compatibility support
	engine.CONSTANTS = CONSTANTS;
	
	engine.GlobalAssets = {"topLevelViewScope": {}, "scopeClasses": {}, "watchHandles": [], "local" : {}, "epv": {},
					  "system": {}, "systemData": {}, "resource": {}, "options": {}, boundaryEventData: {} };
	
	engine.localizedMessages = {};
	
	engine._isCoachUnloaded = false;
	
	engine.isProcessingValidation = false;
	
	engine.initLocalizedMessage = function() {
		commonHelper.timestamp("Enter engine.initLocalizedMessage()");
		logger.log("engine", "initLocalizedMessage", "is called", logger.INFO);
		var jsonResURL = "jsonResBundleLoader?bundle=JavaScriptMessages";
		
		xhr.get( {
		    url: jsonResURL,
		    handleAs: "json",
		    load: function(responseObject, ioArgs) {
		    	engine.localizedMessages = responseObject;		
				logger.log("engine", "initLocalizedMessage", "localizedMessages: " + engine.localizedMessages);
		    }
		});
	};

    // Loads localized messages in synchronous fashion if only they are not yet initialized.
    // This function is required to avoid timing issues in collaboration scenarios.
    engine.forceLocalizedMessages = function() {
        logger.log("engine", "forceLocalizedMessages", "is called", logger.INFO);
        var jsonResURL = "jsonResBundleLoader?bundle=JavaScriptMessages";
        var initialized = false;
        // check if messages are empty
        for (var prop in engine.localizedMessages) {
            if (engine.localizedMessages.hasOwnProperty(prop)) {
                initialized = true;
                break;
            }
        }
        // if messages are empty we need to pull them in a sync way
        if (initialized === false) {
            xhr.get( {
                url: jsonResURL,
                handleAs: "json",
                sync: true,
                load: function(responseObject, ioArgs) {
                    engine.localizedMessages = responseObject;
                    logger.log("engine", "forceLocalizedMessages", "localizedMessages: " + engine.localizedMessages);
                }
            });
        }
    }

	/**Refresh all the views in the coach
	 *@param oldValue optional
	 *@param newValue optional
	*/
	engine.refreshViews = function(oldValue){
		var bindingStrToListMap = new Object();
		for (var viewId in engine.GlobalAssets.topLevelViewScope) {
			var viewScope = engine.GlobalAssets.topLevelViewScope[viewId];
			// update rtDataBinding here is more efficient since bindingStrToListMap can be shared among the topLevelViews
			if(viewScope._isRtDataBindingDirty()) {
				dataBindingHelper.updateAllRtDataBinding(viewScope, bindingStrToListMap);
				viewScope._markRtDataBindingDirty(false);
			}
			if (!!oldValue){
				engine._refreshView(viewScope, oldValue._objectPath);
			}
			else{
				engine._refreshView(viewScope);
			}
		}
	};

	//Internal helper function for refresh one view and its descendants
	//objectPath optional parameter: only used when this function is invoked by a set method of Stateful object
	//This method is also used by CoachView.context.refreshView and engine._refreshContentBox
	// NOTE: make sure rtDataBinding is update to date before calling this function.
	engine._refreshView = function(viewScope, objectPath){
	    
	    var bindingStr = dataBindingHelper.rtDataBinding._get(viewScope.context.element);
		
		if(bindingStr.indexOf ("[]") > -1){
		  bindingStr = bindingStr.substr(0, bindingStr.length - 2);
		}
		
		/*1. update binding only if it matters
		*   objectPath                                          bindingStr                      affected
		*   local.phoneBook.publishData         local.phoneBook.publishData.prop.prop1               y
		*   local.phoneBook.person[1].birthdate local.phoneBook.person[1].birthdate.day              y    
		*   local.phoneBook.publishData         local.dataObject01                                   no
		*   local.phoneBook.publishData         local.phoneBook.publishData                          no
		*   undefined                           *                                                    y
		*/
		var affected = !objectPath || dataBindingHelper.startsWithPath(bindingStr, objectPath);
		var dataBinding;
		var oldDataBinding = viewScope.context.binding;
		//var oldValue = viewScope.context.binding.get("value");
		if (affected){				
				viewScope.context._updateBinding();	
	            //register call back on the new binding object
				var handleToCallBack;
				dataBinding = viewScope.context.binding;
				if (!!dataBinding && !!dataBinding.boundObject) {
					if (dataBinding.boundObject instanceof com.ibm.bpm.coach.Stateful || dataBinding.boundObject instanceof com.ibm.bpm.coach.List ) {
						//register the callback on the (boundObject)
						handleToCallBack = dataBinding.boundObject.registerCallback(dataBinding.property, viewScope._changeObject);					
						viewScope._bindingHandles[dataBinding.property]	= handleToCallBack;
					}					
				}			
										
		}
		
		//update configuration
		engine.resolveAndRegisterOptionsBindingCallBack(viewScope, true, objectPath);	
			
		//2. recursively goes to children to update children's binding if affected
		
		//if the view bind to a list and if the list is not the object set (if the list is set, then the this._change method will be called)
		if(!!oldDataBinding && oldDataBinding.get("value") instanceof com.ibm.bpm.coach.List){
			if (objectPath != oldDataBinding.boundObject._objectPath + "." + oldDataBinding.property){
			//if (!objectPath || (objectPath.indexOf(bindingStr) > -1) && (objectPath != bindingStr)){
				var newDatabinding = !!dataBinding ? dataBinding : oldDataBinding;
				engine.refreshRepeatableView(viewScope, oldDataBinding.boundObject[oldDataBinding.property] , newDatabinding.get("value"), objectPath);
			}
		}
		else{ // otherwise, recursively go down to refresh nested view
			var subViewScope;
			for (var viewId in viewScope.context.subview) {		    
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					subViewScope = subviews[divId];
					engine._refreshView(subViewScope, objectPath);
				}
			}
		}
		
		//3. Call this.change
		if(affected){
			var propertyName = null;
			if(!!dataBinding) {
				propertyName = dataBinding.property;
			} else if(!!oldDataBinding) {
				propertyName = oldDataBinding.property;
			}
			
			if(!!propertyName) {
				var oldValue = null;
				var newValue = null;
				
				if(!!oldDataBinding && typeof oldDataBinding.boundObject[propertyName] !== "undefined") {
					oldValue = oldDataBinding.boundObject[propertyName];
				}
				if(!!dataBinding && typeof dataBinding.boundObject[propertyName] !== "undefined") {
					newValue = dataBinding.boundObject[propertyName];
				}
				
				if(oldValue != newValue) {
					var event = {property: propertyName, oldVal: oldValue, newVal: newValue, removedFrom: null, insertedInto: null};
					viewScope._change(event);			   
				}
			}
 		}
		engine.notifyDataConfigBindingChange(viewScope);
	};

	/**
	 * This function is used to refresh a view whose binding is an array/list
	 */
	engine.refreshRepeatableView = function(viewScope, oldValue, newValue, objectPath){
			var topLevelParentView = viewScope._getTopLevelView();
			if(topLevelParentView._isRtDataBindingDirty()) {
				dataBindingHelper.updateAllRtDataBinding(topLevelParentView);
				topLevelParentView._markRtDataBindingDirty(false);
			}
			
	    	var subviewScope;
	    	
			/* The view could contains:
			 * framework managed content box OR
			 * view-managed content box. Note that: the content boxes are either all fwk-managed or all view-managed, not both
			 * views: during creation time, framework creates the viewScope, but does not repeat them
			 */
			
			//case:length equal, refresh sub views, regardless view managed or framework managed
			if(!!oldValue && oldValue.items && newValue && !!newValue.items && oldValue.items.length == newValue.items.length) {					
               	
				//do not use _refreshContentBoxs here, because that function only handle framework managed content box
				for (var viewId in viewScope.context.subview) {		    
					var subviews = viewScope.context.subview[viewId];
					for(var divId in subviews) {
						subViewScope = subviews[divId];						
						engine._refreshView(subViewScope, objectPath);
					}
				}
			}
			else{//length not equal				
				//delete all the content boxes: framework managed				
				viewScope._deleteContentBoxs();

				//refresh direct views
				for (var viewId in viewScope.context.subview) {		    
					var subviews = viewScope.context.subview[viewId];
					for(var divId in subviews) {
						subViewScope = subviews[divId];						
						engine._refreshView(subViewScope, objectPath);
					}
				}
				
				//repeat all the content boxes: framework managed		
				if (!!newValue && !!newValue.items){
					for(var i=0; i<newValue.items.length; i++) {
						viewScope._repeatContentBoxs(i, true);
					}	
				}
			}
	    };
    /*
	 *  Refresh view END
	 */
	
	/* refresh the contentBox (essentially refresh all the "framework managed" views inside this contentBox)

	Example: all ContentBox here are framework managed
	ViewX
		CB
			ViewA
			ViewB
			CB1
				CB11
					ViewC
					ViewD
				CB12
					ViewE
			CB2
				CB21
					ViewF
				CB22
					ViewG
	
	1. refresh CB means that ViewA - ViewG need to be refreshed
	2. ViewX is the parentViewScope of ViewA - ViewG		
	*/
    engine._refreshContentBox = function(contentBoxDiv, parentViewScope) {
		var immediateViewList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.COACH_VIEW);			
		array.forEach(immediateViewList, function(item, index) {
			var viewScope = engine._getViewScope(parentViewScope, item);
			viewScope.context.refreshView();
		});
		
		// find the direct ContentBox under this contentBox.
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.CONTENT_BOX);
		// handle all ContentBox, because view author doesn't have chance to refresh the view managed contentBoxs		
		for(var k=0; k<immediateContentBoxList.length; k++) {
			engine._refreshContentBox(immediateContentBoxList[k], parentViewScope);
		}
    }
    
    engine._getViewScope = function(parentViewScope, viewDomNode) {
		var viewDivId = viewDomNode.getAttribute("id");
		var viewId = viewDomNode.getAttribute(engine.CONSTANTS.VIEW_ID);
    	return parentViewScope.context.subview[viewId][viewDivId];
    }	    
	    
	
	engine.initViews = function() {
		commonHelper.timestamp("Enter engine.initViews()");
		logger.log("engine", "initViews", "is called", logger.INFO);
		
		// find out all top level views
		var topLevelViewList = query('#mainBody .' + engine.CONSTANTS.TOP_LEVEL);
		array.forEach(topLevelViewList, function(item, index) {
			var topLevelviewId = item.getAttribute(engine.CONSTANTS.VIEW_ID);
			commonHelper.timestamp("Call engine.initView() for top level view " + topLevelviewId);
			var oneTopLevelView = engine.initView.apply(this, [item, null]);
			commonHelper.timestamp("Finish engine.initView() for top level view " + topLevelviewId);
			this.topLevelViewScope[topLevelviewId] = oneTopLevelView;
		}, this);
		engine._isCoachUnloaded = false;
		
		// If no view has data binding, engine.GlobalAssets.local is not initialized yet.
		// So initialize it here accordingly
		if (engine.GlobalAssets.local == undefined || engine.GlobalAssets.local == null ||
		    !(engine.GlobalAssets.local instanceof com.ibm.bpm.coach.Stateful)) {
			engine.GlobalAssets.local = new stateful(engine.GlobalAssets.local, "local");
		}

		commonHelper.timestamp("Exit engine.initViews()");
	};

	 
	/*
	 * Start of configuration data handling
	 *
	 */
	 
	/**
	 * This function is used by code gen team to pass in the needed information to runtime 
	 * It is for the static value case, ie, user defined an option, the configuration value is static: type in by the user
	 */
	engine.resolveStaticOptionValue = function(configObjId, optionKey, optionValue){
	
	    //use the config object id and the option key to construct a unique key
		var optionValue = {staticOptionStr : "config." +  configObjId + "." + optionKey, value: optionValue };
		if(optionKey === "_metadata.visibility" || optionKey === "_metadata.labelVisibility" ){
			optionValue.isStandardValue = true;
		}
		return optionValue;
	}

	/**
	 * This function is used by code gen team to pass in the needed information to runtime 
	 * User defined an option, the configuration value for this option points to a bo or points to another option
	 * engine.resolveOptionValue.call(this, configId, "_metadata.visibility", "local.dataObject02"),
	 * engine.resolveOptionValue.call(this, configId, "_metadata.visibility", "config.config1._metadata.visibility"),
	 */
	engine.resolveOptionValue = function(configObjId, optionKey, optionBindingStr){
		var optionValue;
		if (!!optionBindingStr && optionBindingStr.indexOf("config.") == 0)
		{
			optionValue = {staticOptionStr : optionBindingStr};
		}
		else{
			//for bo case
			optionValue = {bindingStr : optionBindingStr};
		}
		return optionValue;
	}

	/**
	 * This function is used to get the value of an option
	 */
	engine.getOptionValue = function(configId, optionKey, isMetadata){
		var options = this.GlobalAssets.options[configId];
		if (!options){
			return;
		}
		//fix for defect 80359. we modified the template config object here so the change is limited to minor code path
		var optionValue, value;
		if (!!isMetadata){
			optionValue = options._metadata[optionKey];
		}
		else {
			optionValue = options[optionKey];
		}
		if (!!optionValue && (!!optionValue.staticOptionStr)){
			value = optionValue.value;
		}
		var binding = engine._resolveAndRegisterOptionBindingCallBack(null, options, optionKey, isMetadata);
		// for non-staticOptionStr cases, vlaue is still undefined
		if(!!value) {
			binding.value = value;
		}
		return !!binding ? binding.get("value") : null;
	}
	
	
	/**
	 * This function and its helper is used to resolve all the option binding object (both bo and static value) for a view scope
	 * and then register the scopeobject's change method as the model change callback
	 * Currently, the most reasonable place to invoke it is from init view, where the scope object is created
	 */
	engine.resolveAndRegisterOptionsBindingCallBack = function(scopeObject, update, objectPath) {
	    var options = scopeObject.context.options;
		for(var optionKey in options) {	
		    if (optionKey == "_metadata"){
				for (var metadataKey in options[optionKey]){
					engine._resolveAndRegisterOptionBindingCallBack(scopeObject, options, metadataKey, true, update, objectPath);
				}
			}
			else{			
			  engine._resolveAndRegisterOptionBindingCallBack(scopeObject, options, optionKey, false, update, objectPath);
			}		
		}	
	}
	
	/*
	 * This function is used by resolveAndRegisterOptionsBindingCallBack to perform the task on one option
	 * either under the config object or under _metadata
	 */
	engine._resolveAndRegisterOptionBindingCallBack = function(scopeObject, options, shortOptionKey, isMetadata, update, objectPath){

	        var optionValue;
			var binding;
			var optionKey = shortOptionKey;
			optionValue = options[optionKey];	
			
			if (!!isMetadata){
				optionKey = "_metadata." + shortOptionKey;
				optionValue = options["_metadata"][shortOptionKey];	
			}
			//fix a bug: the user may not give the option a value, 
			//for update case, nothing need to be done for non bo case
			if(!optionValue || !!update && !optionValue.bindingStr)
				return;
				
			//resolved previously, and not for update case
			if(!!optionValue.boundObject && !update){
				binding = optionValue;
			}
				
	         //resolve the object if it is ref option to local, epv, resource
			else {
			if(!!optionValue.bindingStr){
				var affected = (!update) || !objectPath || dataBindingHelper.startsWithPath(optionValue.bindingStr, objectPath);
				if(affected){					
					binding = engine.resolveBinding.call(engine.GlobalAssets, optionValue.bindingStr);					
					// fix JR45548
					if(!!binding) {
					//This is useful later when check the binding string of this option
					binding.bindingStr = optionValue.bindingStr;
					if(!!update){
						binding.changed = true;
					}					
					}
				}
			}
			
			else if (!!optionValue.staticOptionStr){
				//static value	
				binding = engine._resolveStaticOptionBinding (optionValue.staticOptionStr, optionValue.value);
				binding.staticOptionStr = optionValue.staticOptionStr; //to be used for clone case
				//consider the case:  option for config 2 actually pointing to option in config 1 with staticOptionStr  options.config1.showAddButton
				//the binding is stored in _resolveStaticOptionBinding. set it here too
			}
			if (!!binding){
				if((optionKey === "_metadata.visibility" || optionKey === "_metadata.labelVisibility") && !optionValue.isStandardValue){			
					binding.get = function (propName) {
						var result = (propName=="value") ? this.boundObject.get(this.property) : null;
						result = (!!result? result.toUpperCase().replace(/[^A-Z]/g,'') : result);
						return result;
					};					
				}
				//store the value
				if (!!isMetadata){
					options["_metadata"][shortOptionKey] = binding;
				}
				else
					options[shortOptionKey] = binding;
			}
			}
						
			/*
			* register callback function:
			* Same code apply for all 3 cases because they all have a binding object, either wraps a Stateful or a List
			* 1. bo case non array
			* 2. bo case 2: list properites like listSelectedIndex on a List
			* 3. static value 
			* The difference are:
			* For bo, one binding for each option or view scope, they points to the same stateful or list
			* For static, one binding object is used
			*/
			var handle;
			if(!!binding && !!scopeObject && !(lang.isFunction(binding))){
					
					var callBackFunction = engine._createDataConfigCallBack(scopeObject, optionKey);
					
					if (!!binding.boundObject){
						handle = scopeObject._bindingHandles[optionKey];
						if (!!handle && handle.unbind){
							handle.unbind();
						}
						handle = binding.boundObject.registerCallback(binding.property, callBackFunction);
						//store the handles so that we will call unbind/cancel on delete
						scopeObject._bindingHandles[optionKey] = handle;
					}
					else{
						//for epv, resource, we only provides get and set method, no registration of callback
					}
			} 
			
			return binding;
	}
	
	engine._resolveStaticOptionBinding = function(staticOptionStr, value){
				var result;
				
				var pathArray = staticOptionStr.split(".");
				var target = pathArray[0] == "config" ? engine.GlobalAssets : engine.GlobalAssets.options;
				var parent;
				var optionKey;
				
				//the data structure property name is options( engine.GlobalAssets.options;), but the object path start with config
				if (pathArray[0] == "config")
				  pathArray[0] = "options";				  
				  
				/* Path can be 
				 * config.config2.option1.prop1.name,   config.config2.option1, config.config2.option1.prop1.name, 
				 * In any case, we use .notition until we get option1, and then make option1 stateful if it is not yet
				 */
				var bindToProperties = false;
				var upTo = 3;
				if (staticOptionStr.indexOf("_metadata") > -1){
					upTo = 4;
				}

				// handle the scenario: config.config1.option1[3].name, or config.config1.option1[3]
				// option1 will be the stateful object, we need to reserve the index (e.g. 3) for further resolving.
				var planedStatefulObjectStr = pathArray[upTo - 1];
				var leftBracketPosition = planedStatefulObjectStr.indexOf("[");
				if(leftBracketPosition != -1) {
					var rightBracketPosition = planedStatefulObjectStr.indexOf("]");
					var numericValue = planedStatefulObjectStr.substring(leftBracketPosition+1, rightBracketPosition);					
					if(numericValue != "") {
						pathArray.splice(upTo, 0, numericValue);
					}
					pathArray[upTo - 1] = planedStatefulObjectStr.substring(0, leftBracketPosition);
				}
				
				bindToProperties = pathArray.length > upTo;
				
				for (var i=0; i<upTo; i++){
					parent = target;
					optionKey = pathArray[i];
					target = target[optionKey];
				}
				
				//if the option is a service, nothing need to do here. This will cover the following cases
				//1. option value is a service: code gen will generate a function
				//2. option value refer to another option whose value is a function: config.config2.myServiceOption
				if (lang.isFunction(target)){
					result = target;//it is not a real binding, just the function it self
					parent[optionKey] = result;
				}
				else{
					var binding;
					if (!!target.boundObject){
						binding = target;
					}
					else{
					//create the binding object for the option with getters and setters, so that the view can get it using this.context.options.option1.get('value')
						var boundObject = new com.ibm.bpm.coach.Stateful({value : target.value}, staticOptionStr);
						binding = engine._createBindingObject(boundObject, "value");
						
						// remember the staticOptionStr (if exists) of parent[optionKey] before assigning new object to parent[optionKey]
						var possibleStaticOptionStr = !!parent[optionKey] ? parent[optionKey].staticOptionStr : null;
						parent[optionKey] = binding;
						// if parent[optionKey].staticOptionStr exists before we assign a new object to parent[optionKey], we need to keep this staticOptionStr in new object because clone needs it.
						// when _resolveStaticOptionBinding() is called by _resolveAndRegisterOptionBindingCallBack(), staticOptionStr is set back. 
						// But _resolveStaticOptionBinding has other callers, that's why we handle here.
						if(!!possibleStaticOptionStr) {
							parent[optionKey].staticOptionStr = possibleStaticOptionStr;
						}
						
						//for system change handler: Nothing needs to be done in the Stateful object, instead
						//the controller should be able to understand it by the object path
					}
					if (bindToProperties){
						//options.config1.option1.name
						pathArray.splice(0, upTo - 1);
						pathArray[0] = "value";
						result = this._resolveBindingObject(binding.boundObject, pathArray );
					}
					else {
						result = binding;
					}
					//if the binding is to some property in the option value, ie, option value is a complex object, then we need to continue to resolve the binding
					
				}
				
			return result;
	}
	
	engine._createBindingObject = function(boundObject, property){
		var get = function (propName) {return ((propName=="value") ? this.boundObject.get(this.property) : null)};
		var set = function(propName, newVal) {
			if(propName =="value") {
				this.boundObject.set("value", newVal);
			}
		};
		
		//when initView ->resolvebinding, register callback on binding, if the property is null, it register on all property changes 
		var binding = {get: get, set: set, property : property, boundObject : boundObject};
	    return binding;	
	}
	
	/*
	* Create a function to be used as the call back for config data change
	*/
	engine._createDataConfigCallBack = function(scopeObject, optionKey){	       
		   
	       function optionsBindingcallBack (property, oldVal, newVal){	 
              	var event = { type : "config", property:optionKey, oldVal: oldVal, newVal: newVal, removedFrom: null, insertedInto: null};
				scopeObject._change(event);
		   }
	       return optionsBindingcallBack;
	}
	
	/* 
	 * This is for repeatting case
     * We first clone the config object
	 * If an option points to something under an array, then the index in binding will be adjusted
	 * If an option's value is a non bo object, we adjust the staticOptionStr to use the new config object id
	 * 
	 * static value      object                own: replace the configId portion of the staticOptionStr with the new configId
	 *                   function              same
	 * refrence to 
	 * 			local(non-array)         same
	 *          local(array)             points to the right index in array
	 *          
	 * reference to  static              same
	 *               static array        same array object
	 *               function            same
	 */
	engine.cloneAndAdjustDataConfig = function(viewElement, bindingIndex, maxNumOfUnresolvedRepeating, templateConfigIdToInstanceConfigIdMap){
		
		var tempViewCfgBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_CONFIG);
		if(!tempViewCfgBinding){
			return;
		}
	    var configObj = engine.GlobalAssets.options[tempViewCfgBinding];
		var clonedObjId = "config_" + viewElement.id;
		
		//clone the object and store it under GlobalAssets.options
		var clonedObj = {"_metadata" : {}, "templateConfigId" : tempViewCfgBinding, "configId": clonedObjId};
		
		// add info of newly cloned config to templateConfigIdToInstanceConfigIdMap
		engine._processOneConfig(templateConfigIdToInstanceConfigIdMap, clonedObj);
		
		var configValue;
		var bindingStr;
		var metaDataOptionValue;
		var configInOptionStr;
		var pathArray;
		for(var configKey in configObj) {
			configValue = configObj[configKey];
		    
			if (configKey == "_metadata"){
			for(var metadataKey in configValue){				
				metaDataOptionValue = configObj[configKey][metadataKey];
				if (!metaDataOptionValue){
					continue;
				}
				//check if it points to an bo
				if (configValue[metadataKey].bindingStr){	
					bindingStr = configValue[metadataKey].bindingStr;
					//adjust index if needed
					bindingStr = dataBindingHelper.updateBindingStr(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating); 
					clonedObj[configKey][metadataKey] = {"bindingStr" : bindingStr };
					
				}
				//for static value case
				else if(!!configValue[metadataKey].staticOptionStr){
						// defect 96461
						staticOptionStr1 = configValue[metadataKey].staticOptionStr;
						//adjust index if needed
						staticOptionStr1 = dataBindingHelper.updateBindingStr(staticOptionStr1, bindingIndex, maxNumOfUnresolvedRepeating); 

						var staticOptionValue = metaDataOptionValue.value;
						if(staticOptionValue == undefined && metaDataOptionValue.boundObject && metaDataOptionValue.get) {
							// 
							staticOptionValue = metaDataOptionValue.get("value");
						}
						clonedObj[configKey][metadataKey] =
                        {staticOptionStr: staticOptionStr1,
						 value: staticOptionValue
						}
						
						// following logic handles 2 scenarios: 
						// a) clonedObj[configKey][metadataKey].staticOptionStr is "config.configABC....". 
						//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ"), and we have added configABC --> configXYZ to templateConfigIdToInstanceConfigIdMap.
						//    Now it's time to update the config reference (replacing "configABC" with "configXYZ".
						// b) clonedObj[configKey][metadataKey].staticOptionStr is "config.configDEF....", 
						//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ").
						//    Now we need to check if configDEF has been cloned earlier, in the scope of from currentView up to the topLevelView. If so, replace "configDEF" with its cloned instance "configLMN". 
						clonedObj[configKey][metadataKey].staticOptionStr = engine._updateStaticOptionStr(staticOptionStr1, templateConfigIdToInstanceConfigIdMap);
				}
				else{
					clonedObj[configKey][metadataKey] = metaDataOptionValue;
				}
			}
			}
			else{
				if(!configValue){
					continue;
				}
				//check if it points to an object
				if (!!configValue.bindingStr){	
					bindingStr = configValue.bindingStr;
					//adjust index if needed
					bindingStr = dataBindingHelper.updateBindingStr(bindingStr, bindingIndex, maxNumOfUnresolvedRepeating); 
					clonedObj[configKey] = {"bindingStr" : bindingStr };
				}
				//static value which maybe shared	
				else if (!!configValue.staticOptionStr) {
                    var optionValue = configValue.value;
                    if(configValue.boundObject && configValue.get) {
                        optionValue = configValue.get("value");
                    }
                    clonedObj[configKey] =
                    {value: optionValue}
					
					// following logic handles 2 scenarios: 
					// a) staticOptionStr is "config.configABC....". 
					//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ"), and we have added configABC --> configXYZ to templateConfigIdToInstanceConfigIdMap.
					//    Now it's time to update the config reference (replacing "configABC" with "configXYZ".
					// b) staticOptionStr is "config.configDEF....", 
					//    We just cloned configXYZ from configABC (i.e. tempViewCfgBinding is "configABC" and clonedObjId is "configXYZ").
					//    Now we need to check if configDEF has been cloned earlier, in the scope of from currentView up to the topLevelView. If so, replace "configDEF" with its cloned instance "configLMN". 
					clonedObj[configKey].staticOptionStr = engine._updateStaticOptionStr(configValue.staticOptionStr, templateConfigIdToInstanceConfigIdMap);
				}
				else{//simply copy over the value
					clonedObj[configKey] = 	configValue;			
				}
			}
		}	 
		
		engine.GlobalAssets.options[clonedObjId] = clonedObj;
		
		//update the view    
		viewElement.setAttribute(engine.CONSTANTS.DATA_CONFIG, clonedObjId);
	}
	
	engine._updateStaticOptionStr = function(staticOtionStr, templateConfigIdToInstanceConfigIdMap) {
		var pathArray = staticOtionStr.split(".");
		var referencedConfigId = pathArray[1];
		var newReferencedConfigId = templateConfigIdToInstanceConfigIdMap[referencedConfigId];
		if(!!newReferencedConfigId) {
			pathArray[1] = newReferencedConfigId;
			return pathArray.join(".");
		}
		
		return staticOtionStr;
	}
	
	engine.notifyDataConfigBindingChange = function(scopeObject){
		var dataConfig = scopeObject.context.options;
		var optionValue;
		if (!dataConfig){
			return;
		}
		
		var bindingForMetadata;
		for(var optionKey in dataConfig) {	
		    if (optionKey == "_metadata"){
				for (var metadataKey in dataConfig[optionKey]){
					bindingForMetadata = dataConfig[optionKey][metadataKey];
					// if config option binds to resource, system, epv etc, bindingForMetadata doesn't have boundObject at all. And this kind of binding is supposed to be readonly, no change event should be fired.
					if (!!bindingForMetadata.bindingStr && !!bindingForMetadata.changed && !!bindingForMetadata.boundObject){
						   //call the change method
						   var event = { type : "config", property: optionKey + "." + metadataKey, oldVal: null, newVal: bindingForMetadata.boundObject[bindingForMetadata.property]};
						   scopeObject._change(event);
						   // PMR 124679 Need to reset the .changed attribute
						   if ((typeof bindingForMetadata.changed != 'undefined') && (bindingForMetadata.changed == true)) {
							scopeObject.context.options[optionKey][metadataKey].changed = false;
						   }
					}
				}
			}
			else{			
			   optionValue = dataConfig[optionKey];
				// if config option binds to resource, system, epv etc, optoinValue doesn't have boundObject at all. And this kind of binding is supposed to be readonly, no change event should be fired.
			   if (!!optionValue.bindingStr && !!optionValue.changed && !!optionValue.boundObject){
				   //call the change method
				   var event = { type : "config", property:optionKey, oldVal: null, newVal: optionValue.boundObject[optionValue.property]};
				   scopeObject._change(event);
				   // PMR 124679 Need to reset the .changed attribute
				   if ((typeof optionValue.changed != 'undefined') && (optionValue.changed == true)) {
					scopeObject.context.options[optionKey].changed = false;
				   }
			   }
			}		
		}		
	}	
	//# END of configuration data handling
	
	 
	 
	//##Data Binding start
	engine.resolveBinding = function(bindingString) {
		//Resolves a binding string in the form of BO.property

		if (bindingString == null || bindingString == "") return;

		var result;
		var refArray = bindingString.split(".");
		if (refArray[0] == "local") {
			if (!(this["local"] instanceof com.ibm.bpm.coach.Stateful)) {
				this["local"] = new stateful(this["local"], "local");
			}
			refArray.splice(0, 1);
			result = engine._resolveBindingObject(this["local"] , refArray);			
		}
		else if (bindingString.indexOf("config") == 0){
			//bind to parent's options, resolve it
			result = engine._resolveStaticOptionBinding(bindingString);
		}	
		else {
			//handle system, resource, epv
			//add a setter, as required for consistency, but no callback is registered
			//if we want to notify change, we will make them as stateful so we do not need to maintain the callback functions
			var get = function (propName) {return (propName=="value")?this.value:null};
			var set = function (propName, val) {
				logger.log("engine", "resolveBinding", "set method is not supported on: " + bindingString);					
			};
			bindingString = bindingString.substring(refArray[0].length);
			var value = eval("this['"+refArray[0]+"']" + bindingString);
			result = {value: value, get: get, set: set};
		}
		return result;
	}

	engine._resolveBindingObject = function (objectRoot, refArray){
		var parentObject = objectRoot;
		var boundObject = objectRoot;
		var prop = null;
		for (var i=0; i<refArray.length; i++) {
			parentObject = boundObject;
			if (!boundObject){
				//for binding like  local.myList.listSelected.pro1, if the list is empty, listSelected will be undefined
		//		logger.log("engine", "_resolveBindingObject ", "boundObject is undefined. property:" + refArray[i-1]);
				break;
			}
			if (refArray[i].indexOf("[]") != -1) {
				refArray[i] = refArray[i].substring(0, refArray[i].indexOf("[]"));
			}
			var subMatch = refArray[i].match(/\[.*\]/);
			var index = null;
			if (subMatch != null) {
				index = subMatch[0].substring(1, subMatch[0].length-1);
				refArray[i] = refArray[i].substring(0, subMatch.index);
			}
			prop = refArray[i];
			if(index == null) {
				boundObject = boundObject.get(refArray[i]);
			} else {
				parentObject = boundObject.get(refArray[i]);
				boundObject = boundObject.get(refArray[i]).get(index);
				prop = index;
			}
		}
		var set;
		var get;
		if ( parentObject instanceof com.ibm.bpm.coach.List && (parseInt(prop) >= 0)) {
			var id = parentObject.indexToId[prop];
			set = function(propName, val) {(propName=="value")? this.boundObject.set(id, val):0};
			get = function (propName) {return (propName=="value")? this.boundObject.getById(id):null};
		} else {
			set = function(propName, val) {(propName=="value")? this.boundObject.set(this.property, val):0};
			get = function (propName) {return (propName=="value")? this.boundObject.get(this.property):null};
		}
		
		var result = !!parentObject ? {boundObject: parentObject, property: prop, get: get, set: set} : undefined;
		return result;
	}
	
	//##Data Binding end
	
	engine.initView = function(viewElement, parentViewScope) {
		logger.log("engine", "initView ", "viewElement: " + viewElement.id);
		
		var tempViewDivId = viewElement.getAttribute("id");
		var tempViewId = viewElement.getAttribute(engine.CONSTANTS.VIEW_ID);
		var tempViewType = viewElement.getAttribute(engine.CONSTANTS.VIEW_DATA_TYPE);
		var tempViewDataBinding = engine.rtDataBinding._get(viewElement);
		var tempViewCfgBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_CONFIG);
				
	//##Data Binding start
		//data
		var dataBinding = engine.resolveBinding.call(this, tempViewDataBinding);

		//config
		var cfgBinding = this.options[tempViewCfgBinding]; 
		
		var subclass = this.scopeClasses[tempViewType];
		try {
			var viewScope = new subclass(new ViewContext(), viewElement, dataBinding, cfgBinding);
	    } catch(ex) {
	    	var errorMsg = "engine.initView() caught Exception when calling view constructor for " + tempViewDivId + ", Exception: " + ex;
	    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			return;
	    }
	    
		//data
		if (dataBinding != null  && dataBinding.boundObject != null) {
			var bindingHandle;
			if (dataBinding.boundObject instanceof com.ibm.bpm.coach.Stateful) {
			    //register the callback on the parent(boundObject)
				bindingHandle = dataBinding.boundObject.registerCallback(dataBinding.property, viewScope._changeObject);				
			} else if(dataBinding.boundObject instanceof com.ibm.bpm.coach.List){
				//it is for List. Note that it is for those 4 properties on a list, not for the items in the list
				//so we should use viewScope._changeObject as call back
				
				//defect 86200 start - need to specifically call the change function when items in a list change
				var funct = function(object, removedFrom, insertedInto) {
					if (!!object && !!object.bpm_id && dataBinding.boundObject.idToIndex(object.bpm_id) == dataBinding.property) {
						viewScope._changeObject(dataBinding.property, "", object.listItem);
					}
					
				};
				//if it is an index then call the wrapped function, otherwise just register the change function
				if (parseInt(dataBinding.property) >-1) {
					bindingHandle = dataBinding.boundObject.registerCallback(null, funct);
				} else {
					bindingHandle = dataBinding.boundObject.registerCallback(dataBinding.property, viewScope._changeObject);
				}
				//defect 86200 end				
				
				
				//there is no binding produced by code gen will match to list item inserted or removed kind of change will never show up here
				//instead, user should use bindAll to register callback on those kind of changes.
			}
			else{
				//for resource, epv, readonly, we only provide get and set method, but we do not notify changes
			}
			if (!!bindingHandle){
				viewScope._bindingHandles[dataBinding.property] = bindingHandle;
			}
		}

		//config
		if (!!cfgBinding) {
			try {
				engine.resolveAndRegisterOptionsBindingCallBack(viewScope);
			} catch(ex) {
		    	var errorMsg = "engine.initView() caught Exception when calling engine.resolveAndRegisterOptionsBindingCallBack(), Exception: " + ex;
		    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
			}
			
		}
		 
	//##Data Binding end
		
		if (!!parentViewScope) {
			// Set the scope to the parent
			var viewId = viewElement.getAttribute(engine.CONSTANTS.VIEW_ID);
			var divId = viewElement.getAttribute("id");
			if(!parentViewScope.context.subview[viewId]) {
				parentViewScope.context.subview[viewId] = {};
			}
			parentViewScope.context.subview[viewId][divId] = viewScope;
			viewScope.context._parentView = parentViewScope;
		}
		
		// continue to build the children views
		viewScope._buildChildren();

		viewScope._preLoad();
		
		// Call the view onLoad lifecycle method
		engine.callViewLifeCycleAPI(viewScope, "load", CONSTANTS.ELAPSED_TIME_THRESHOLD_LOAD);
	    
	    if(!!tempViewDataBinding && tempViewDataBinding != "") {
	    	// if dataBinding doesn't exist,
	    	// or dataBinding is list, but can't get value from it (index is out of bound). 
	    	// When dataBinding is list.listSelected, list.listSelected can't be resolved at this moment just because nothing is selected currently, 
	    	// Nothing wrong with this kind of situlation. That's why list.listSelected is seperated out.
	    	if(!dataBinding ||(dataBinding.boundObject instanceof com.ibm.bpm.coach.List && dataBinding.get("value") != "" && !dataBinding.get("value") && dataBinding.property != "listSelected")) {
		    	// we failed to resolve binding for this view. so highlight it here to let user be aware 
		    	viewElement.style.outline = "1px dotted red";
		    }
	    }	    
	    
	    // if the performance of following line is concern, we can collect bpmIds when the binding is resolved initially.
	    viewScope._bpmIds = dataBindingHelper.collectBpmIds(tempViewDataBinding);
	    
		return viewScope;
	};
	
	engine.callViewLifeCycleAPI = function(viewScope, apiName, elapsedTimeThreshold, parameter) {
		try {		
			var startTime = commonHelper.isDebugMode() ? (new Date()).getTime() : null;
			viewScope[apiName].call(viewScope, parameter);
			if(startTime) {
				var elapsedTime = (new Date()).getTime() - startTime;
				if(elapsedTime >= elapsedTimeThreshold) {
					var info = apiName + "() for " + viewScope.context.element.id + " used " + elapsedTime + "ms";
					commonHelper.timestamp(info);
				}
			}
	    } catch(ex) {
	    	try {
		    	var message = "Coach framework caught exception when calling Life cycle API " + apiName + "() on view " + viewScope.context.element.id;
		    	message +=  ", Exception: " + ex;
		    	commonHelper.consoleLog(message, commonHelper.ERROR);
	    	} catch(ex1) {
		    	commonHelper.consoleLog("Even failed to gracefully log this Exception: " + ex, commonHelper.ERROR);
	    	}
	    }
	}

	engine.prepareView = function() {
		commonHelper.timestamp("Enter engine.prepareView()");
		logger.log("engine", "prepareView", "is called", logger.INFO);
		for (var viewId in this.topLevelViewScope) {
			   var viewScope = this.topLevelViewScope[viewId];
			   engine.callView(viewScope);
		}
		commonHelper.timestamp("Exit engine.prepareView()");
	};

	engine.callView = function(viewScope) {
		// Call nested view view first
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {
				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					engine.callView(subViewScope);
				}
			}		
		}
		
	    // Call the view view lifecycle method		
		engine.callViewLifeCycleAPI(viewScope, "view", CONSTANTS.ELAPSED_TIME_THRESHOLD_VIEW);
	    
	    viewScope._monitorRelatedListsIfNeeded();
	    
	    // Display the content
		// viewScope.context.element.style.display = 'block';
		var viewClass = viewScope.context.element.getAttribute('class');
		if(!domClass.contains(viewScope.context.element, engine.CONSTANTS.COACH_VIEW_SHOW)) {
			domClass.add(viewScope.context.element, engine.CONSTANTS.COACH_VIEW_SHOW);
		}
	};

	//The function, adjustIdAndBinding, is called from 2 different places
	//one is from createView. The node( divNode )is cloned and passed into createView.
	//another is from _repeatContentBoxs, the node, divNode, is cloned in _repeatContentBoxs and passed into this function
	//I need to clone the data config object and adjust the binding str based on the index.
	//If clone the data before this function is called, then I need to do recursive traveral and clone 
	//Rather, do it here, so that it use the same traveral code
	engine.adjustIdAndBinding = function(divNode, parentViewScope, bindingIndex) {
		if(divNode.tagName != "DIV")
			return null;
		
		// var idSuffix = "_" + bindingIndex;  // bindingIndex is not enough as idSuffix due to the dynamic delete/add
		var idSuffix = "_r" + parentViewScope._counter;
		parentViewScope._counter++;

		var templateConfigIdToInstanceConfigIdMap = engine._prepareTemplateConfigIdToInstanceConfigIdMap(parentViewScope);
		if(engine.isContentBox(divNode)) {
			// this divNode is a contentBox
			engine._adjustIdAndBindingForContentBox(divNode, idSuffix, bindingIndex, 1, parentViewScope, templateConfigIdToInstanceConfigIdMap);
		} else {
			// this divNode is expected to be a view
			engine._adjustIdAndBindingForView(divNode, idSuffix, bindingIndex, 1, parentViewScope, templateConfigIdToInstanceConfigIdMap);
		}
	};
	
	engine._prepareTemplateConfigIdToInstanceConfigIdMap = function(viewScope) {
		var templateConfigIdToInstanceConfigIdMap = {};
		var oneView = viewScope;
		while(!!oneView) {
			var viewOptions = oneView.context.options; 
			engine._processOneConfig(templateConfigIdToInstanceConfigIdMap, viewOptions);
			oneView = oneView.context.parentView();
		} 
		return templateConfigIdToInstanceConfigIdMap;
	}
	
	engine._processOneConfig = function(templateConfigIdToInstanceConfigIdMap, aConfig) {
		if(!!aConfig && !!aConfig.templateConfigId) {
			templateConfigIdToInstanceConfigIdMap[aConfig.templateConfigId] = aConfig.configId;
		} 
	}
	
	engine._shallowCloneMap = function(aMap) {
		var clonedMap = {};
		for(var key in aMap) {
			clonedMap[key] = aMap[key];
		}
		return clonedMap;
	}
	
	engine._adjustIdAndBindingForContentBox = function(conentBoxDomNode, idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, templateConfigIdToInstanceConfigIdMap) {
		// adjust id
		conentBoxDomNode.setAttribute('id', conentBoxDomNode.id + idSuffix);
		
		// handle nested Views
		var immediateViewList = domHelper.shallowQueryByClass(conentBoxDomNode, engine.CONSTANTS.COACH_VIEW, false);
		for(var i=0; i<immediateViewList.length; i++) {
			var newMap = engine._shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForView(immediateViewList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}
		
		// handle nested ContentBox
		var immediateContentBoxList = domHelper.shallowQueryByClass(conentBoxDomNode, engine.CONSTANTS.CONTENT_BOX, false);
		for(var i=0; i<immediateContentBoxList.length; i++) {
			var newMap = engine._shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForContentBox(immediateContentBoxList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}			
	}
	
	
	engine._adjustIdAndBindingForView = function(viewDomNode, idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, templateConfigIdToInstanceConfigIdMap) {
		// adjust id
		viewDomNode.setAttribute('id', viewDomNode.id + idSuffix);

		var viewDataBinding = dataBindingHelper.rtDataBinding._get(viewDomNode);		
		
		// if view's data binding is a config option, need to make sure this config option is up-to-date (in case the config option has been cloned)
		if(viewDataBinding.indexOf("config") == 0) {
			var newConfigBindingStr = engine._updateStaticOptionStr(viewDataBinding, templateConfigIdToInstanceConfigIdMap);
			if(viewDataBinding != newConfigBindingStr) {
				dataBindingHelper.rtDataBinding._set(viewDomNode, newConfigBindingStr);
				viewDataBinding = newConfigBindingStr;
			}
		}

		// now adjust this view's databinding
		var numOfIndex = dataBindingHelper.getNumberOfIndex(viewDataBinding);
		if(numOfIndex >= maxNumOfUnresolvedRepeating) {
		
			if(maxNumOfUnresolvedRepeating == 1 && numOfIndex == 1) {
				// binding of viewDomNode recieves the index value (currentItem) from the binding list of its parent view
				// if the length of viewDomNode binding list could be differnt from that of parent view' binding list,
				// we need to manage this kind of situation.
				var indexWithBracket = "[" + CONSTANTS.CURRENT_ITEM + "]";
				var involvedList = viewDataBinding.substring(0, viewDataBinding.indexOf(indexWithBracket));
				viewControlRepeating._addRelatedList(viewDomNode.id, involvedList);
			}				
		
			var newBinding = dataBindingHelper.updateBindingStr(viewDataBinding, bindingIndex, maxNumOfUnresolvedRepeating, numOfIndex);
			dataBindingHelper.rtDataBinding._set(viewDomNode, newBinding);
		} else {
			// now numOfIndex < maxNumOfUnresolvedRepeating, we will adjust the index when hande next repeat. 
		}
		
		// now handle the dataConfig
		engine.cloneAndAdjustDataConfig(viewDomNode, bindingIndex, maxNumOfUnresolvedRepeating, templateConfigIdToInstanceConfigIdMap);	
				
		// handle nested Views
		var immediateViewList = domHelper.shallowQueryByClass(viewDomNode, engine.CONSTANTS.COACH_VIEW, false);
		for(var i=0; i<immediateViewList.length; i++) {
			var newMap = engine._shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForView(immediateViewList[i], idSuffix, bindingIndex, maxNumOfUnresolvedRepeating, viewControlRepeating, newMap);
		}
		
		// handle nested ContentBox
		var immediateContentBoxList = domHelper.shallowQueryByClass(viewDomNode, engine.CONSTANTS.CONTENT_BOX, false);
		var newMaxNumOfUnresolvedRepeating = maxNumOfUnresolvedRepeating;
		var newViewControlRepeating = viewControlRepeating;
		if(dataBindingHelper.isBindingArray(viewDataBinding)) {
			newMaxNumOfUnresolvedRepeating++;
			// repeating of immediateContentBoxList[] is controlled by viewDomNode. but view object for viewDomNode is not
			// creaed yet. And newMaxNumOfUnresolvedRepeating won't be used in the called method because there is more than one
			// unresolved repeating at this moment.
			newViewControlRepeating = null;
		}
		for(var i=0; i<immediateContentBoxList.length; i++) {
			var newMap = engine._shallowCloneMap(templateConfigIdToInstanceConfigIdMap);
			engine._adjustIdAndBindingForContentBox(immediateContentBoxList[i], idSuffix, bindingIndex, newMaxNumOfUnresolvedRepeating, newViewControlRepeating, newMap);
		}
		
	}

//************************* remove this part later, replace with the ones on dataBindingHelper	
	// rtDataBinding provides 3 helper method for resolving List related data-binding during runtime.
	engine.rtDataBinding = new Object();
	engine.rtDataBinding._get = function(viewElement) {
		var rtBinding = viewElement.getAttribute(engine.CONSTANTS.DATA_BINDING_RT);
		if(!rtBinding) {
			return viewElement.getAttribute(engine.CONSTANTS.DATA_BINDING);
		} else {
			return rtBinding;
		}
	}
	engine.rtDataBinding._set = function(viewElement, bindingValue) {
		viewElement.setAttribute(engine.CONSTANTS.DATA_BINDING_RT, bindingValue);
	}
	engine.rtDataBinding._delete = function(viewElement) {	
		// viewElement.removeAttribute(engine.CONSTANTS.DATA_BINDING_RT);
	}
//************* end

	engine.isContentBox = function(node) {
		if(!!node) {
			return domClass.contains(node, engine.CONSTANTS.CONTENT_BOX);
		}
		return false;
	};
	
	// check if the provided contentBox is view managed
	engine.isViewManagedContentBox = function(contentBoxNode) {
		return contentBoxNode.getAttribute(engine.CONSTANTS.IS_VIEW_MANAGED) == "true";
	}
	
	// contentBoxDiv can be framework managed ContentBox or view managed ContentBox
	// newViewArray is optional. If provided, it stores the new views created by this method.
	engine.buildContentBox = function(parentView, contentBoxDiv, newViewArray) {
		var immediateViewList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.COACH_VIEW, true);
		var oneDirectView;
		array.forEach(immediateViewList, function(item, index) {
			oneDirectView = engine.initView.apply(engine.GlobalAssets, [item, parentView]);
			if(!!newViewArray) {
				newViewArray.push(oneDirectView);
			}
		});
		
		// find the direct ContentBox under this contentBox.
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.CONTENT_BOX, true);
		// only care about the framework managed ContentBox. view managed ContentBox will be built by its owner view.
		for(var k=0; k<immediateContentBoxList.length; k++) {
			if(!engine.isViewManagedContentBox(immediateContentBoxList[k])) {
				// Framework managed
				engine.buildContentBox(parentView, immediateContentBoxList[k], newViewArray);
			}
		}
	}
	
	// delete all subviews that reside in the specified contentBox, including 
	// 1) the views owned by specified contentBox directly
	// 2) the views under the framework or view managed contentBox that is owned by specified contentBox
	engine.deleteContentBox = function(contentBoxDiv, ownerView) {
		var directViewNodes = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.COACH_VIEW);
		array.forEach(directViewNodes, function(item, index) {
			engine.deleteView(item, ownerView);				
		}, ownerView);

		// find the direct ContentBox under this contentBox.
		var immediateContentBoxList = domHelper.shallowQueryByClass(contentBoxDiv, engine.CONSTANTS.CONTENT_BOX, true);
		// delete all contentBox regardless it's framework managed or view managed.
		for(var k=0; k<immediateContentBoxList.length; k++) {
			engine.deleteContentBox(immediateContentBoxList[k], ownerView);
		}
	}
	
	engine.deleteView = function(subviewDivNode, ownerView) {
		var viewId = subviewDivNode.getAttribute(engine.CONSTANTS.VIEW_ID);
		var divId = subviewDivNode.getAttribute("id");
		var subviews = ownerView.context.subview[viewId];
		if(!!subviews && !!subviews[divId]) {
			engine.callUnload(subviews[divId]);
			delete subviews[divId];
		}
	}

	// Future consideration - engine holds a map between divId and viewScope object.
	// Pro: search vewScope object by div id (like following few methods) will perform better and looks smple.
	// Con: have to maitain the map properly when add/delete occurs
	// The performance of following methods (about finding viewScope) should be ok since it just traverse DOM tree straightly up/down.

	engine.findEndStateId = function(bdEventId) {
	    var systemData = engine.GlobalAssets.systemData;
	    if (systemData.boundaryEventInformation) {
	        for (var i=0; i<systemData.boundaryEventInformation.length; i++) {
	            if (systemData.boundaryEventInformation[i].boundaryEventID == bdEventId) {
	                return systemData.boundaryEventInformation[i].endStateID;
	            }
	        }
	    }
	    return "";
	}

    // helper function for retrieving boundary event information by the given boundary event id
    engine.getBoundaryEventInfo = function(bdEventId) {
        var systemData = engine.GlobalAssets.systemData;
        if (systemData.boundaryEventInformation) {
            for (var i=0; i<systemData.boundaryEventInformation.length; i++) {
                if (systemData.boundaryEventInformation[i].boundaryEventID == bdEventId) {
                    return {boundaryEventId: bdEventId,
                            boundaryEventEndStateId: systemData.boundaryEventInformation[i].endStateID,
                            fireValidation: systemData.boundaryEventInformation[i].fireValidation};
                }
            }
        }
        return {boundaryEventId: bdEventId, boundaryEventEndStateId: "", fireValidation: 0};
    }

    // helper function for retrieving validation boundary event id and end state id
    engine.getValidateBoundaryEventInfo = function() {
        var systemData = engine.GlobalAssets.systemData;
        if (!!systemData.validateBoundaryEventInformation) {
            // for now we assume that there may be only one validation boundary event per coach
            if (!!systemData.validateBoundaryEventInformation[0]) {
                return {validateBoundaryEventId: systemData.validateBoundaryEventInformation[0].boundaryEventID,
                        validateBoundaryEventEndStateId: systemData.validateBoundaryEventInformation[0].endStateID};
            }
        }
        return null;
    }

	// Helper API for handling debug response
	engine.handleDebugMode = function(nextURL) {
		engine.onCoachUnload.call(engine.GlobalAssets);

		// redirect to the next URL
		window.location.assign(nextURL);
	}
	
	// Helper API for handling boundary event response
	engine.TaskCompleted = function(nextURL, nextTaskId) {
		engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			if (!!nextTaskId) {
			var event = {
				"name": "onCompleted",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": [nextTaskId]
			};
			// work item 131966
			var serializedEvent = json.toJson(event);
			try {
				window.parent.postMessage(serializedEvent, "*");
			} catch (e) {
				// postMessage() failed, use direct call onMessage instead with faked event. Container MUST be co-located
				if ((!!(window.parent.onMessage)) && typeof window.parent.onMessage === 'function') {
					logger.log("engine", "TaskCompleted", "iframe postMessage() failed, call onMessage() directly - event=" + serializedEvent, logger.FINE);
					window.parent.onMessage({origin: window.location.protocol + "//" + window.location.host,
											data: serializedEvent, 
											source: window});
				} else {
					logger.log("engine", "TaskCompleted", "iframe postMessage() failed, no onMessage() to call on parent - event=" + serializedEvent, logger.FINE);
				}
			}			
			}
			
			// Defect 70546.  We have to show task completed page in case of Process Admin Administrative service
			// if(!!nextTaskId) {
			if(!!nextURL) {
				//commenting this out for defect 79695, breaks auto-flow
				//if(window.parent.location.pathname == "/mum/enabler") {
				//      return;
				//}
				window.location.assign(nextURL);
			}
			// }
		} else {
			// redirect to the next URL in the non-embedded case
			if(!!nextURL) {
				if ((window.parent.location.pathname == "/mum/enabler") && (nextURL.indexOf("tm_process_finished.lsw") > -1)) {
					return;
				}
				window.location.assign(nextURL);
			}
		}
	}
	
	engine.TaskCompletedNotOwned = function(nextURL) {
		engine._taskCompletedOthers(nextURL);
	}

	engine.TaskCompletedAlreadyClosed = function(nextURL) {
		engine._taskCompletedOthers(nextURL);
	}
	
	engine._taskCompletedOthers = function(nextURL) {
		engine.onCoachUnload.call(engine.GlobalAssets);
		if(!!nextURL) {
			window.location.assign(nextURL);
		}
	}	
	
	engine.TaskPostponed = function(nextURL) {
		engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onSuspended",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": []
			};
			// work item 131966
			var serializedEvent = json.toJson(event);
			try {
				window.parent.postMessage(serializedEvent, "*");
			} catch (e) {
				// postMessage() failed, use direct call onMessage instead with faked event. Container MUST be co-located
				if ((!!(window.parent.onMessage)) && typeof window.parent.onMessage === 'function') {
					logger.log("engine", "TaskPostponed", "iframe postMessage() failed, call onMessage() directly - event=" + serializedEvent, logger.FINE);
					window.parent.onMessage({origin: window.location.protocol + "//" + window.location.host,
											data: serializedEvent, 
											source: window});
				} else {
					logger.log("engine", "TaskPostponed", "iframe postMessage() failed, no onMessage() to call on parent - event=" + serializedEvent, logger.FINE);
				}
			}			
		} else {
			if(!!nextURL) {
				window.location.assign(nextURL);
			}
		}
	}


	engine.StateChange = function(nextURL) {
		engine.onCoachUnload.call(engine.GlobalAssets);
		// redirect to the next URL
		window.location.assign(nextURL);
	}


	engine.ErrorBoundaryEvent = function(message) {
		engine.onCoachUnload.call(engine.GlobalAssets);
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onError",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": [message]
			};
			// work item 131966
			var serializedEvent = json.toJson(event);
			try {
				window.parent.postMessage(serializedEvent, "*");
			} catch (e) {
				// postMessage() failed, use direct call onMessage instead with faked event. Container MUST be co-located
				if ((!!(window.parent.onMessage)) && typeof window.parent.onMessage === 'function') {
					logger.log("engine", "ErrorBoundaryEvent", "iframe postMessage() failed, call onMessage() directly - event=" + serializedEvent, logger.FINE);
					window.parent.onMessage({origin: window.location.protocol + "//" + window.location.host,
											data: serializedEvent, 
											source: window});
				} else {
					logger.log("engine", "ErrorBoundaryEvent", "iframe postMessage() failed, no onMessage() to call on parent - event=" + serializedEvent, logger.FINE);
				}
			}			
		}
		// Showing the coachNG error page.  This is necessary in particular in Process Admin Administrative service case.
		window.location.assign("html/coachNGError.jsp");
	}

	// Helper API for handling collab related event
	engine.handleFocusEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var outlineColor = !!event.outlineColor ? event.outlineColor : "red"; // now use "red" as default color. refine later
			var message = !!event.userName ? event.userName : engine.localizedMessages.coachNG_unknown;
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "brown"; // now use "brown" as default background color. refine later
			
		    var collaborationEvent = {"type": "focus", "domId": domId, "outlineColor": outlineColor, "color": color, "bgColor": bgColor, "message": message};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleBlurEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);	
					
		    var collaborationEvent = {"type": "blur", "domId": domId};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleMouseOnEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "red"; // now use "red" as default background color. refine later
			var message = !!event.message ? event.message : engine.localizedMessages.coachNG_unknown;			
		    
		    var collaborationEvent = {"type": "mouseOn", "domId": domId, "color": color, "bgColor": bgColor, "message": message};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleMouseOffEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);			
		    
		    var collaborationEvent = {"type": "mouseOff", "domId": domId};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}

	engine.handleCollabDataChangeEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {
			var domId = engine._findCollaboratingDomId(viewScope, event);
			var color = !!event.color ? event.color : "black"; // now use "black" as default color. refine later
			var bgColor = !!event.bgColor ? event.bgColor : "red"; // now use "red" as default background color. refine later
			var message = !!event.message ? event.message : engine.localizedMessages.coachNG_unknown;
			var userName = !!event.userName ? event.userName : engine.localizedMessages.coachNG_unknown_user;
			var value = !!event.value ? event.value : engine.localizedMessages.coachNG_unknown_value;			

		    var collaborationEvent = {"type": "dataChange", "domId": domId, "color": color, "bgColor": bgColor, "message": message, "userName": userName, "value": value};	
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, collaborationEvent);
		}
	}
	
	engine.handleCustomCollaborationEvent = function(event) {
		var viewScope = engine._findCollaboratingView(event);
		if(!!viewScope) {			
			engine.callViewLifeCycleAPI(viewScope, "collaboration", CONSTANTS.ELAPSED_TIME_THRESHOLD_COLLABORATION, event);
		}
	}
	
	// controller calls this internal API when it receives validation information of one boundary event
	engine.handleValidationEvent = function(rawEvent) {
		try {		
			var topLevelViews = engine.GlobalAssets.topLevelViewScope;
			// prepare validationHelper, make sure it's clean
			validationHelper.initialize();
			var hasValidationEvent = validationHelper.validateAllViews(rawEvent, topLevelViews);
			if(hasValidationEvent) {	
				engine.deliverValidationEvents();
				// broadcast that all validation events have been delivered.
				connect.publish("com.ibm.bpm.coach.validationCompleted", {});
			} else {
				validationHelper.clearTransientInfo();	
			}
		} catch(ex) {
			logger.log("engine", "engine.handleValidationEvent", "ex: " + ex);
		}
	}

	// controller calls this internal API right before it submit boundary event.
	// engine will ask all the views that have validation errors to clear themself
	engine.clearValidationEvent = function() {
		var treeNode = validationHelper.viewTree.rootNode.firstChild;
		if(!!treeNode) {
			engine._deliverValidationEvents(treeNode, validationHelper.viewDomIdToViewMap, true);

			// now we have delivered all the clear events, it's time for framework to clear all the memory of last validation cycle.
			validationHelper.initialize();

			// broadcast that all validation events (clear) have been delivered.
			connect.publish("com.ibm.bpm.coach.validationCleared", {});
		}
	}
	
	engine.deliverValidationEvents = function() {
		if(engine.isProcessingValidation) {
			// engine is already in the middle of delivering validation events. All the new validation events 
			// prepared by the caller will be handled by the existing delivery process. So need to do nothing for now.
			return;
		}
	
		engine.isProcessingValidation = true;
		
		// new validation events may occur as the result of processing of other validation event. The new event may
		// locate at the tree node which current round of delivery has gone through. So we have to go through
		// the entire tree multiple times to make sure no event is left. However, in order to avoid the possible
		// overflow, we only tranverse the entire tree up to 10 times.
		for(var k=0; k<10; k++) {
			var treeNode = validationHelper.viewTree.rootNode.firstChild;
			var foundValidationEvent = engine._deliverValidationEvents(treeNode, validationHelper.viewDomIdToViewMap);
			if(!foundValidationEvent) {
				break;
			}
		}
		// clear the transient data in validationHelper. Leave other data alone in validationHelper for future use.
		// we don't call clearTransientInfo() earlier (e.g. after validateAllViews() is executed and before deliverValidationEvents() is executed), 
		// the main reason is for performance - new validation event may occurs during the execution of deliverValidationEvents(), the calculation of new valdation event
		// need the help from the transient data in validationHelper. 
		validationHelper.clearTransientInfo();
		engine.isProcessingValidation = false;		
	}
	
	engine._deliverValidationEvents = function(treeNode, viewDomIdToViewMap, isToClear) {
		var foundValidationEvent = false;
		if(!treeNode) {
			return foundValidationEvent;
		}
		
		if(!!isToClear) {
			var clearEvent = {"type": CONSTANTS.VALIDATION_EVENT_TYPE_CLEAR};
			// send clear event to all the valid views who have received validation events
			if(!!treeNode.validationEvent) {
				var aView = engine._findViewInstance(treeNode, viewDomIdToViewMap);
				if(!!aView) {
					engine.callViewLifeCycleAPI(aView, "validate", CONSTANTS.ELAPSED_TIME_THRESHOLD_VALIDATE, clearEvent);
				}
				if(!foundValidationEvent) {
					foundValidationEvent = true;
				}
			}			
		} else {
			// deliver the validation events if exists
			if(!treeNode.isHandled) {
				// handle treeNode's validationEvent if exists
				if(!!treeNode.validationEvent) {
					var aView = engine._findViewInstance(treeNode, viewDomIdToViewMap);
					if(!!aView) {
						engine.callViewLifeCycleAPI(aView, "validate", CONSTANTS.ELAPSED_TIME_THRESHOLD_VALIDATE, treeNode.validationEvent);
					}
					if(!foundValidationEvent) {
						foundValidationEvent = true;
					}
				}		
				treeNode.isHandled = true;
			}
		}
		
		// handle treeNode's children
		if(!!treeNode.firstChild) {
			var childResult = engine._deliverValidationEvents(treeNode.firstChild, viewDomIdToViewMap, isToClear);
			if(!foundValidationEvent) {
				foundValidationEvent = childResult;
			}	
		}
		
		// hanlde treeNode's sibling
		if(!!treeNode.next) {
			var siblingResult = engine._deliverValidationEvents(treeNode.next, viewDomIdToViewMap, isToClear);
			if(!foundValidationEvent) {
				foundValidationEvent = siblingResult;
			}	
		}
		
		return foundValidationEvent;
	}
	
	engine._findViewInstance = function(treeNode, viewDomIdToViewMap) {
		var cachedView = viewDomIdToViewMap[treeNode.domId];
		if(!!cachedView) {
			if(cachedView._isReady()) {
				return cachedView;
			}			
		}
		
		var aView = null;

		if(!!treeNode.locationInfo) {
			// viewIdMetaData exists
			aView = coachViewHelper.findViewByLocation(treeNode.locationInfo, engine.GlobalAssets.topLevelViewScope);
		} else {
			aView = coachViewHelper.findViewScope(treeNode.domId, engine.GlobalAssets.topLevelViewScope);
		}	

		if(!!aView) {
			viewDomIdToViewMap[treeNode.viewDomId] = aView;
			return aView;
		}
		
		return null;
	}

	engine._findCollaboratingView = function(event) {
		if(!!event.viewIdMetaData) {
			// if viewIdMetaData exists, use it to figure out view's DOM id
			return coachViewHelper.findViewByLocation(event.viewIdMetaData, engine.GlobalAssets.topLevelViewScope);
		} else if(!!event.viewId) {
			// event.viewId means the dom id of the closest viewScope node
			return coachViewHelper.findViewScope(event.viewId, engine.GlobalAssets.topLevelViewScope);
		} else {
			return null;
		}	
	}
	
	engine._findCollaboratingDomId = function(collaboratingView, event) {
		var node = dom.byId(event.domId);
		if(!!node && dom.isDescendant(node, collaboratingView.context.element)) {
			return event.domId;
		} else {
			// fallback to dom id of collaboratingView
			return collaboratingView.context.element.id;
		}
	}
	
	engine.getViewIdMetaData = function(viewDomId) {
		var viewScope = coachViewHelper.findViewScope(viewDomId, engine.GlobalAssets.topLevelViewScope);
		return coachViewHelper.getViewIdMetaData(viewScope);
	}
	
	engine.callAjaxService = function(ajaxUrl, args) {
		var paramsKey = "&params="
		var params = null;
		if (!!args) {
			if (!!(args.params)) {
				if (lang.isString(args.params)) {
					params = encodeURIComponent(args.params);					
				} else {
					params = encodeURIComponent(json1.stringify(args.params));
				}
			}
		}
		var tempStr = (!!params)? (ajaxUrl+paramsKey+params) : ajaxUrl;
		var tempArray = tempStr.split("?");
		
		var xhrArgs = {
			url: tempArray[0],
			handleAs: "json",
		    headers: {"Content-Type": "application/x-www-form-urlencoded",
		    	"Accept": "application/json"},
			sync: false,
			postData: tempArray[1],
			load: function(data){
				if ((!!args) && (!!(args.load)) && (lang.isFunction(args.load))) {
					args.load(data.data.data);
				}
	        },
	        error: function(err) {
	        	if ((!!args) && (!!(args.error)) && (lang.isFunction(args.error))) {
	        		args.error(err);
	        	} else {
			    	var errorMsg = "engine.callAjaxService() caught Error when handling " + ajaxUrl + ", Error: " + err;
			    	commonHelper.consoleLog(errorMsg, commonHelper.ERROR);
	        	}
	        }
		};
		xhr.post(xhrArgs);
	}
	
	
	engine.setReadonly = function(readonly) {
		if(readonly) {
			// if engine.coachShimmer exists, that means already in readonly mode. So current readonly mode request should be ingored
			if(!engine.coachShimmer) {
				engine.coachShimmer = new Shimmer(engine.CONSTANTS.COACH_SHIMMER_ID_BASE);
			}
		} else {
			// if engine.coachShimmer doesn't exist, that means already in edit mode. So current edit mode request should be ignored
			if(!!engine.coachShimmer) {
				engine.coachShimmer.dispose();
                engine.coachShimmer = null;
			}
		}
	}

					  
	engine.completeInitialization = function() {
		logger.log("engine", "completeInitialization", "is called", logger.INFO);
		engine.addMessageListeners();
		if ((!!(window.parent)) && (window != window.parent)) {
			var event = {
				"name": "onInitialized",
				"taskID" : engine.GlobalAssets.systemData.zTaskId,
				"applicationInstanceId" : engine.GlobalAssets.systemData.applicationInstanceId,
				"parameters": [{"name": "URL", "value": window.document.location.href}]
			};
			// work item 131966
			var serializedEvent = json.toJson(event);
			try {
				window.parent.postMessage(serializedEvent, "*");
			} catch (e) {
				// postMessage() failed, use direct call onMessage instead with faked event. Container MUST be co-located
				if ((!!(window.parent.onMessage)) && typeof window.parent.onMessage === 'function') {
					logger.log("engine", "completeInitialization", "iframe postMessage() failed, call onMessage() directly - event=" + serializedEvent, logger.FINE);
					window.parent.onMessage({origin: window.location.protocol + "//" + window.location.host,
											data: serializedEvent, 
											source: window});
				} else {
					logger.log("engine", "completeInitialization", "iframe postMessage() failed, no onMessage() to call on parent - event=" + serializedEvent, logger.FINE);
				}
			}			
		} 
		
		commonHelper.timestamp("Exit engine.completeInitialization()");
	}
//support for portlet enablement
	engine.addMessageListeners = function() {
		logger.log("engine", "addMessageListeners", "is called", logger.FINER);
		if(typeof window.addEventListener != 'undefined') {
			window.addEventListener("message", engine.receivePostMessage, false);
		} else if(typeof window.attachEvent != 'undefined') {
			window.attachEvent("onmessage", engine.receivePostMessage);
		}	
	}
	
	engine.receivePostMessage = function(event) {
		logger.log("engine", "receivePostMessage", ["is called with event:", event], logger.FINER);
		
		var eventObj = json.fromJson(event.data);
		if (eventObj.name == 'broadcastBoundaryEvent') {
			//initializing external messaging for boundary events
			engine.postMessageSource = event.source;
			engine.postMessageDomain = event.origin;
		} else if (!!engine.postMessageSource) {
			connect.publish("com.ibm.bpm.coach.externalEventReceived", [eventObj]);
		}
	}
	
	engine.postMessage = function(event) {
		if (!!engine.postMessageSource) {
			// work item 131966
			var serializedEvent = json.toJson(event);
			try {
				engine.postMessageSource.postMessage(serializedEvent, engine.postMessageDomain);
			} catch (e) {
				// postMessage() failed, use direct call onMessage instead with faked event. Container MUST be co-located
				if ((!!(engine.postMessageSource.onMessage)) && typeof engine.postMessageSource.onMessage === 'function') {
					logger.log("engine", "postMessage", "iframe postMessage() failed, call onMessage() directly - event=" + serializedEvent, logger.FINE);
					engine.postMessageSource.onMessage({origin: window.location.protocol + "//" + window.location.host,
											data: serializedEvent, 
											source: window});
				} else {
					logger.log("engine", "postMessage", "iframe postMessage() failed, no onMessage() to call on parent - event=" + serializedEvent, logger.FINE);
				}
			}			
		}
	}

	engine.onWindowClose = function() {
		engine.onCoachUnload.call(engine.GlobalAssets);
	}

	engine.onCoachUnload = function() {
		logger.log("engine", "onCoachUnload", "is called", logger.INFO);
		if (!(engine._isCoachUnloaded)) {
			var topLevelViewIds = new Array();
			for (var viewId in this.topLevelViewScope) {
				topLevelViewIds.push(viewId);
				var viewScope = this.topLevelViewScope[viewId];
				engine.callUnload(viewScope);
			}
			for (var k=0; k<topLevelViewIds.length; k++) {
				delete this.topLevelViewScope[topLevelViewIds[k]];
			}
			
			connect.publish("com.ibm.bpm.coach.coachUnload", []);
			engine._isCoachUnloaded = true;
		}
	}

	engine.callUnload = function(viewScope) {
		if(!viewScope) return;
		
	    // Call the view unload lifecycle method
		viewScope._preUnload();

		engine.callViewLifeCycleAPI(viewScope, "unload", CONSTANTS.ELAPSED_TIME_THRESHOLD_UNLOAD);

		// Then call nested view unload
		if (!!viewScope.context.subview) {
			for (var viewId in viewScope.context.subview) {

				var subviews = viewScope.context.subview[viewId];
				for(var divId in subviews) {
					var subViewScope = subviews[divId];
					engine.callUnload(subViewScope);
				}
			}		
		}
		
	    // Finally call _postUnload() to dispose viewScope (make it ready for garbage collection)
		// viewScope.context.subview is set to null in viewScope.postUnload(), so have to get viewScope.context.subview before calling viewScope.postUnload()
		viewScope._postUnload();
	}	
	
	return engine;
});
